import { GlobalService } from 'totvs-log-mobile-foundation';
import { BaseMessage } from 'totvs-log-base-foundation';
import { Injectable, NgModule } from '@angular/core';
import { Camera } from '@ionic-native/camera/ngx';
import { Platform, Events } from '@ionic/angular';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe que contém os eventos que o utilitário assina e publica
 */
class CameraUtilEvents {
}
// FIXME - Nomes de eventos mais específicos, para evitar possíveis conflitos
/**
 * Evento publicado pelo utilitário quando uma foto é tirada e o arquivo é gerado
 * no cache do aplicativo
 */
CameraUtilEvents.CACHE_FILE_CREATED = 'CacheFile:Created';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// TODO - Verificar se é válido ter classes abstratas para os utilitários
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de camera
 * com métodos que facilitam a solicitação de permissão e apresentação da
 * camera
 */
class CameraUtilProvider {
    /**
     * @ignore
     * Método responsável por construir o serviço
     * platform Serviço relacionado a plataforma
     * camera Serviço do plugin nativo de camera
     * global Serviço global da aplicação
     * @param {?} platform
     * @param {?} camera
     * @param {?} global
     * @param {?} events
     */
    constructor(platform, camera, global, events) {
        this.camera = camera;
        this.global = global;
        this.events = events;
        /**
         * @ignore
         * Indica se a plataforma é cordova
         */
        this.isCordova = false;
        platform.ready().then((/**
         * @return {?}
         */
        () => {
            this.isCordova = platform.is('cordova');
            if (this.isCordova) {
                this.defaultOptions = {
                    quality: 75,
                    destinationType: this.camera.DestinationType.FILE_URI,
                    encodingType: this.camera.EncodingType.JPEG,
                    mediaType: this.camera.MediaType.PICTURE,
                    targetWidth: 1920,
                    targetHeight: 1080,
                    correctOrientation: true
                };
            }
        }));
    }
    /**
     * Método responsável por apresentar a camera para que o usuário
     * possa tirar uma foto
     * Este método publica o evento CameraUtilEvents.CACHE_FILE_CREATED quando um arquivo é criado em
     * cache (Mais informações sobre eventos {\@link https://ionicframework.com/docs/api/util/Events/|Ionic Events})
     * {CameraOptions} options Opções da camera
     * @param {?=} options
     * @return {?} Promessa com o FILE_URI (opção padrão) ou o que for definido na opção 'destinationType' que pode ser passada como parâmetro
     */
    takePicture(options) {
        return new Promise((/**
         * @param {?} resolve
         * @param {?} reject
         * @return {?}
         */
        (resolve, reject) => {
            if (this.isCordova === false) {
                this.global.msg.showMessage(new BaseMessage({
                    literal: 'recursoNaoEstaDisponivelMsg'
                }));
                reject();
            }
            /** @type {?} */
            const cameraOptions = Object.assign({}, this.defaultOptions);
            if (options) {
                for (const prop of Object.keys(options)) {
                    cameraOptions[prop] = options[prop];
                }
            }
            this.camera.getPicture(cameraOptions)
                .then((/**
             * @param {?} imageData
             * @return {?}
             */
            (imageData) => {
                this.events.publish(CameraUtilEvents.CACHE_FILE_CREATED, imageData);
                resolve(imageData);
            }), (/**
             * @param {?} error
             * @return {?}
             */
            (error) => reject(error))).catch((/**
             * @param {?} error
             * @return {?}
             */
            (error) => {
                /** @type {?} */
                let literal;
                if (error && error === 20) {
                    literal = 'permissaoNaoConcedidaParaORecursoMsg';
                }
                else {
                    literal = 'recursoNaoEstaDisponivelMsg';
                }
                this.global.msg.showMessage(new BaseMessage({
                    literal
                }));
                reject(error);
            }));
        }));
    }
    /**
     * Método responsável por limpar as imagens em cache geradas durante
     * a utilização do plugin enquanto estiver ativo
     * @return {?}
     */
    cleanup() {
        if (this.isCordova) {
            this.camera.cleanup()
                .then((/**
             * @return {?}
             */
            () => { }))
                .catch((/**
             * @return {?}
             */
            () => { }));
        }
    }
}
CameraUtilProvider.decorators = [
    { type: Injectable }
];
/** @nocollapse */
CameraUtilProvider.ctorParameters = () => [
    { type: Platform },
    { type: Camera },
    { type: GlobalService },
    { type: Events }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Módulo que contém um provedor utilitário que auxilia na
 * utilização do plugin nativo de camera
 */
class CameraUtilModule {
}
CameraUtilModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    Camera,
                    CameraUtilProvider
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { CameraUtilModule, CameraUtilEvents, CameraUtilProvider };

//# sourceMappingURL=totvs-log-mobile-foundation-src-camera-util.js.map