import { __awaiter } from 'tslib';
import { CriticalError } from 'totvs-log-base-foundation';
import { Platform } from '@ionic/angular';
import { Injectable, NgModule } from '@angular/core';
import { File } from '@ionic-native/file/ngx';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// TODO - Verificar se é válido ter classes abstratas para os utilitários
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de arquivo
 * com métodos que facilitam a manipulação de arquivos no aplicativo
 */
class FileUtilProvider {
    /**
     * @ignore
     * Método construtor responsável por inicializar o serviço
     * platform Serviço relacionado a plataforma
     * file Serviço do plugin nativo de arquivo
     * events Serviço de eventos do Ionic
     * @param {?} platform
     * @param {?} file
     */
    constructor(platform, file) {
        this.file = file;
        /**
         * @ignore
         * Indica se a plataforma é cordova
         */
        this.isCordova = false;
        platform.ready().then((/**
         * @return {?}
         */
        () => this.isCordova = platform.is('cordova')));
    }
    /**
     * Método responsável por remover um arquivo
     * uri URI do arquivo que deve ser removido
     * Objeto que representa o resultado da remoção do arquivo
     * @param {?} uri
     * @return {?}
     */
    removeFile(uri) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isCordova === false) {
                throw new CriticalError('Platform is not cordova (removeFile)');
            }
            if (!uri || uri === '') {
                throw new CriticalError('URI invalid (removeFile)');
            }
            /** @type {?} */
            const fileName = this.getFilenameFromUri(uri);
            /** @type {?} */
            const path = this.getPathFromUri(uri);
            /** @type {?} */
            let removeResult;
            try {
                removeResult = yield this.file.removeFile(path, fileName);
            }
            catch (error) {
                throw new CriticalError('Error in remove file', error);
            }
            return removeResult;
        });
    }
    /**
     * Método responsável por transferir os arquivos para o storage do aplicativo
     * uris URIs dos arquivos que devem ser copiados
     * Lista de URIs dos arquivos copiados para o storage do aplicativo
     * @param {...?} uris
     * @return {?}
     */
    fileCacheToAppStorage(...uris) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isCordova === false) {
                throw new CriticalError('Platform is not cordova');
            }
            /** @type {?} */
            const promisesCopy = [];
            uris.forEach((/**
             * @param {?} uri
             * @return {?}
             */
            (uri) => {
                /** @type {?} */
                const fileName = decodeURI(this.getFilenameFromUri(uri));
                /** @type {?} */
                const path = this.getPathFromUri(uri);
                promisesCopy.push(this.file.copyFile(path, fileName, this.file.dataDirectory, fileName)
                    .then((/**
                 * @param {?} entry
                 * @return {?}
                 */
                (entry) => {
                    return {
                        sourcePath: path,
                        sourceFileName: fileName,
                        targetFile: entry
                    };
                })));
            }));
            /** @type {?} */
            let filesCopied;
            try {
                filesCopied = yield Promise.all(promisesCopy);
            }
            catch (error) {
                throw new CriticalError(`Error in copy file`, error);
            }
            /** @type {?} */
            const promisesRemove = [];
            /** @type {?} */
            const filesUrl = [];
            filesCopied.forEach((/**
             * @param {?} fileCopied
             * @return {?}
             */
            (fileCopied) => {
                filesUrl.push(fileCopied.targetFile.toURL());
                promisesRemove.push(this.file.removeFile(fileCopied.sourcePath, fileCopied.sourceFileName)
                    .then((/**
                 * @param {?} result
                 * @return {?}
                 */
                (result) => {
                    return result;
                })));
            }));
            try {
                yield Promise.all(promisesRemove);
            }
            catch (error) {
                throw new CriticalError(`Error in remove file`, error);
            }
            return filesUrl;
        });
    }
    /**
     * @ignore
     * Método responsável por recuperar o nome do arquivo de um URI
     * uri URI do arquivo
     * @private
     * @param {?} uri
     * @return {?}
     */
    getFilenameFromUri(uri) {
        /** @type {?} */
        let fileName;
        try {
            /** @type {?} */
            const fileNameArray = uri.split('/');
            fileName = fileNameArray[fileNameArray.length - 1];
        }
        catch (error) {
            throw new CriticalError('Error (getFilenameFromUri)', error);
        }
        return fileName;
    }
    /**
     * @ignore
     * Método responsável por recuperar o path do arquivo de um URI
     * uri URI do arquivo
     * @private
     * @param {?} uri
     * @return {?}
     */
    getPathFromUri(uri) {
        /** @type {?} */
        let filePath;
        try {
            /** @type {?} */
            const fileNameArray = uri.split('/');
            fileNameArray.splice(fileNameArray.length - 1, 1);
            filePath = fileNameArray.join('/');
        }
        catch (error) {
            throw new CriticalError('Error (getPathFromUri)', error);
        }
        return filePath;
    }
}
FileUtilProvider.decorators = [
    { type: Injectable }
];
/** @nocollapse */
FileUtilProvider.ctorParameters = () => [
    { type: Platform },
    { type: File }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Módulo que contém um provedor utilitário que auxilia na
 * utilização do plugin nativo de manipulação de arquivo
 */
class FileUtilModule {
}
FileUtilModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    File,
                    FileUtilProvider
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { FileUtilModule, FileUtilProvider };

//# sourceMappingURL=totvs-log-mobile-foundation-src-file-util.js.map