import { __awaiter } from 'tslib';
import { CriticalError, AuthBaseService } from 'totvs-log-base-foundation';
import { Platform } from '@ionic/angular';
import { Injectable, NgModule } from '@angular/core';
import { HTTP } from '@ionic-native/http/ngx';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// TODO - Verificar se é válido ter classes abstratas para os utilitários
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de requisição http
 * nativa do dispositivo com métodos que facilitam a utilização do plugin
 */
class HttpOauth2UtilProvider {
    /**
     * @ignore
     * Método construtor responsável por inicializar o serviço
     * platform Serviço relacionado a plataforma
     * file Serviço do plugin nativo de arquivo
     * events Serviço de eventos do Ionic
     * @param {?} platform
     * @param {?} auth
     * @param {?} http
     */
    constructor(platform, auth, http) {
        this.auth = auth;
        this.http = http;
        /**
         * @ignore
         * Indica se a plataforma é cordova
         */
        this.isCordova = false;
        platform.ready().then((/**
         * @return {?}
         */
        () => this.isCordova = platform.is('cordova')));
    }
    /**
     * Método responsável por realizar o upload de um arquivo
     * url URL do Endpoint de Upload
     * filePath URI do arquivo no dispositivo
     * name Nome do arquivo
     * body Informações no corpo da mensagem
     * headers Informações de cabeçalho da mensagem
     * {IHttpUtilResult} Resultado da requisição
     * @template T
     * @param {?} url
     * @param {?} filePath
     * @param {?=} name
     * @param {?=} body
     * @param {?=} headers
     * @return {?}
     */
    uploadFile(url, filePath, name, body, headers) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isCordova === false) {
                throw new CriticalError('Platform is not cordova (uploadFile)');
            }
            url = url.replace('{tenant}', this.auth.shortNameTenant || '');
            // Injeção da autorização Oauth2
            /** @type {?} */
            const user = yield this.auth.getUser();
            /** @type {?} */
            let postHeaders;
            if (headers) {
                postHeaders = Object.assign({}, headers, {
                    Authorization: user ? `${user.token_type} ${user.access_token}` : ''
                });
            }
            else {
                postHeaders = {
                    Authorization: user ? `${user.token_type} ${user.access_token}` : ''
                };
            }
            /** @type {?} */
            let postBody;
            if (body) {
                postBody = body;
            }
            else {
                postBody = {};
            }
            if (name === undefined || name === '') {
                name = 'file';
            }
            /** @type {?} */
            const result = {
                status: 0,
                data: undefined,
                error: undefined,
                sourceResponse: undefined
            };
            try {
                /** @type {?} */
                const response = yield this.http.uploadFile(url, postBody, postHeaders, filePath, name);
                result.sourceResponse = response;
                if (response) {
                    if (response.hasOwnProperty('status')) {
                        result.status = response.status;
                    }
                    if (response.hasOwnProperty('data')) {
                        try {
                            result.data = (/** @type {?} */ (JSON.parse(response.data)));
                        }
                        catch (error) {
                            result.error = error;
                        }
                    }
                }
            }
            catch (error) {
                result.error = error;
            }
            return result;
        });
    }
    /**
     * Método que permite a alteração do modo de verificação do certificado SSL
     * mode Modo de verificação do certificado SSL
     * @param {?} mode
     * @return {?}
     */
    setSSLCertMode(mode) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.http.setSSLCertMode(mode);
        });
    }
}
HttpOauth2UtilProvider.decorators = [
    { type: Injectable }
];
/** @nocollapse */
HttpOauth2UtilProvider.ctorParameters = () => [
    { type: Platform },
    { type: AuthBaseService },
    { type: HTTP }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Módulo que contém um provedor utilitário que auxilia na
 * utilização do plugin nativo de requisição http
 */
class HttpOauth2UtilModule {
}
HttpOauth2UtilModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    HTTP,
                    HttpOauth2UtilProvider
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { HttpOauth2UtilModule, HttpOauth2UtilProvider };

//# sourceMappingURL=totvs-log-mobile-foundation-src-http-util.js.map