import { __awaiter } from 'tslib';
import { GlobalService } from 'totvs-log-mobile-foundation';
import { BaseMessage, CriticalError } from 'totvs-log-base-foundation';
import { Platform, Events } from '@ionic/angular';
import { ImagePicker, OutputType } from '@ionic-native/image-picker/ngx';
import { Injectable, NgModule } from '@angular/core';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe que contém os eventos que o utilitário assina e publica
 */
class ImagePickerUtilEvents {
}
// FIXME - Nomes de eventos mais específicos, para evitar possíveis conflitos
/**
 * Evento publicado pelo utilitário quando uma ou mais imagens são
 * selecionadas e geram uma cópia no cache do aplicativo
 */
ImagePickerUtilEvents.CACHE_FILE_CREATED = 'CacheFile:Created';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// TODO - Verificar se é válido ter classes abstratas para os utilitários
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de seleção
 * de imagens do dispositivo com métodos que facilitam a solicitação de permissão,
 * apresentação e seleção de imagens
 */
class ImagePickerUtilProvider {
    /**
     * @ignore
     * Método responsável por construir o serviço
     * platform Serviço relacionado a plataforma
     * imagePicker Serviço do plugin nativo de galeria
     * global Serviço global da aplicação
     * @param {?} platform
     * @param {?} imagePicker
     * @param {?} global
     * @param {?} events
     */
    constructor(platform, imagePicker, global, events) {
        this.imagePicker = imagePicker;
        this.global = global;
        this.events = events;
        /**
         * @ignore
         * Indica se a plataforma é cordova
         */
        this.isCordova = false;
        /**
         * @ignore
         * Opções padrões para a galeria
         */
        this.defaultOptions = {
            outputType: OutputType.FILE_URL,
            quality: 60
        };
        platform.ready().then((/**
         * @return {?}
         */
        () => this.isCordova = platform.is('cordova')));
    }
    /**
     * Método responsável por abrir o seletor de imagens e retornar o caminho dos arquivos salvos no cache
     * Este método publica o evento ImagePickerUtilEvents.CACHE_FILE_CREATED quando um ou mais arquivos são selecionados
     * e criados em cache (Mais informações sobre eventos {\@link https://ionicframework.com/docs/api/util/Events/|Ionic Events})
     * opt Customização das opções enviadas ao seletor de images
     * Lista de imagens escolhidas
     * @param {?=} opt
     * @return {?}
     */
    chooseImage(opt) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isCordova === false) {
                /** @type {?} */
                const msg = new BaseMessage({
                    literal: 'recursoNaoEstaDisponivelMsg'
                });
                this.global.msg.showMessage(msg);
                throw new CriticalError('Platform is not cordova');
            }
            /** @type {?} */
            let images = [];
            try {
                /** @type {?} */
                const options = Object.assign({}, this.defaultOptions);
                if (opt) {
                    for (const prop of Object.keys(opt)) {
                        options[prop] = opt[prop];
                    }
                }
                /** @type {?} */
                const hasPermission = yield this.requestReadPermission();
                if (hasPermission) {
                    images = yield this.imagePicker.getPictures(options);
                    this.events.publish(ImagePickerUtilEvents.CACHE_FILE_CREATED, ...images);
                }
                else {
                    /** @type {?} */
                    const msg = new BaseMessage({
                        literal: 'permissaoNaoConcedidaParaORecursoMsg'
                    });
                    this.global.msg.showMessage(msg);
                    throw new CriticalError('Permission denied');
                }
            }
            catch (error) {
                throw new CriticalError('Error in get pictures', error);
            }
            return images;
        });
    }
    /**
     * Método responsável por requisitar permissão para acesso a galeria
     * Indicação se possui ou não permissão
     * @return {?}
     */
    requestReadPermission() {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            let hasPermission = false;
            try {
                /** @type {?} */
                const permission = yield this.imagePicker.hasReadPermission();
                if (!permission) {
                    yield this.imagePicker.requestReadPermission();
                    /** @type {?} */
                    const permissionAfterRequest = yield this.imagePicker.hasReadPermission();
                    hasPermission = permissionAfterRequest;
                }
                else {
                    hasPermission = true;
                }
            }
            catch (error) {
                hasPermission = false;
            }
            return hasPermission;
        });
    }
}
ImagePickerUtilProvider.decorators = [
    { type: Injectable }
];
/** @nocollapse */
ImagePickerUtilProvider.ctorParameters = () => [
    { type: Platform },
    { type: ImagePicker },
    { type: GlobalService },
    { type: Events }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Módulo que contém um provedor utilitário que auxilia na
 * utilização do plugin nativo de galeria
 */
class ImagePickerUtilModule {
}
ImagePickerUtilModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    ImagePicker,
                    ImagePickerUtilProvider
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { ImagePickerUtilEvents, ImagePickerUtilModule, ImagePickerUtilProvider };

//# sourceMappingURL=totvs-log-mobile-foundation-src-image-picker-util.js.map