import { Subject } from 'rxjs';
import { take } from 'rxjs/operators';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class OidcCordovaIabEvents {
}
OidcCordovaIabEvents.LoadSuccessful = 'OidcCordovaIabEventsLoadSuccessful';
OidcCordovaIabEvents.LoadFailed = 'OidcCordovaIabEventsLoadFailed';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe responsável pelo Popup utilizado no processo
 * de autenticação com Oidc no Ionic
 */
class OidcCordovaPopupWindow {
    /**
     * Método construtor que inicializa a classe
     * iab Serviço do plugin nativo ionic InAppBrowser
     * popupParams Parâmetros para o popup
     * @param {?} iab
     * @param {?} popupParams
     */
    constructor(iab, popupParams) {
        this.iab = iab;
        this.popupParams = popupParams;
        this.navigated = new Subject();
        this.redirectUri = this.popupParams.startUrl;
    }
    /**
     * Método acessor do promise retornado pelo método Navigate
     * @return {?}
     */
    get promise() {
        return this.internalPromise;
    }
    /**
     * Método responsável por realizar a navegação
     * navParams Parâmetros de navegação
     * @param {?} navParams
     * @return {?}
     */
    navigate(navParams) {
        this.internalPromise = new Promise((/**
         * @param {?} resolve
         * @param {?} reject
         * @return {?}
         */
        (resolve, reject) => {
            this.internalReject = reject;
            /** @type {?} */
            let errorMsg = '';
            if (!navParams || !navParams.url) {
                errorMsg = 'No url provided';
            }
            else {
                /** @type {?} */
                const features = this.popupParams.popupWindowFeatures || OidcCordovaPopupWindow.DefaultPopupFeatures;
                /** @type {?} */
                const target = this.popupParams.popupWindowTarget || OidcCordovaPopupWindow.DefaultPopupTarget;
                this.popup = this.iab.create(navParams.url, target, features);
                if (this.popup) {
                    // Evento 'exit'
                    this.popupExitSub = this.popup.on('exit').subscribe((/**
                     * @param {?} event
                     * @return {?}
                     */
                    (event) => {
                        this.closePopup();
                        reject(new Error(event.message));
                    }));
                    // Evento 'loadstart'
                    this.popupLoadStartSub = this.popup.on('loadstart').subscribe((/**
                     * @param {?} event
                     * @return {?}
                     */
                    (event) => {
                        /** @type {?} */
                        const url = event.url;
                        if (url.indexOf(this.redirectUri) === 0) {
                            this.closePopup();
                            resolve({ url });
                        }
                        else if (url.indexOf('error') !== -1) {
                            this.closePopup();
                            reject(new Error(`Load start error. URL: ${url}`));
                        }
                    }));
                }
                else {
                    errorMsg = 'Error opening popup window';
                }
            }
            if (errorMsg !== '') {
                this.closePopup();
                reject(new Error('Error opening popup window'));
            }
        }));
        this.navigated.next(this.promise);
        return this.promise;
    }
    /**
     * Método responsável por fechar
     * @return {?}
     */
    close() {
        this.closePopup();
        this.internalReject();
    }
    /**
     * Método responsável por fechar o popup
     * @private
     * @return {?}
     */
    closePopup() {
        if (this.popup) {
            this.popupExitSub.unsubscribe();
            this.popupLoadStartSub.unsubscribe();
            this.popup.close();
            this.popup = undefined;
        }
    }
}
/**
 * Propriedades padrão do Popup
 */
OidcCordovaPopupWindow.DefaultPopupFeatures = 'location=no,toolbar=no,zoom=no';
/**
 * Destino padrão do Popup
 */
OidcCordovaPopupWindow.DefaultPopupTarget = '_blank';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe responsável pela navegação com popup utilizado no processo
 * de autenticação com Oidc no Ionic
 */
class OidcCordovaPopupNavigator {
    /**
     * Método construtor que inicializa a classe
     * iab Serviço do plugin nativo InAppBrowser
     * @param {?} iab
     * @param {?} events
     */
    constructor(iab, events) {
        this.iab = iab;
        this.events = events;
    }
    /**
     * Método que prepara a janela de popup
     * params Parâmetros para a janela de popup
     * @param {?} params
     * @return {?}
     */
    prepare(params) {
        /** @type {?} */
        const popup = new OidcCordovaPopupWindow(this.iab, params);
        popup.navigated.pipe(take(1)).subscribe((/**
         * @param {?} promise
         * @return {?}
         */
        promise => {
            promise
                .then((/**
             * @param {?} value
             * @return {?}
             */
            value => {
                this.events.publish(OidcCordovaIabEvents.LoadSuccessful);
                return value;
            }))
                .catch((/**
             * @param {?} err
             * @return {?}
             */
            err => {
                this.events.publish(OidcCordovaIabEvents.LoadFailed);
                throw err;
            }));
        }));
        return Promise.resolve(popup);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe responsável pelo Iframe utilizado no processo
 * de autenticação com Oidc no Ionic
 */
class OidcCordovaIFrameNavigator {
    /**
     * Método construtor que inicializa a classe
     * iab Serviço do plugin nativo InAppBrowser
     * @param {?} iab
     */
    constructor(iab) {
        this.iab = iab;
    }
    /**
     * Método que prepara a janela de popup
     * params Parâmetros para a janela de popup
     * @param {?} params
     * @return {?}
     */
    prepare(params) {
        params.popupWindowFeatures = 'hidden=yes';
        params.isIframe = true;
        /** @type {?} */
        const popup = new OidcCordovaPopupWindow(this.iab, params);
        return Promise.resolve(popup);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { OidcCordovaIabEvents, OidcCordovaPopupWindow, OidcCordovaPopupNavigator, OidcCordovaIFrameNavigator };

//# sourceMappingURL=totvs-log-mobile-foundation-src-oidc-cordova.js.map