import { GlobalService } from 'totvs-log-mobile-foundation';
import { QRScanner } from '@ionic-native/qr-scanner/ngx';
import { BaseMessage } from 'totvs-log-base-foundation';
import { Subject } from 'rxjs';
import { Injectable, NgModule } from '@angular/core';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// TODO - Verificar se é válido ter classes abstratas para os utilitários
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de leitura de QR Code
 * com métodos que facilitam a solicitação de permissão e leitura do código
 */
class QrScannerUtilProvider {
    /**
     * @ignore
     * Método responsável por construir o serviço
     * qrScanner Serviço do plugin nativo de leitura do QR-Code
     * global Serviço global da aplicação
     * @param {?} qrScanner
     * @param {?} global
     */
    constructor(qrScanner, global) {
        this.qrScanner = qrScanner;
        this.global = global;
        /**
         * Propriedade que possui informações sobre o QR Code Scanner após ele estar
         * preparado, indicando assim se é possível ligar o flash ou trocar de camera,
         * por exemplo.
         */
        this.qrScannerLastStatus = new Subject();
    }
    /**
     * Método responsável por preparar o plugin para leitura do QR-Code
     * Indicação da preparação
     * @return {?}
     */
    prepare() {
        return new Promise((/**
         * @param {?} resolve
         * @return {?}
         */
        (resolve) => {
            this.qrScanner.prepare()
                .then((/**
             * @param {?} status
             * @return {?}
             */
            (status) => {
                this.qrScannerLastStatus.next(status);
                if (status.authorized) {
                    resolve(true);
                }
                else {
                    this.global.msg.showMessage(new BaseMessage({
                        literal: 'permissaoNaoConcedidaParaORecursoMsg'
                    }));
                    resolve(false);
                }
            }))
                .catch((/**
             * @param {?} error
             * @return {?}
             */
            (error) => {
                if (error && error.hasOwnProperty('name') && error.name === 'CAMERA_ACCESS_DENIED') {
                    this.global.msg.showMessage(new BaseMessage({
                        literal: 'permissaoNaoConcedidaParaORecursoMsg'
                    }));
                }
                else {
                    this.global.msg.showMessage(new BaseMessage({
                        literal: 'recursoNaoEstaDisponivelMsg'
                    }));
                }
                resolve(false);
            }));
        }));
    }
    /**
     * Método responsável por apresentar o leitor do QR-Code
     * String com o conteúdo lido
     * @return {?}
     */
    show() {
        return new Promise((/**
         * @param {?} resolve
         * @param {?} reject
         * @return {?}
         */
        (resolve, reject) => {
            if (!this.qrScannerLastStatus) {
                reject(undefined);
            }
            this.scanSub = this.qrScanner.scan().subscribe((/**
             * @param {?} result
             * @return {?}
             */
            (result) => {
                this.qrScanner.hide();
                this.scanSub.unsubscribe(); // stop scanning
                resolve(result);
            }), (/**
             * @param {?} error
             * @return {?}
             */
            (error) => {
                reject(error);
            }));
            this.qrScanner.show()
                .then((/**
             * @return {?}
             */
            () => { }))
                .catch((/**
             * @param {?} error
             * @return {?}
             */
            (error) => reject(error)));
        }));
    }
    /**
     * Método responsável por esconder a camera da leitura do QR-Code
     * @return {?}
     */
    hide() {
        this.qrScanner.getStatus().then((/**
         * @param {?} status
         * @return {?}
         */
        (status) => {
            if (status && status.showing) {
                this.qrScanner.hide().then((/**
                 * @param {?} statusAfter
                 * @return {?}
                 */
                (statusAfter) => {
                    this.qrScannerLastStatus.next(statusAfter);
                }));
            }
            if (status && status.scanning) {
                this.scanSub.unsubscribe();
            }
        }));
    }
    /**
     * Método responsável por ligar ou desligar o flash
     * @return {?}
     */
    flash() {
        this.qrScanner.getStatus().then((/**
         * @param {?} status
         * @return {?}
         */
        (status) => {
            if (status && status.scanning && status.canEnableLight) {
                if (status.lightEnabled) {
                    this.qrScanner.disableLight()
                        .then((/**
                     * @param {?} statusAfter
                     * @return {?}
                     */
                    (statusAfter) => {
                        this.qrScannerLastStatus.next(statusAfter);
                    }))
                        .catch((/**
                     * @return {?}
                     */
                    () => { }));
                }
                else {
                    this.qrScanner.enableLight()
                        .then((/**
                     * @param {?} statusAfter
                     * @return {?}
                     */
                    (statusAfter) => {
                        this.qrScannerLastStatus.next(statusAfter);
                    }))
                        .catch((/**
                     * @return {?}
                     */
                    () => { }));
                }
            }
        })).catch((/**
         * @return {?}
         */
        () => { }));
    }
    /**
     * Método responsável por alternar entre a camera frontal e a camera traseira
     * @return {?}
     */
    changeCamera() {
        this.qrScanner.getStatus().then((/**
         * @param {?} status
         * @return {?}
         */
        (status) => {
            if (status && status.scanning && status.canChangeCamera) {
                if (status.currentCamera === 0) { // Back camera
                    this.qrScanner.useFrontCamera()
                        .then((/**
                     * @param {?} statusAfter
                     * @return {?}
                     */
                    (statusAfter) => {
                        this.qrScannerLastStatus.next(statusAfter);
                    }))
                        .catch((/**
                     * @return {?}
                     */
                    () => { }));
                }
                else {
                    this.qrScanner.useBackCamera()
                        .then((/**
                     * @param {?} statusAfter
                     * @return {?}
                     */
                    (statusAfter) => {
                        this.qrScannerLastStatus.next(statusAfter);
                    }))
                        .catch((/**
                     * @return {?}
                     */
                    () => { }));
                }
            }
        })).catch((/**
         * @return {?}
         */
        () => { }));
    }
    /**
     * Método responsável por destruir o plugin nativo desabilitando a leitura do QR-Code até
     * que o plugin seja novamente preparado
     * @return {?}
     */
    destroy() {
        this.qrScanner.getStatus().then((/**
         * @param {?} status
         * @return {?}
         */
        (status) => {
            if (status) {
                if (status.lightEnabled) {
                    this.qrScanner.disableLight()
                        .then((/**
                     * @return {?}
                     */
                    () => { }))
                        .catch((/**
                     * @return {?}
                     */
                    () => { }));
                }
                this.qrScanner.destroy()
                    .then((/**
                 * @return {?}
                 */
                () => { }))
                    .catch((/**
                 * @return {?}
                 */
                () => { }));
            }
        }));
    }
}
QrScannerUtilProvider.decorators = [
    { type: Injectable }
];
/** @nocollapse */
QrScannerUtilProvider.ctorParameters = () => [
    { type: QRScanner },
    { type: GlobalService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Módulo que contém um provedor utilitário que auxilia na
 * utilização do plugin nativo de leitura de QR-Code
 */
class QRScannerUtilModule {
}
QRScannerUtilModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    QRScanner,
                    QrScannerUtilProvider
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { QRScannerUtilModule, QrScannerUtilProvider };

//# sourceMappingURL=totvs-log-mobile-foundation-src-qr-scanner-util.js.map