import { GlobalService } from 'totvs-log-mobile-foundation';
import { BaseMessage } from 'totvs-log-base-foundation';
import { Platform } from '@ionic/angular';
import { Injectable, NgModule } from '@angular/core';
import { SpeechRecognition } from '@ionic-native/speech-recognition/ngx';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// TODO - Verificar se é válido ter classes abstratas para os utilitários
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de reconhecimento de voz
 * com métodos que facilitam a solicitação de permissão e reconhecimento
 */
class SpeechRecognitionUtilProvider {
    /**
     * @ignore
     * Método responsável por construir o serviço
     * platform Serviço relacionado a plataforma
     * speechRecognition Serviço do plugin nativo de reconhecimento de voz
     * global Serviço global da aplicação
     * @param {?} platform
     * @param {?} speechRecognition
     * @param {?} global
     */
    constructor(platform, speechRecognition, global) {
        this.speechRecognition = speechRecognition;
        this.global = global;
        /**
         * @ignore
         * Indica se a plataforma é cordova
         */
        this.isCordova = false;
        platform.ready().then((/**
         * @return {?}
         */
        () => this.isCordova = platform.is('cordova')));
    }
    // FIXME - Em relação a implementação no IOS, provavelmente será necessário implementar o stopListening()
    /**
     * Método responsável por habilitar o reconheicimento de voz
     * Lista com os textos reconhecidos (geralmente o primeiro item é o mais provável)
     * @return {?}
     */
    listening() {
        return new Promise((/**
         * @param {?} resolve
         * @param {?} reject
         * @return {?}
         */
        (resolve, reject) => {
            this.verifyPermission().then((/**
             * @param {?} hasPermission
             * @return {?}
             */
            (hasPermission) => {
                if (hasPermission === false) {
                    this.showToast('permissaoNaoConcedidaParaORecursoMsg');
                    resolve([]);
                }
                else {
                    this.speechRecognition.isRecognitionAvailable()
                        .then((/**
                     * @param {?} available
                     * @return {?}
                     */
                    (available) => {
                        if (available === false) {
                            this.showToast('recursoNaoEstaDisponivelMsg');
                            resolve([]);
                            return;
                        }
                        this.speechRecognition.startListening(this.options)
                            .subscribe((/**
                         * @param {?} matches
                         * @return {?}
                         */
                        (matches) => resolve(matches)), (/**
                         * @param {?} onerror
                         * @return {?}
                         */
                        (onerror) => {
                            if (onerror && onerror instanceof Error && onerror.message !== '0') {
                                this.showToast('erroAoUtilizarRecursoMsg');
                            }
                            reject(onerror);
                        }));
                    })).catch((/**
                     * @param {?} error
                     * @return {?}
                     */
                    (error) => {
                        this.showToast('recursoNaoEstaDisponivelMsg');
                        reject(error);
                    }));
                }
            })).catch((/**
             * @param {?} error
             * @return {?}
             */
            (error) => {
                reject(error);
            }));
        }));
    }
    /**
     * @ignore
     * Método responsável por verificar se há permissão e se não houver, solicitar
     * Indicação se há ou não permissão
     * @private
     * @return {?}
     */
    verifyPermission() {
        return new Promise((/**
         * @param {?} resolve
         * @param {?} reject
         * @return {?}
         */
        (resolve, reject) => {
            if (this.isCordova === false) {
                this.showToast('recursoNaoEstaDisponivelMsg');
                reject('Cordova is not available.');
                return;
            }
            this.speechRecognition.hasPermission().then((/**
             * @param {?} hasPermission
             * @return {?}
             */
            (hasPermission) => {
                if (hasPermission) {
                    resolve(hasPermission);
                }
                else {
                    this.requestPermission()
                        .then((/**
                     * @param {?} requestPermissionSuccessful
                     * @return {?}
                     */
                    (requestPermissionSuccessful) => resolve(requestPermissionSuccessful)));
                }
            })).catch((/**
             * @return {?}
             */
            () => resolve(false)));
        }));
    }
    /**
     * @ignore
     * Método responsável por solicitar permissão de voz
     * Indicação se a permissão foi ou não concedida
     * @private
     * @return {?}
     */
    requestPermission() {
        return new Promise((/**
         * @param {?} resolve
         * @return {?}
         */
        (resolve) => {
            this.speechRecognition.requestPermission()
                .then((/**
             * @return {?}
             */
            () => resolve(true)), (/**
             * @return {?}
             */
            () => resolve(false))).catch((/**
             * @return {?}
             */
            () => resolve(false)));
        }));
    }
    /**
     * @ignore
     * Método para apresentar mensagens
     * literalMessage Literal da mensagem
     * @private
     * @param {?} literalMessage
     * @return {?}
     */
    showToast(literalMessage) {
        this.global.msg.showMessage(new BaseMessage({
            literal: literalMessage
        }));
    }
}
SpeechRecognitionUtilProvider.decorators = [
    { type: Injectable }
];
/** @nocollapse */
SpeechRecognitionUtilProvider.ctorParameters = () => [
    { type: Platform },
    { type: SpeechRecognition },
    { type: GlobalService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Módulo que contém um provedor utilitário que auxilia na
 * utilização do plugin nativo de reconhecimento de voz
 */
class SpeechRecognitionUtilModule {
}
SpeechRecognitionUtilModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    SpeechRecognition,
                    SpeechRecognitionUtilProvider
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { SpeechRecognitionUtilModule, SpeechRecognitionUtilProvider };

//# sourceMappingURL=totvs-log-mobile-foundation-src-speech-recognition-util.js.map