import { __awaiter } from 'tslib';
import { UtilsFn } from 'totvs-log-base-foundation';
import { Platform } from '@ionic/angular';
import { SQLite } from '@ionic-native/sqlite/ngx';
import { Injectable, NgModule } from '@angular/core';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Gerenciador de conexão de banco de dados sqlite
 * @abstract
 */
class SqLiteUtilBaseProvider {
    /**
     * @param {?} sqlLite
     * @param {?} platform
     */
    constructor(sqlLite, platform) {
        this.sqlLite = sqlLite;
        this.platform = platform;
    }
    /**
     * Método para auxiliar a criar uma conexão com o banco sqlite
     * name Nome do banco
     * location Localização do banco
     * DbConnectionBaseProvider
     * @protected
     * @param {?} name
     * @param {?} location
     * @return {?}
     */
    createConn(name, location) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.platform.ready();
            return this.sqlLite.create({ name, location });
        });
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Gerenciador de conexão de banco de dados sqlite
 */
class SqLiteUtilProvider extends SqLiteUtilBaseProvider {
    /**
     * @param {?} sqlLite
     * @param {?} platform
     */
    constructor(sqlLite, platform) {
        super(sqlLite, platform);
        this.connections = new Array();
        this.creating = new Array();
    }
    /**
     * Recupera uma conexão de banco de dados
     *
     * name o banco de dados desejado
     * {string} [location='default'] Configuração adicional de localização no dispositivo
     * @param {?} name
     * @param {?=} location
     * @return {?} Um objeto de conexão com o banco
     */
    getSqLiteConn(name, location = 'default') {
        return __awaiter(this, void 0, void 0, function* () {
            yield UtilsFn.lock((/**
             * @return {?}
             */
            () => this.creating.find((/**
             * @param {?} x
             * @return {?}
             */
            (x) => x.name === name && x.location === location))));
            /** @type {?} */
            const element = this.connections.find((/**
             * @param {?} x
             * @return {?}
             */
            (x) => x.name === name && x.location === location));
            /** @type {?} */
            let sqlConn;
            if (!element) {
                this.creating.push({ name, location, conn: undefined });
                sqlConn = yield this.createConn(name, location);
                this.connections.push({ name, location, conn: sqlConn });
                /** @type {?} */
                const idx = this.creating.findIndex((/**
                 * @param {?} x
                 * @return {?}
                 */
                (x) => x.name === name && x.location === location));
                if (idx !== -1) {
                    this.creating.splice(idx, 1);
                }
            }
            else {
                sqlConn = element.conn;
            }
            return sqlConn;
        });
    }
}
SqLiteUtilProvider.decorators = [
    { type: Injectable }
];
/** @nocollapse */
SqLiteUtilProvider.ctorParameters = () => [
    { type: SQLite },
    { type: Platform }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Módulo que controla as conexões com banco sqlite
 * e provê conexão com o serviço SqLite
 * Usar SqLiteUtilModule.forRoot() ao importar
 * @export
 * SqLiteUtilModule
 */
class SqLiteUtilModule {
    /**
     * @return {?}
     */
    static forRoot() {
        return {
            ngModule: SqLiteUtilModule,
            providers: [
                SQLite,
                {
                    provide: SqLiteUtilBaseProvider,
                    useClass: SqLiteUtilProvider
                }
            ]
        };
    }
}
SqLiteUtilModule.decorators = [
    { type: NgModule, args: [{
                providers: []
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { SqLiteUtilBaseProvider, SqLiteUtilProvider, SqLiteUtilModule };

//# sourceMappingURL=totvs-log-mobile-foundation-src-sqlite-util.js.map