import { __awaiter } from 'tslib';
import { UtilsFn, SqlUtilsFn } from 'totvs-log-base-foundation';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Utilitário com funções para sincronização de dados em banco sqlite
 */
class SyncSqLiteUtil {
    /**
     * Creates an instance of SyncDbUtils.
     * cn a conexão que será utilizada para salvar os dados
     * @param {?} cn
     */
    constructor(cn) {
        this.cn = cn;
    }
    /**
     * Cria um relacionamento 1-n, inserindo na entidade filha o id do pai
     *
     * items os registros pai
     * fatherTable o nome da tabela pi
     * @template T
     * @param {?} items
     * @param {?} fatherTable
     * @return {?} Os registros relacionados
     */
    updateChildrenRelation(items, fatherTable) {
        // FIXME: Fazer com que todos os filhos também tenha relacionamento com o pai mestre
        /** @type {?} */
        const childrenFound = [];
        /** @type {?} */
        const models = Object.keys(items[0]);
        for (const item of items) {
            for (const model of models) {
                if (UtilsFn.isArray(item[model])) {
                    /** @type {?} */
                    const children = (/** @type {?} */ (item[model]));
                    /** @type {?} */
                    const cf = childrenFound.find((/**
                     * @param {?} x
                     * @return {?}
                     */
                    (x) => x.table === model));
                    if (!cf) {
                        childrenFound.push({ table: model, items: children });
                    }
                    else {
                        cf.items = cf.items.concat(children);
                    }
                    for (const child of children) {
                        child[`${fatherTable}Id`] = item.id;
                    }
                }
            }
        }
        return childrenFound;
    }
    /**
     * Armazena os dados enviados, substituindo o registro anterior e criando tabelas para
     * entidades filhas(arrays)
     * data A requisição de sync com os dados
     * mainTable O nome da tabela principal
     * deleteBefore se deve sempre deletar antes de inserir o registro
     * currentTx a transação corrente, não deve ser passado sem entender o mecanismo interno
     * @template T
     * @param {?} data
     * @param {?} mainTable
     * @param {?=} deleteBefore
     * @param {?=} currentTx
     * @return {?}
     */
    storeData(data, mainTable, deleteBefore = true, currentTx) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!data.items || data.items.length === 0) {
                return;
            }
            /** @type {?} */
            const children = this.updateChildrenRelation(data.items, mainTable);
            /** @type {?} */
            const model = this.modelWithoutArrays(data.items[0]);
            if (!currentTx) {
                try {
                    yield this.cn.transaction((/**
                     * @param {?} tx
                     * @return {?}
                     */
                    (tx) => {
                        this.checkTableStateTx(tx, mainTable, model, (/**
                         * @param {?} txa
                         * @param {?} rs
                         * @return {?}
                         */
                        (txa, rs) => {
                            for (const item of data.items) {
                                this.upsert(txa, mainTable, model, item, deleteBefore);
                            }
                            for (const child of children) {
                                this.storeData({
                                    items: child.items
                                }, `${mainTable}_${child.table}`, deleteBefore, tx);
                            }
                        }));
                    }));
                }
                catch (error) {
                    console.log(error);
                }
            }
            else { // Continua na transação corrente
                this.checkTableStateTx(currentTx, mainTable, model, (/**
                 * @param {?} tx
                 * @param {?} rs
                 * @return {?}
                 */
                (tx, rs) => {
                    if (mainTable.search('_') !== -1 && data.items.length > 0) {
                        // Delete childs first
                        /** @type {?} */
                        const tables = mainTable.split('_');
                        tables.splice(tables.length - 1, 1);
                        /** @type {?} */
                        const fieldId = tables.join('_') + 'Id';
                        /** @type {?} */
                        const fathersId = [];
                        data.items.forEach((/**
                         * @param {?} e
                         * @return {?}
                         */
                        (e) => {
                            if (!fathersId.find((/**
                             * @param {?} x
                             * @return {?}
                             */
                            (x) => x === e[fieldId]))) {
                                fathersId.push(e[fieldId]);
                            }
                        }));
                        fathersId.forEach((/**
                         * @param {?} e
                         * @return {?}
                         */
                        (e) => {
                            console.log(`DELETE FROM ${mainTable} where ${fieldId} = ?`, [e]);
                            tx.executeSql(`DELETE FROM ${mainTable} where ${fieldId} = ?`, [e], (/**
                             * @param {?} txa
                             * @param {?} res
                             * @return {?}
                             */
                            (txa, res) => { }), (/**
                             * @param {?} error
                             * @return {?}
                             */
                            (error) => console.log(error)));
                        }));
                    }
                    for (const item of data.items) {
                        this.upsert(currentTx, mainTable, model, item, deleteBefore);
                    }
                    for (const child of children) {
                        this.storeData({
                            items: child.items
                        }, `${mainTable}_${child.table}`, deleteBefore, currentTx);
                    }
                }));
            }
        });
    }
    /**
     * Retorna se a tabela existe
     * table o nome da tabela que se deseja verificar se existe
     * @param {?} table
     * @return {?}
     */
    tableExists(table) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            let rs;
            try {
                rs = yield this.cn.executeSql('SELECT name FROM sqlite_master WHERE type=? AND name=?', ['table', table]);
            }
            catch (error) {
                console.log(JSON.stringify(error, null, 2));
            }
            if (rs && rs.rows.length > 0 && rs.rows.item(0).name === table) {
                return true;
            }
            return false;
        });
    }
    /**
     * Cria a tabela desejada, se ela não existir
     * table o nome da tabela
     * fields os campos pertencentes a tabela
     * @param {?} table
     * @param {...?} fields
     * @return {?}
     */
    createTable(table, ...fields) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.cn.executeSql('CREATE TABLE IF NOT EXISTS ' + table + ' (' + fields.join(',') + ')');
            }
            catch (rs) {
                console.log('success', rs);
            }
        });
    }
    /**
     * Verifica se os campos já existem na tabela e os adiciona
     * table O nome da tabela
     * fields os campos que serão adicionados, se ainda não existirem na tabela
     * @param {?} table
     * @param {...?} fields
     * @return {?}
     */
    checkAndAddFields(table, ...fields) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.cn.executeSql('PRAGMA table_info(' + table + ')');
            }
            catch (rs) {
                for (const field of fields) {
                    /** @type {?} */
                    const hasField = this.checkField(rs, field);
                    if (hasField === false) {
                        try {
                            yield this.cn.executeSql(`ALTER TABLE ${table} ADD ${field}`);
                        }
                        catch (rs) {
                            console.log('success', rs);
                        }
                    }
                }
            }
        });
    }
    /**
     * Retorna um array com os nomes dos campos da entidade, removendo os que são do tipo array
     * obj o objeto que se deseja avaliar
     * @private
     * @param {?} obj
     * @return {?}
     */
    modelWithoutArrays(obj) {
        /** @type {?} */
        const model = Object.keys(obj);
        for (let i = model.length - 1; i > -1; i--) {
            /** @type {?} */
            const field = model[i];
            if (UtilsFn.isArray(obj[field])) {
                model.splice(i, 1);
            }
        }
        return model;
    }
    /**
     * Executa o sql necessário para realizar o upsert de um registro, dentro de uma transação
     * tx a transação corrente
     * table o nome da tabela
     * model os campos envolvidos
     * item o item que se deseja inserir
     * deleteBefore se deve sempre excluir o registro antes de inserir
     * @private
     * @param {?} tx
     * @param {?} table
     * @param {?} model
     * @param {?} item
     * @param {?} deleteBefore
     * @return {?}
     */
    upsert(tx, table, model, item, deleteBefore) {
        if (deleteBefore) {
            if (table.search('_') === -1) { // Pula deleção de tabelas-filha
                console.log(`DELETE FROM ${table} where id = ?`, [item.id]);
                tx.executeSql(`DELETE FROM ${table} where id = ?`, [item.id], (/**
                 * @param {?} txa
                 * @param {?} rsa
                 * @return {?}
                 */
                (txa, rsa) => { }), (/**
                 * @param {?} error
                 * @return {?}
                 */
                (error) => console.log(error)));
            }
        }
        console.log(`INSERT INTO ${table} (${model.join(',')}) VALUES(${SqlUtilsFn.objectToSqlstring(item, model)})`);
        // tslint:disable-next-line:max-line-length
        tx.executeSql(`INSERT INTO ${table} (${model.join(',')}) VALUES(${SqlUtilsFn.objectToSqlstring(item, model)})`, undefined, (/**
         * @param {?} txa
         * @param {?} rsa
         * @return {?}
         */
        (txa, rsa) => { }), (/**
         * @param {?} error
         * @return {?}
         */
        (error) => console.log(error)));
    }
    /**
     * Na consulta de campos, verifica se o campo está presente na tabela
     * rs o recordset com os registros
     * field o nome do campo desejado
     * @private
     * @param {?} rs
     * @param {?} field
     * @return {?}
     */
    checkField(rs, field) {
        /** @type {?} */
        let hasField = false;
        for (let i = 0; i < rs.rows.length; i++) {
            if (rs.rows.item(i).name === field) {
                hasField = true;
                break;
            }
        }
        return hasField;
    }
    /**
     * Cria ou atualização a estrutura da tabela dentro de uma transação
     * tx a transação corrente
     * mainTable o nome da tabela
     * model os campos da tabela
     * afterTx o que fazer após a transação ocorrer com sucesso
     * @private
     * @param {?} tx
     * @param {?} mainTable
     * @param {?} model
     * @param {?} afterTx
     * @return {?}
     */
    checkTableStateTx(tx, mainTable, model, afterTx) {
        tx.executeSql('SELECT name FROM sqlite_master WHERE type=? AND name=?', ['table', mainTable], (/**
         * @param {?} txa
         * @param {?} rsa
         * @return {?}
         */
        (txa, rsa) => {
            if (rsa && rsa.rows.length > 0 && rsa.rows.item(0).name === mainTable) {
                txa.executeSql('PRAGMA table_info(' + mainTable + ')', undefined, (/**
                 * @param {?} txb
                 * @param {?} rsb
                 * @return {?}
                 */
                (txb, rsb) => {
                    /** @type {?} */
                    let qtd = 0;
                    /** @type {?} */
                    const fieldsToAdd = [];
                    for (const field of model) {
                        /** @type {?} */
                        const hasField = this.checkField(rsb, field);
                        if (hasField === false) {
                            qtd++;
                            fieldsToAdd.push(field);
                        }
                    }
                    if (fieldsToAdd.length === 0) {
                        afterTx(txb, rsb);
                    }
                    for (const field of fieldsToAdd) {
                        txb.executeSql(`ALTER TABLE ${mainTable} ADD ${field}`, undefined, (/**
                         * @param {?} txc
                         * @param {?} rsc
                         * @return {?}
                         */
                        (txc, rsc) => {
                            qtd--;
                            if (qtd === 0) {
                                afterTx(txc, rsc);
                            }
                        }));
                    }
                }));
            }
            else {
                txa.executeSql('CREATE TABLE IF NOT EXISTS ' + mainTable + ' (' + model.join(',') + ')', undefined, (/**
                 * @param {?} txb
                 * @param {?} rsb
                 * @return {?}
                 */
                (txb, rsb) => {
                    afterTx(txb, rsb);
                }));
            }
        }));
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { SyncSqLiteUtil };

//# sourceMappingURL=totvs-log-mobile-foundation-src-sync-sqlite-util.js.map