import { __awaiter } from 'tslib';
import { forkJoin, of } from 'rxjs';
import { Storage } from '@ionic/storage';
import { Platform, Events } from '@ionic/angular';
import { StorageUtil } from 'totvs-log-mobile-foundation/src/storage-util';
import { delay } from 'rxjs/operators';
import { Network } from '@ionic-native/network/ngx';
import { Injectable, InjectionToken, Inject, Injector, NgModule } from '@angular/core';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe abstrata que todo recurso de sincronização deve implementar,
 * independente do sentido
 * Para registrar o serviço em que a sincronização automática reconheça:
 * export const entidadeToken = new InjectionToken<entidadeProvider>('');
 * export class entidadeProvider extends SyncHandler{}
 * {
 * provide: entidadeToken,
 * useClass: entidadeProvider
 * }
 * @abstract
 */
class SyncHandler {
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe que contém todos os eventos disponíbilizados pelo serviço gerenciador de
 * sincronismo
 */
class SyncEvents {
}
/**
 * Evento que o sincronizador dispara ao terminar um sincronismo.
 * Deve-se filtrar pelo nome da entidade
 */
SyncEvents.GOT_ENTITY_SYNC_DATA = 'Sync:GotEntitySyncData';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const SYNC_CONFIG = new InjectionToken('SYNC_CONFIG: ISyncConfig token de configuração da sincronização');
/**
 * Serviço provedor de sincronismo utilizando serviços externos
 */
class SyncProvider {
    /**
     * Construtor da classe
     * platform Platform do ionic
     * authService Serviço de autenticação
     * storage Serviço de armazenamento do ionic
     * config Token de configuração do serviço
     * network Serviço de rede do ionic
     * events Serviço de eventos do ionic
     * injector Injetor de dependências do angular
     * @param {?} platform
     * @param {?} storage
     * @param {?} config
     * @param {?} network
     * @param {?} events
     * @param {?} injector
     */
    constructor(platform, storage, config, network, events, injector) {
        this.storage = storage;
        this.config = config;
        this.network = network;
        this.events = events;
        this.injector = injector;
        /**
         * @ignore
         */
        this.syncing = false;
        platform.ready().then((/**
         * @return {?}
         */
        () => __awaiter(this, void 0, void 0, function* () {
            if (platform.is('cordova')) {
                this.syncStorage = new StorageUtil(this.storage);
                yield this.syncStorage.config({
                    name: 'SyncEntitiesState',
                    exclusive: true,
                    lazy: true,
                    suffixFn: (/**
                     * @return {?}
                     */
                    () => this.storageName),
                    initFn: (/**
                     * @return {?}
                     */
                    () => {
                        /** @type {?} */
                        const init = [];
                        config.entities.forEach((/**
                         * @param {?} e
                         * @return {?}
                         */
                        (e) => init.push(this.newState(e))));
                        return init;
                    })
                });
                this.setDefaults();
                this.subscribeEvents();
            }
        })));
    }
    /**
     * Se a sincronização está acontecendo
     * @return {?}
     */
    get isSyncing() {
        return this.syncing;
    }
    /**
     * @param {?} storageNameFunction
     * @return {?}
     */
    set storageNameFunction(storageNameFunction) {
        this.storageNameFn = storageNameFunction;
    }
    /**
     * Atributo que retorna o nome do storage
     * @return {?}
     */
    get storageName() {
        if (this.storageNameFn) {
            return this.storageNameFn();
        }
        else {
            return 'sync';
        }
    }
    /**
     * Retorna se o sincronismo está ativo
     * @return {?}
     */
    get autoWatchEnabled() {
        if (this.currentNetworkType === 'none') {
            return false;
        }
        if (!this.config.autoWatch) {
            return false;
        }
        if (this.config.autoWatch.enabledOnNetwork === 'any' || this.config.autoWatch.enabledOnNetwork === this.currentNetworkType) {
            return true;
        }
        else {
            /** @type {?} */
            const array = (/** @type {?} */ (this.config.autoWatch.enabledOnNetwork));
            if (array.findIndex((/**
             * @param {?} x
             * @return {?}
             */
            (x) => x === 'any' || x === this.currentNetworkType)) !== -1) {
                return true;
            }
        }
        return false;
    }
    /**
     * Dispara o sincronismo de todas as entidades registradas.
     * Consulte isSyncing para saber se a sincronização está em andamento.
     * @return {?}
     */
    syncAll() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.syncing === false) {
                this.syncing = true;
                try {
                    yield this.syncStorage.read();
                    this.syncAllByOrder(this.config.entities);
                }
                catch (error) {
                    console.error('Sync autowatch stopped. Reason', error);
                    this.syncing = false;
                    this.stopAutoWatch();
                }
            }
        });
    }
    /**
     * Função recursiva que dispara a sincronização de entidades conforme ordem recebida
     * entities Lista com todas as entidades
     * order Ordem da execução
     * @param {?} entities
     * @param {?=} order
     * @return {?}
     */
    syncAllByOrder(entities, order = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            const entitiesByOrder = [];
            entities.forEach((/**
             * @param {?} e
             * @return {?}
             */
            (e) => {
                if ((order === 0 && (e.order === undefined || e.order === 0)) ||
                    (order !== 0 && (e.order === order))) {
                    entitiesByOrder.push(e);
                }
            }));
            if (entitiesByOrder.length > 0) {
                /** @type {?} */
                const result = yield this.syncAllEntities(entitiesByOrder);
                // Aborta demais sincronizações
                if (!result) {
                    return;
                }
            }
            if (entities.some((/**
             * @param {?} e
             * @return {?}
             */
            (e) => e.order > order))) {
                this.syncAllByOrder(entities, order + 1);
            }
            else {
                this.syncing = false;
            }
        });
    }
    /**
     * Dispara o sincronismo de todas as entidades registradas conforme ordem
     * Publica o evento SyncEvents.GOT_ENTITY_SYNC_DATA da finalização da sincronização da entidade.
     * entities Lista com as entidades que deverão ser sincronizadas
     * Indicação se ocorreu com sucesso
     * @param {?} entities
     * @return {?}
     */
    syncAllEntities(entities) {
        return new Promise((/**
         * @param {?} resolve
         * @return {?}
         */
        (resolve) => {
            /** @type {?} */
            const requests = [];
            entities.forEach((/**
             * @param {?} e
             * @return {?}
             */
            (e) => {
                /** @type {?} */
                const provider = (/** @type {?} */ (this.injector.get(e.providerToken)));
                /** @type {?} */
                const state = this.getState(e);
                state.isSyncing = true;
                requests.push(provider.getSyncData(state));
            }));
            forkJoin(requests).subscribe((/**
             * @param {?} x
             * @return {?}
             */
            (x) => {
                if (x) {
                    x.forEach((/**
                     * @param {?} e
                     * @return {?}
                     */
                    (e) => {
                        if (e) {
                            this.syncStorage.data[this.getStateIdx(e.state.entity)] = e.state;
                        }
                    }));
                    this.syncStorage.updateStorage()
                        .then((/**
                     * @return {?}
                     */
                    () => {
                        x.forEach((/**
                         * @param {?} e
                         * @return {?}
                         */
                        (e) => {
                            this.events.publish(SyncEvents.GOT_ENTITY_SYNC_DATA, e);
                        }));
                        resolve(true);
                    }));
                }
                else {
                    this.syncing = false;
                    resolve(false);
                }
            }), (/**
             * @return {?}
             */
            () => {
                this.syncing = false;
                resolve(false);
            }));
        }));
    }
    /**
     * Inicia o serviço que executa temporalmente a sincronização de todas as entidades,
     * caso o dispositivo esteja na rede escolhida
     * now Se o serviço deve iniciar imediatamente ou deve aguardar o próximo cíclo
     * @param {?} now
     * @return {?}
     */
    startAutoWatch(now) {
        this.watchNetwork();
        this.stopAutoWatch();
        if (now) {
            this.nowToken = setTimeout((/**
             * @return {?}
             */
            () => {
                if (this.autoWatchEnabled) {
                    this.syncAll();
                }
            }));
            this.intervalToken = setInterval((/**
             * @return {?}
             */
            () => {
                if (this.autoWatchEnabled) {
                    this.syncAll();
                }
            }), this.getNextCall());
        }
        else {
            this.scheduleToken = of({}).pipe(delay(this.getNextCall())).subscribe((/**
             * @return {?}
             */
            () => {
                this.intervalToken = setInterval((/**
                 * @return {?}
                 */
                () => {
                    if (this.autoWatchEnabled) {
                        this.syncAll();
                    }
                }), this.getNextCall());
            }));
        }
    }
    /**
     * Cancela a sincronização automática.
     * @return {?}
     */
    stopAutoWatch() {
        // TODO: Implementar uma espera da sincronização em andamento terminar antes de encerrar os timers ativos.
        if (this.nowToken) {
            clearInterval(this.nowToken);
            this.nowToken = undefined;
        }
        if (this.scheduleToken) {
            this.scheduleToken.unsubscribe();
            this.scheduleToken = undefined;
        }
        if (this.intervalToken) {
            clearInterval(this.intervalToken);
            this.intervalToken = undefined;
        }
    }
    /**
     * Permite que seja trocado quais os tipos de rede
     * que a sincronização automática está ativa
     * types
     * @param {...?} types
     * @return {?}
     */
    changeEnabledAutoWatchNetWorks(...types) {
        this.config.autoWatch.enabledOnNetwork = types;
        this.setDefaults();
    }
    /**
     * Permite que sejam trocadas as configurações de sincronização automática
     * newConfiguration Novas configurações
     * @param {?=} newConfiguration
     * @return {?}
     */
    changeSyncWatch(newConfiguration) {
        this.config.autoWatch = newConfiguration;
        if (this.config.autoWatch) {
            this.startAutoWatch(true);
        }
    }
    /**
     * Recupera o estado de sincronização da entidade
     * entity a entidade de sincronização
     * @private
     * @param {?} entity
     * @return {?}
     */
    getState(entity) {
        /** @type {?} */
        let e = this.syncStorage.data.find((/**
         * @param {?} x
         * @return {?}
         */
        (x) => x.entity.name === entity.name));
        if (e === undefined) {
            /** @type {?} */
            const newEntity = this.newState(entity);
            this.syncStorage.data.push(newEntity);
            e = newEntity;
        }
        return e;
    }
    /**
     * Recupera o índice do estado de sincronização da entidade no storage
     * entity a entidade de sincronização
     * @private
     * @param {?} entity
     * @return {?}
     */
    getStateIdx(entity) {
        /** @type {?} */
        let idx = this.syncStorage.data.findIndex((/**
         * @param {?} x
         * @return {?}
         */
        (x) => x.entity.name === entity.name));
        if (idx === -1) {
            /** @type {?} */
            const newEntity = this.newState(entity);
            idx = this.syncStorage.data.push(newEntity) - 1;
        }
        return idx;
    }
    /**
     * Cria um novo estado de sincronização para a entidade
     * entity a entidade de sincronização
     * @private
     * @param {?} entity
     * @return {?}
     */
    newState(entity) {
        return { entity, entityVersion: undefined, isSyncing: false };
    }
    /**
     * Calcula o tempo em ms para a próxima chamada da sincronização
     * Quando não configurado, o tempo será um minuto
     * @private
     * @return {?}
     */
    getNextCall() {
        if (this.config.autoWatch) {
            /** @type {?} */
            let times = this.config.autoWatch.checkQuantity;
            if (times === undefined || times <= 0 || isNaN(times)) {
                times = 1;
            }
            /** @type {?} */
            const type = this.config.autoWatch.checkType;
            if (type === 'days') {
                return SyncProvider.daysMs * times;
            }
            else if (type === 'hours') {
                return SyncProvider.hoursMs * times;
            }
            else if (type === 'minutes') {
                return SyncProvider.minutesMs * times;
            }
            else if (type === 'seconds') {
                return SyncProvider.secondsMs * times;
            }
            else { // Defaults for new types
                return SyncProvider.minutesMs * times;
            }
        }
        else {
            return SyncProvider.minutesMs;
        }
    }
    /**
     * @ignore
     * Configura os valores padrão, quando não informados
     * @private
     * @return {?}
     */
    setDefaults() {
        if (this.config.autoWatch) {
            if (this.config.autoWatch.enabledOnNetwork instanceof Array) {
                /** @type {?} */
                const array = (/** @type {?} */ (this.config.autoWatch.enabledOnNetwork));
                if (array.length === 0) {
                    this.config.autoWatch.enabledOnNetwork = 'any';
                }
            }
        }
    }
    /**
     * @ignore
     * Assina a todos os eventos do interesse da classe
     * @private
     * @return {?}
     */
    subscribeEvents() {
        if (this.config.autoWatch) {
            this.currentNetworkType = this.network.type;
            if (this.config.autoWatch.auto) {
                this.startAutoWatch(false);
            }
            this.watchNetwork();
        }
    }
    /**
     * Assina o evento de trocas de rede
     * @private
     * @return {?}
     */
    watchNetwork() {
        if (!this.onNetworkChange) {
            this.onNetworkChange = this.network.onchange().subscribe((/**
             * @return {?}
             */
            () => {
                setTimeout((/**
                 * @return {?}
                 */
                () => {
                    this.currentNetworkType = this.network.type;
                }), 3000);
            }));
        }
    }
}
/**
 * Constante convertendo 1 dia em milisegundos
 */
SyncProvider.daysMs = 86400000;
/**
 * Constante convertendo 1 hora em milisegundos
 */
SyncProvider.hoursMs = 3600000;
/**
 * Constante convertendo 1 minuto em milisegundos
 */
SyncProvider.minutesMs = 60000;
/**
 * Constante convertendo 1 segundo em milisegundos
 */
SyncProvider.secondsMs = 1000;
SyncProvider.decorators = [
    { type: Injectable }
];
/** @nocollapse */
SyncProvider.ctorParameters = () => [
    { type: Platform },
    { type: Storage },
    { type: undefined, decorators: [{ type: Inject, args: [SYNC_CONFIG,] }] },
    { type: Network },
    { type: Events },
    { type: Injector }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Módulo com as dependências para sincronização mobile
 */
class SyncModule {
    /**
     * @return {?}
     */
    static forRoot() {
        return {
            ngModule: SyncModule,
            providers: [
                SyncProvider,
                Network
            ]
        };
    }
}
SyncModule.decorators = [
    { type: NgModule, args: [{},] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { SyncHandler, SyncEvents, SyncProvider, SYNC_CONFIG, SyncModule };

//# sourceMappingURL=totvs-log-mobile-foundation-src-sync.js.map