import { TranslateService, TranslateModule, TranslateLoader } from '@ngx-translate/core';
import { TranslateHttpLoader } from '@ngx-translate/http-loader';
import { ReplaySubject } from 'rxjs';
import { map, take } from 'rxjs/operators';
import { CommonModule, DatePipe } from '@angular/common';
import { WebStorageStateStore } from 'oidc-client';
import { InAppBrowser } from '@ionic-native/in-app-browser/ngx';
import { CriticalError, I18nBaseService, AbstractShowMessage, ShowMessageType, ShowMessageLinkActionType, MessageDisplayerService, LoadingDefault, ApiDateTimeService, HttpInterceptorBaseService, MessageUtils, DefaultDateViewConverterService, DateViewConverter, DateTimeModule, TimeViewConverter, DefaultTimeViewConverterService, AuthBaseService, AUTH_SETTINGS, AUTH_TYPE, AuthHandlerType, UtilsFn, AuthInterceptorHandlerService, UserPermissionService } from 'totvs-log-base-foundation';
import { OidcCordovaIabEvents, OidcCordovaIFrameNavigator, OidcCordovaPopupNavigator } from 'totvs-log-mobile-foundation/src/oidc-cordova';
import { HttpClient, HTTP_INTERCEPTORS } from '@angular/common/http';
import { __awaiter } from 'tslib';
import { StorageUtil } from 'totvs-log-mobile-foundation/src/storage-util';
import { Storage } from '@ionic/storage';
import { ToastController, AlertController, LoadingController, Events, Platform } from '@ionic/angular';
import { Injectable, Inject, InjectionToken, NgModule, Pipe, APP_INITIALIZER, Injector } from '@angular/core';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Serviço utilizado para carregar as literais
 */
class IntlLiteralsLoaderService {
    /**
     * Método construtor da classe
     * translate
     * @param {?} translate
     */
    constructor(translate) {
        this.translate = translate;
        /**
         * Responsável por receber as configurações do serviço de tradução para carregar as literais
         */
        this.mobileOptions = { language: 'pt-br', url: '' };
    }
    /**
     * Método responsável por setar as opções e retornar a literal atualizada
     * options
     * @param {?} options
     * @return {?}
     */
    updateOptions(options) {
        try {
            Object.assign(this.mobileOptions, options);
            return this.load();
        }
        catch (error) {
            throw new CriticalError(error);
        }
    }
    /**
     * Método responsável por setar o carregamento e retornar a literal atualizada
     * @return {?}
     */
    load() {
        return this.translate.getTranslation(this.mobileOptions.language)
            .pipe(map((/**
         * @param {?} res
         * @return {?}
         */
        (res) => res)));
    }
}
IntlLiteralsLoaderService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
IntlLiteralsLoaderService.ctorParameters = () => [
    { type: TranslateService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Token de configuração da url de internacionalização
 * @type {?}
 */
const INTL_CONFIG = new InjectionToken('Configurações da internacionalização');
/**
 * Serviço de internacionalização da aplicação
 */
class IntlService extends I18nBaseService {
    /**
     * Método construtor que inicializa a classe
     * translate Serviço de internacionalização
     * @param {?} config
     * @param {?} intlLiteralsLoader
     */
    constructor(config, intlLiteralsLoader) {
        super(intlLiteralsLoader);
        /**
         * Atributo de opções de internacionalização
         */
        this.options = { language: 'pt-br', url: '' };
        /** @type {?} */
        let matchedNavigatorAppLanguage;
        if (config.supportedLanguages) {
            matchedNavigatorAppLanguage = config.supportedLanguages.find((/**
             * @param {?} appSupportedLanguage
             * @return {?}
             */
            appSupportedLanguage => {
                return appSupportedLanguage.language === navigator.language;
            }));
        }
        if (!matchedNavigatorAppLanguage) {
            matchedNavigatorAppLanguage = { language: 'pt-br' };
        }
        this.options.language = matchedNavigatorAppLanguage.language.toLowerCase();
    }
}
IntlService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
IntlService.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [INTL_CONFIG,] }] },
    { type: IntlLiteralsLoaderService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Serviço responsável pela apresentação de mensagens
 */
class ShowMessageService extends AbstractShowMessage {
    /**
     * Método construtor que inicializa a classe
     * toastCtrl Serviço de TOAST
     * alertCtrl Serviço de Alerta
     * i18n Serviço de internacionalização
     * @param {?} toastCtrl
     * @param {?} alertCtrl
     */
    constructor(toastCtrl, alertCtrl) {
        super();
        this.toastCtrl = toastCtrl;
        this.alertCtrl = alertCtrl;
    }
    /**
     * Método responsável por apresentar uma mensagem
     * @param {?} showMessageOptions
     * @return {?}
     */
    displayMessage(showMessageOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            switch (showMessageOptions.messageType) {
                case ShowMessageType.NotificationSuccess:
                    yield this.showNotification(showMessageOptions);
                    break;
                case ShowMessageType.DialogAlert:
                case ShowMessageType.DialogConfirmation:
                    yield this.showDialog(showMessageOptions);
                    break;
                default:
                    throw new Error('Tipo de mensagem inválido');
            }
        });
    }
    /**
     * Método responsável por apresentar uma mensagem de notificação
     * @private
     * @param {?} showMessageOptions
     * @return {?}
     */
    showNotification(showMessageOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            const toast = yield this.toastCtrl.create({
                message: showMessageOptions.messageText,
                duration: showMessageOptions.messageLinkActionType ===
                    ShowMessageLinkActionType.No
                    ? 3000
                    : 10000,
                position: 'middle',
                showCloseButton: showMessageOptions.messageLinkActionType !==
                    ShowMessageLinkActionType.No,
                closeButtonText: showMessageOptions.messageLinkLabel
            });
            toast.onDidDismiss().then((/**
             * @return {?}
             */
            () => {
                if (showMessageOptions.messageLinkActionType ===
                    ShowMessageLinkActionType.LinkNewTab) {
                    this.openLink(showMessageOptions.messageLinkAction);
                }
                else if (showMessageOptions.messageLinkActionType ===
                    ShowMessageLinkActionType.Method &&
                    showMessageOptions.messageLinkAction) {
                    showMessageOptions.messageLinkAction();
                }
            }));
            yield toast.present();
        });
    }
    /**
     * Método responsável por apresentar uma mensagem de diálogo
     * @private
     * @param {?} showMessageOptions
     * @return {?}
     */
    showDialog(showMessageOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            const buttons = [];
            /** @type {?} */
            let alert;
            if (showMessageOptions.messageCancelLabel &&
                showMessageOptions.messageCancelLabel.trim() !== '' &&
                showMessageOptions.messageType === ShowMessageType.DialogConfirmation) {
                buttons.push({
                    text: showMessageOptions.messageCancelLabel,
                    handler: (/**
                     * @return {?}
                     */
                    () => {
                        alert.dismiss(false);
                        return false;
                    })
                });
            }
            if (showMessageOptions.messageConfirmLabel &&
                showMessageOptions.messageConfirmLabel.trim() !== '') {
                buttons.push({
                    text: showMessageOptions.messageConfirmLabel,
                    handler: (/**
                     * @return {?}
                     */
                    () => {
                        alert.dismiss(true);
                        return false;
                    })
                });
            }
            alert = yield this.alertCtrl.create({
                message: showMessageOptions.messageDetails,
                header: showMessageOptions.messageTitle,
                subHeader: showMessageOptions.messageText,
                buttons
            });
            alert.onDidDismiss().then((/**
             * @param {?} data
             * @return {?}
             */
            data => {
                switch (data) {
                    case true:
                        if (showMessageOptions.messageConfirmAction) {
                            showMessageOptions.messageConfirmAction();
                        }
                        break;
                    default:
                        if (showMessageOptions.messageCancelAction) {
                            showMessageOptions.messageCancelAction();
                        }
                }
            }));
            yield alert.present();
        });
    }
    /**
     * Método responsável pela abertura de um link
     * @private
     * @param {?} link
     * @return {?}
     */
    openLink(link) {
        link = link.indexOf('http://') === -1 ? `http://${link}` : link;
        window.open(link, '_blank');
    }
}
ShowMessageService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
ShowMessageService.ctorParameters = () => [
    { type: ToastController },
    { type: AlertController }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ShowMessageModule {
}
ShowMessageModule.decorators = [
    { type: NgModule, args: [{
                declarations: [],
                imports: [],
                exports: [],
                providers: [
                    ShowMessageService,
                    MessageDisplayerService,
                    {
                        provide: AbstractShowMessage,
                        useClass: ShowMessageService
                    }
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Pipe de internacionalização da aplicação
 */
class I18NPipe {
    /**
     * Método construtor que inicializa a classe
     * i18nService Serviço de internacionalização
     * @param {?} i18nService
     */
    constructor(i18nService) {
        this.i18nService = i18nService;
    }
    /**
     * Método responsável pela transformação da string
     * query
     * data
     * @param {?} query
     * @param {?} data
     * @return {?}
     */
    transform(query, data) {
        if (!query || query.length === 0) {
            return '';
        }
        /** @type {?} */
        const ret = this.cached(query, data);
        if (ret) {
            return ret;
        }
        if (!data) {
            data = [''];
        }
        else if (!Array.isArray(data)) {
            data = [data];
        }
        return this.i18nService.format(data, query);
    }
    /**
     * Método responsável por retornar o que está em cache
     * query
     * args
     * @private
     * @param {?} query
     * @param {...?} args
     * @return {?}
     */
    cached(query, ...args) {
        return '';
    }
}
I18NPipe.decorators = [
    { type: Pipe, args: [{ name: 'format' },] }
];
/** @nocollapse */
I18NPipe.ctorParameters = () => [
    { type: I18nBaseService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const i18nFactory = (/**
 * @param {?} i18n
 * @param {?} config
 * @return {?}
 */
(i18n, config) => {
    return (/**
     * @return {?}
     */
    () => new Promise((/**
     * @param {?} resolve
     * @return {?}
     */
    resolve => {
        if (config.lazy) {
            resolve(true);
        }
        else {
            i18n.updateOptions({}).subscribe((/**
             * @return {?}
             */
            () => {
                resolve(true);
            }), (/**
             * @param {?} error
             * @return {?}
             */
            (error) => {
                console.error(error);
                resolve(false);
            }));
        }
    })));
});

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Função responsável por indicar o diretório que contém os arquivos de tradução
 * http
 * @param {?} http
 * @return {?}
 */
function createTranslateLoader(http) {
    return new TranslateHttpLoader(http, './assets/i18n/', '.json');
}
const ɵ0 = i18nFactory;
/**
 * Módulo responsável pela configuração da internacionalização da aplicação
 */
class IntlModule {
}
IntlModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    TranslateModule.forRoot({
                        loader: {
                            provide: TranslateLoader,
                            useFactory: createTranslateLoader,
                            deps: [HttpClient],
                        },
                    }),
                ],
                declarations: [I18NPipe],
                exports: [TranslateModule, I18NPipe],
                providers: [
                    IntlLiteralsLoaderService,
                    {
                        provide: I18nBaseService,
                        useClass: IntlService,
                    },
                    {
                        provide: APP_INITIALIZER,
                        useFactory: ɵ0,
                        deps: [I18nBaseService, INTL_CONFIG],
                        multi: true,
                    },
                ],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Serviço responsável pela apresentação de loading
 */
class ShowLoadingService extends LoadingDefault {
    /**
     * Método construtor que inicializa a classe
     * i18n Serviço de internacionalização
     * loadingController Serviço de loading
     * @param {?} i18n
     * @param {?} loadingController
     */
    constructor(i18n, loadingController) {
        super();
        this.i18n = i18n;
        this.loadingController = loadingController;
        this.loadingActived = new ReplaySubject(1);
        this.thfContentTemplate = '<span class="thf-loading-icon"></span>{content}';
    }
    /**
     * @return {?}
     */
    showLoading() {
        if (!this.active) {
            this.active = true;
            /** @type {?} */
            let content = '';
            if (this.loadingText) {
                content = this.loadingText.text;
            }
            this.loadingController
                .create({
                message: this.thfContentTemplate.replace('{content}', content),
                spinner: null,
            })
                .then((/**
             * @param {?} loading
             * @return {?}
             */
            loading => {
                this.loading = loading;
                this.loading.present().then((/**
                 * @return {?}
                 */
                () => {
                    this.loadingActived.next();
                }));
            }));
        }
        return this.loadingActived.pipe(take(1));
    }
    /**
     * @return {?}
     */
    hideLoading() {
        this.loadingActived.pipe(take(1)).subscribe((/**
         * @return {?}
         */
        () => {
            this.active = false;
            this.loadingText = undefined;
            this.loading.dismiss();
        }));
    }
    /**
     * @param {?} loadingText
     * @return {?}
     */
    setLoadingText(loadingText) {
        this.loadingActived.pipe(take(1)).subscribe((/**
         * @return {?}
         */
        () => {
            setTimeout((/**
             * @return {?}
             */
            () => {
                /** @type {?} */
                const baseMessage = this.i18n.text(loadingText);
                if (this.loading) {
                    this.loading.message = this.thfContentTemplate.replace('{content}', baseMessage.text);
                    // this.loading.forceUpdate();
                }
                this.loadingText = baseMessage;
            }), 33);
        }));
    }
}
ShowLoadingService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
ShowLoadingService.ctorParameters = () => [
    { type: I18nBaseService },
    { type: LoadingController }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Serviço responsável por fornecer atributos e métodos que
 * poderão ser utilizados em toda a aplicação
 */
class GlobalService {
    /**
     * Método construtor que inicializa o serviço
     * i18n Serviço de internacionalização
     * msg Serviço de mensageria para usuário
     * date Serviço de formatação de data
     * @param {?} i18n
     * @param {?} msg
     * @param {?} loading
     * @param {?} date
     */
    constructor(i18n, msg, loading, date) {
        this.i18n = i18n;
        this.msg = msg;
        this.loading = loading;
        this.date = date;
    }
}
GlobalService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
GlobalService.ctorParameters = () => [
    { type: I18nBaseService },
    { type: MessageDisplayerService },
    { type: ShowLoadingService },
    { type: ApiDateTimeService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Serviço utilizado para realizar o tratamento das respostas
 */
class DefaultHttpInterceptorService extends HttpInterceptorBaseService {
    /**
     * Método construtor da classe
     * inj
     * @param {?} inj
     */
    constructor(inj) {
        super();
        this.inj = inj;
    }
    /**
     * Método responsável por realizar o tratamento de respostas de sucesso
     * response
     * @param {?} response
     * @return {?}
     */
    interceptResponse(response) {
        this.msg = this.inj.get(MessageDisplayerService);
        if (response.body && response.body.totvs_messages) {
            if (response.body.totvs_messages instanceof Array) {
                /** @type {?} */
                const messages = response.body.totvs_messages;
                for (const message of messages) {
                    this.showTotvsMessage(message);
                }
            }
            else {
                this.showTotvsMessage((/** @type {?} */ (response.body.totvs_messages)));
            }
        }
    }
    /**
     * Método responsável por realizar o tratamento de respostas de erro
     * response
     * request
     * @param {?} response
     * @param {?} request
     * @return {?}
     */
    interceptError(response, request) {
        /** @type {?} */
        const error = (/** @type {?} */ (response.error));
        if (error && error.message && !HttpInterceptorBaseService.isErrorNotificationDisabled(request)) {
            /** @type {?} */
            const title = HttpInterceptorBaseService.titleFromTotvsError(error);
            /** @type {?} */
            const detail = HttpInterceptorBaseService.detailFromTotvsError(error);
            this.showTotvsMessage({
                type: 'error',
                code: title,
                detail
            });
        }
    }
    /**
     * Método responsável por mostrar a mensagem
     * message
     * @param {?} message
     * @return {?}
     */
    showTotvsMessage(message) {
        /** @type {?} */
        const messageType = MessageUtils.showMessageTypeFromTotvsMessage(message);
        this.msg.displayMessage({
            messageType,
            messageText: message.code,
            messageDetails: message.detail
        });
    }
    /**
     * Método responsável por realizar o tratamento da requisição/retorno
     * req
     * next
     * @param {?} req
     * @param {?} next
     * @return {?}
     */
    intercept(req, next) {
        return super.intercept(req, next);
    }
}
DefaultHttpInterceptorService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
DefaultHttpInterceptorService.ctorParameters = () => [
    { type: Injector }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class InterceptorModule {
}
InterceptorModule.decorators = [
    { type: NgModule, args: [{
                imports: [],
                exports: [],
                providers: [
                    DefaultHttpInterceptorService,
                    {
                        provide: HTTP_INTERCEPTORS,
                        useClass: DefaultHttpInterceptorService,
                        multi: true
                    }
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DateTimeModule$1 {
}
DateTimeModule$1.decorators = [
    { type: NgModule, args: [{
                declarations: [],
                imports: [
                    CommonModule,
                    DateTimeModule
                ],
                exports: [],
                providers: [
                    {
                        provide: DateViewConverter,
                        useClass: DefaultDateViewConverterService
                    },
                    {
                        provide: TimeViewConverter,
                        useClass: DefaultTimeViewConverterService
                    },
                    DatePipe,
                    DefaultDateViewConverterService,
                    DefaultTimeViewConverterService,
                    ApiDateTimeService
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ShowLoadingModule {
}
ShowLoadingModule.decorators = [
    { type: NgModule, args: [{
                declarations: [],
                imports: [],
                exports: [],
                providers: [
                    LoadingController,
                    ShowLoadingService
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FndModule {
    /**
     * @param {?} config
     * @return {?}
     */
    static forRoot(config) {
        return {
            ngModule: FndModule,
            providers: [GlobalService, { provide: INTL_CONFIG, useValue: config }],
        };
    }
}
FndModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    IntlModule,
                    ShowMessageModule,
                    InterceptorModule,
                    DateTimeModule$1,
                    ShowLoadingModule,
                ],
                declarations: [],
                exports: [
                    IntlModule,
                    ShowMessageModule,
                    InterceptorModule,
                    DateTimeModule$1,
                    ShowLoadingModule,
                ],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe de serviço com métodos para autenticação
 * @extends AuthBaseService
 */
class AuthService extends AuthBaseService {
    /**
     * Método construtor da classe
     * authSettings Token de injeção com as configurações de autenticação
     * authType Token de injeção com o tipo de autenticação
     * @param {?} authSettings
     * @param {?} authType
     * @param {?} events
     */
    constructor(authSettings, authType, events) {
        super(authSettings);
        this.events = events;
        this.authType = authType;
    }
    /**
     * Método responsável executar o processo de login/autenticação
     * @return {?}
     */
    loginFlow() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.authType.type === AuthHandlerType.oauth2) {
                /** @type {?} */
                let loaded;
                /** @type {?} */
                let doneFn;
                /** @type {?} */
                let loadedFn;
                /** @type {?} */
                let failedFn;
                /** @type {?} */
                const doneProm = new Promise((/**
                 * @param {?} resolve
                 * @param {?} reject
                 * @return {?}
                 */
                (resolve, reject) => {
                    doneFn = resolve;
                }));
                try {
                    loadedFn = (/**
                     * @return {?}
                     */
                    () => {
                        console.log('loadedFn');
                        loaded = true;
                        UtilsFn.timeout(doneProm, 1000);
                        this.events.unsubscribe(OidcCordovaIabEvents.LoadSuccessful, loadedFn);
                    });
                    this.events.subscribe(OidcCordovaIabEvents.LoadSuccessful, loadedFn);
                    failedFn = (/**
                     * @return {?}
                     */
                    () => {
                        console.log('failedFn');
                        loaded = false;
                        UtilsFn.timeout(doneProm, 1000);
                        this.events.unsubscribe(OidcCordovaIabEvents.LoadFailed, failedFn);
                    });
                    this.events.subscribe(OidcCordovaIabEvents.LoadFailed, failedFn);
                    yield this.manager.signinPopup();
                    doneFn();
                }
                catch (error) {
                    console.log('catch');
                    doneFn();
                    this.events.unsubscribe(OidcCordovaIabEvents.LoadFailed, failedFn);
                    this.events.unsubscribe(OidcCordovaIabEvents.LoadSuccessful, loadedFn);
                    console.log('error', error);
                    if (!loaded || error) {
                        throw error;
                    }
                }
            }
        });
    }
    /**
     * Método responsável executar o processo de logout
     * @return {?}
     */
    logoutFlow() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.authType.type === AuthHandlerType.oauth2) {
                yield this.manager.signoutPopup();
            }
        });
    }
}
AuthService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
AuthService.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [AUTH_SETTINGS,] }] },
    { type: undefined, decorators: [{ type: Inject, args: [AUTH_TYPE,] }] },
    { type: Events }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?} iab
 * @param {?} events
 * @param {?} authSettings
 * @return {?}
 */
function authFactory(iab, events, authSettings) {
    return Object.assign(authSettings, {
        popupNavigator: new OidcCordovaPopupNavigator(iab, events),
        iframeNavigator: new OidcCordovaIFrameNavigator(iab),
        checkSessionInterval: authSettings.checkSessionInterval || 30000,
        userStore: new WebStorageStateStore({ store: window.localStorage }),
    });
}
/** @type {?} */
const AUTH_FACTORY = new InjectionToken('');
/**
 * Módulo que exporta e ativa as dependências para utilizar autenticação com fluxo oauth2
 * Para garantir que os serviços tenham uma única instância, usar AuthModule.forRoot() ao importar
 * @export
 * FndModule
 */
class AuthModule {
    /**
     * \@dynamic
     * @param {?} authSettings
     * @param {?} handlerTypeSettings
     * @return {?}
     */
    static forRoot(authSettings, handlerTypeSettings) {
        return {
            ngModule: AuthModule,
            providers: [
                InAppBrowser,
                UserPermissionService,
                {
                    provide: AuthBaseService,
                    useClass: AuthService,
                },
                {
                    provide: AUTH_FACTORY,
                    useValue: authSettings
                },
                {
                    provide: AUTH_SETTINGS,
                    useFactory: authFactory,
                    deps: [InAppBrowser, Events, AUTH_FACTORY],
                },
                { provide: AUTH_TYPE, useValue: handlerTypeSettings },
            ],
        };
    }
}
AuthModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    {
                        provide: HTTP_INTERCEPTORS,
                        useClass: AuthInterceptorHandlerService,
                        multi: true,
                    },
                ],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe que contém todos os eventos relacionados ao Provider de Tenant
 */
class MultiTenantEvents {
}
/**
 * Evento disparado quando é realizado uma mudança do tenant corrente e este é diferente do anterior
 */
MultiTenantEvents.CURRENT_TENANT_CHANGED = 'CurrentTenant:Changed';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class MultiTenantService {
    /**
     * Método construtor responsável por inicializar o serviço
     * @param {?} platform Serviço relacionado a plataforma
     * @param {?} storage Serviço relacionado ao storage
     * @param {?} events
     */
    constructor(platform, storage, events) {
        this.storage = storage;
        this.events = events;
        /**
         * @ignore
         * Indica se a plataforma é cordova
         */
        this.isCordova = false;
        this.ready = new Promise((/**
         * @param {?} resolve
         * @return {?}
         */
        resolve => {
            this.readyFn = resolve;
        }));
        platform.ready().then((/**
         * @return {?}
         */
        () => {
            this.isCordova = platform.is('cordova');
            if (this.isCordova) {
                this.tenantsStorage = new StorageUtil(this.storage);
                this.tenantsStorage
                    .config({
                    name: 'Tenants',
                    id: 'shortNameTenant',
                    exclusive: false,
                    lazy: false,
                    initFn: (/**
                     * @return {?}
                     */
                    () => []),
                })
                    .then((/**
                 * @return {?}
                 */
                () => { }));
                this.currentTenantStorage = new StorageUtil(this.storage);
                this.currentTenantStorage
                    .config({
                    name: 'CurrentTenant',
                    id: 'shortNameTenant',
                    exclusive: false,
                    lazy: true,
                    initFn: (/**
                     * @return {?}
                     */
                    () => undefined),
                })
                    .then((/**
                 * @return {?}
                 */
                () => { }));
                this.readyFn();
            }
        }));
    }
    /**
     * @return {?}
     */
    get tenants() {
        return this.tenantsStorage ? this.tenantsStorage.data : [];
    }
    /**
     * @return {?}
     */
    get isReady() {
        return this.ready;
    }
    /**
     * @return {?}
     */
    getCurrentTenant() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.ready;
            /** @type {?} */
            const currentTenant = (/** @type {?} */ ((yield this.currentTenantStorage.read())));
            try {
                if (yield this.tenantsStorage.get(currentTenant.shortNameTenant)) {
                    return currentTenant;
                }
            }
            catch (error) {
                // console.error(error);
                this.currentTenantStorage.data = undefined;
                yield this.currentTenantStorage.updateStorage();
                return undefined;
            }
        });
    }
    /**
     * @param {?} currentTenant
     * @return {?}
     */
    setCurrentTenant(currentTenant) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.ready;
            yield this.currentTenantStorage.read();
            if (!this.currentTenantStorage.data) {
                this.currentTenantStorage.data = currentTenant;
            }
            /** @type {?} */
            const tenantChanged = currentTenant.shortNameTenant !==
                this.currentTenantStorage.data.shortNameTenant;
            this.currentTenantStorage.data = currentTenant;
            yield this.currentTenantStorage.updateStorage();
            if (tenantChanged) {
                this.events.publish(MultiTenantEvents.CURRENT_TENANT_CHANGED, currentTenant);
            }
        });
    }
    /**
     * @param {?} tenant
     * @return {?}
     */
    addTenant(tenant) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.ready;
            /** @type {?} */
            const tenants = (/** @type {?} */ ((yield this.tenantsStorage.read())));
            // Verificação de duplicidade de tenant
            if (tenants && tenants.length > 0) {
                tenants.forEach((/**
                 * @param {?} tenantFromStorage
                 * @param {?} index
                 * @return {?}
                 */
                (tenantFromStorage, index) => {
                    if (tenantFromStorage.shortNameTenant === tenant.shortNameTenant &&
                        tenantFromStorage.isDemo === tenant.isDemo) {
                        tenants.splice(index, 1);
                    }
                }));
            }
            this.tenantsStorage.data = tenants;
            this.tenantsStorage.data.push(tenant);
            this.sortTenantsByShortName();
            yield this.tenantsStorage.updateStorage();
            return tenant;
        });
    }
    /**
     * @param {?} tenant
     * @return {?}
     */
    deleteTenant(tenant) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.ready;
            try {
                yield this.tenantsStorage.delete(tenant.shortNameTenant);
                yield this.tenantsStorage.read();
            }
            catch (error) {
                console.error(error);
            }
        });
    }
    /**
     * @private
     * @return {?}
     */
    sortTenantsByShortName() {
        /** @type {?} */
        const sortBy = (/**
         * @param {?} fn
         * @return {?}
         */
        fn => (/**
         * @param {?} a
         * @param {?} b
         * @return {?}
         */
        (a, b) => -(fn(a) < fn(b)) || +(fn(a) > fn(b))));
        /** @type {?} */
        const getShortName = (/**
         * @param {?} o
         * @return {?}
         */
        o => o.shortNameTenant);
        /** @type {?} */
        const sortByShortName = sortBy(getShortName);
        this.tenantsStorage.data.sort(sortByShortName);
    }
}
MultiTenantService.SHORT_NAME_TENANT_FIELD = 'tenantId';
MultiTenantService.IS_DEMONSTRATION = 'isDemo';
MultiTenantService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
MultiTenantService.ctorParameters = () => [
    { type: Platform },
    { type: Storage },
    { type: Events }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class MultiTenantModule {
    /**
     * @return {?}
     */
    static forRoot() {
        return {
            ngModule: MultiTenantModule,
            providers: [
                MultiTenantService
            ],
        };
    }
}
MultiTenantModule.decorators = [
    { type: NgModule, args: [{
                declarations: [],
                imports: [],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { FndModule, authFactory, AuthModule, AuthService, DateTimeModule$1 as DateTimeModule, GlobalService, InterceptorModule, DefaultHttpInterceptorService, i18nFactory, IntlLiteralsLoaderService, IntlModule, I18NPipe, IntlService, INTL_CONFIG, createTranslateLoader, ShowMessageModule, ShowMessageService, ShowLoadingService, ShowLoadingModule, MultiTenantEvents, MultiTenantService, MultiTenantModule };

//# sourceMappingURL=totvs-log-mobile-foundation.js.map