import { __values } from 'tslib';
import { GlobalService } from 'totvs-log-mobile-foundation';
import { BaseMessage } from 'totvs-log-base-foundation';
import { Injectable, NgModule } from '@angular/core';
import { Camera } from '@ionic-native/camera/ngx';
import { Platform, Events } from '@ionic/angular';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe que contém os eventos que o utilitário assina e publica
 */
var CameraUtilEvents = /** @class */ (function () {
    function CameraUtilEvents() {
    }
    // FIXME - Nomes de eventos mais específicos, para evitar possíveis conflitos
    /**
     * Evento publicado pelo utilitário quando uma foto é tirada e o arquivo é gerado
     * no cache do aplicativo
     */
    CameraUtilEvents.CACHE_FILE_CREATED = 'CacheFile:Created';
    return CameraUtilEvents;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// TODO - Verificar se é válido ter classes abstratas para os utilitários
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de camera
 * com métodos que facilitam a solicitação de permissão e apresentação da
 * camera
 */
var CameraUtilProvider = /** @class */ (function () {
    /**
     * @ignore
     * Método responsável por construir o serviço
     * platform Serviço relacionado a plataforma
     * camera Serviço do plugin nativo de camera
     * global Serviço global da aplicação
     */
    function CameraUtilProvider(platform, camera, global, events) {
        var _this = this;
        this.camera = camera;
        this.global = global;
        this.events = events;
        /**
         * @ignore
         * Indica se a plataforma é cordova
         */
        this.isCordova = false;
        platform.ready().then((/**
         * @return {?}
         */
        function () {
            _this.isCordova = platform.is('cordova');
            if (_this.isCordova) {
                _this.defaultOptions = {
                    quality: 75,
                    destinationType: _this.camera.DestinationType.FILE_URI,
                    encodingType: _this.camera.EncodingType.JPEG,
                    mediaType: _this.camera.MediaType.PICTURE,
                    targetWidth: 1920,
                    targetHeight: 1080,
                    correctOrientation: true
                };
            }
        }));
    }
    /**
     * Método responsável por apresentar a camera para que o usuário
     * possa tirar uma foto
     * Este método publica o evento CameraUtilEvents.CACHE_FILE_CREATED quando um arquivo é criado em
     * cache (Mais informações sobre eventos {@link https://ionicframework.com/docs/api/util/Events/|Ionic Events})
     * {CameraOptions} options Opções da camera
     * @returns Promessa com o FILE_URI (opção padrão) ou o que for definido na opção 'destinationType' que pode ser passada como parâmetro
     */
    /**
     * Método responsável por apresentar a camera para que o usuário
     * possa tirar uma foto
     * Este método publica o evento CameraUtilEvents.CACHE_FILE_CREATED quando um arquivo é criado em
     * cache (Mais informações sobre eventos {\@link https://ionicframework.com/docs/api/util/Events/|Ionic Events})
     * {CameraOptions} options Opções da camera
     * @param {?=} options
     * @return {?} Promessa com o FILE_URI (opção padrão) ou o que for definido na opção 'destinationType' que pode ser passada como parâmetro
     */
    CameraUtilProvider.prototype.takePicture = /**
     * Método responsável por apresentar a camera para que o usuário
     * possa tirar uma foto
     * Este método publica o evento CameraUtilEvents.CACHE_FILE_CREATED quando um arquivo é criado em
     * cache (Mais informações sobre eventos {\@link https://ionicframework.com/docs/api/util/Events/|Ionic Events})
     * {CameraOptions} options Opções da camera
     * @param {?=} options
     * @return {?} Promessa com o FILE_URI (opção padrão) ou o que for definido na opção 'destinationType' que pode ser passada como parâmetro
     */
    function (options) {
        var _this = this;
        return new Promise((/**
         * @param {?} resolve
         * @param {?} reject
         * @return {?}
         */
        function (resolve, reject) {
            var e_1, _a;
            if (_this.isCordova === false) {
                _this.global.msg.showMessage(new BaseMessage({
                    literal: 'recursoNaoEstaDisponivelMsg'
                }));
                reject();
            }
            /** @type {?} */
            var cameraOptions = Object.assign({}, _this.defaultOptions);
            if (options) {
                try {
                    for (var _b = __values(Object.keys(options)), _c = _b.next(); !_c.done; _c = _b.next()) {
                        var prop = _c.value;
                        cameraOptions[prop] = options[prop];
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
            }
            _this.camera.getPicture(cameraOptions)
                .then((/**
             * @param {?} imageData
             * @return {?}
             */
            function (imageData) {
                _this.events.publish(CameraUtilEvents.CACHE_FILE_CREATED, imageData);
                resolve(imageData);
            }), (/**
             * @param {?} error
             * @return {?}
             */
            function (error) { return reject(error); })).catch((/**
             * @param {?} error
             * @return {?}
             */
            function (error) {
                /** @type {?} */
                var literal;
                if (error && error === 20) {
                    literal = 'permissaoNaoConcedidaParaORecursoMsg';
                }
                else {
                    literal = 'recursoNaoEstaDisponivelMsg';
                }
                _this.global.msg.showMessage(new BaseMessage({
                    literal: literal
                }));
                reject(error);
            }));
        }));
    };
    /**
     * Método responsável por limpar as imagens em cache geradas durante
     * a utilização do plugin enquanto estiver ativo
     */
    /**
     * Método responsável por limpar as imagens em cache geradas durante
     * a utilização do plugin enquanto estiver ativo
     * @return {?}
     */
    CameraUtilProvider.prototype.cleanup = /**
     * Método responsável por limpar as imagens em cache geradas durante
     * a utilização do plugin enquanto estiver ativo
     * @return {?}
     */
    function () {
        if (this.isCordova) {
            this.camera.cleanup()
                .then((/**
             * @return {?}
             */
            function () { }))
                .catch((/**
             * @return {?}
             */
            function () { }));
        }
    };
    CameraUtilProvider.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    CameraUtilProvider.ctorParameters = function () { return [
        { type: Platform },
        { type: Camera },
        { type: GlobalService },
        { type: Events }
    ]; };
    return CameraUtilProvider;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Módulo que contém um provedor utilitário que auxilia na
 * utilização do plugin nativo de camera
 */
var CameraUtilModule = /** @class */ (function () {
    function CameraUtilModule() {
    }
    CameraUtilModule.decorators = [
        { type: NgModule, args: [{
                    providers: [
                        Camera,
                        CameraUtilProvider
                    ]
                },] }
    ];
    return CameraUtilModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { CameraUtilModule, CameraUtilEvents, CameraUtilProvider };

//# sourceMappingURL=totvs-log-mobile-foundation-src-camera-util.js.map