import { __awaiter, __generator } from 'tslib';
import { CriticalError } from 'totvs-log-base-foundation';
import { Platform } from '@ionic/angular';
import { Injectable, NgModule } from '@angular/core';
import { File } from '@ionic-native/file/ngx';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// TODO - Verificar se é válido ter classes abstratas para os utilitários
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de arquivo
 * com métodos que facilitam a manipulação de arquivos no aplicativo
 */
var FileUtilProvider = /** @class */ (function () {
    /**
     * @ignore
     * Método construtor responsável por inicializar o serviço
     * platform Serviço relacionado a plataforma
     * file Serviço do plugin nativo de arquivo
     * events Serviço de eventos do Ionic
     */
    function FileUtilProvider(platform, file) {
        var _this = this;
        this.file = file;
        /**
         * @ignore
         * Indica se a plataforma é cordova
         */
        this.isCordova = false;
        platform.ready().then((/**
         * @return {?}
         */
        function () { return _this.isCordova = platform.is('cordova'); }));
    }
    /**
     * Método responsável por remover um arquivo
     * uri URI do arquivo que deve ser removido
     * Objeto que representa o resultado da remoção do arquivo
     */
    /**
     * Método responsável por remover um arquivo
     * uri URI do arquivo que deve ser removido
     * Objeto que representa o resultado da remoção do arquivo
     * @param {?} uri
     * @return {?}
     */
    FileUtilProvider.prototype.removeFile = /**
     * Método responsável por remover um arquivo
     * uri URI do arquivo que deve ser removido
     * Objeto que representa o resultado da remoção do arquivo
     * @param {?} uri
     * @return {?}
     */
    function (uri) {
        return __awaiter(this, void 0, void 0, function () {
            var fileName, path, removeResult, error_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.isCordova === false) {
                            throw new CriticalError('Platform is not cordova (removeFile)');
                        }
                        if (!uri || uri === '') {
                            throw new CriticalError('URI invalid (removeFile)');
                        }
                        fileName = this.getFilenameFromUri(uri);
                        path = this.getPathFromUri(uri);
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, this.file.removeFile(path, fileName)];
                    case 2:
                        removeResult = _a.sent();
                        return [3 /*break*/, 4];
                    case 3:
                        error_1 = _a.sent();
                        throw new CriticalError('Error in remove file', error_1);
                    case 4: return [2 /*return*/, removeResult];
                }
            });
        });
    };
    /**
     * Método responsável por transferir os arquivos para o storage do aplicativo
     * uris URIs dos arquivos que devem ser copiados
     * Lista de URIs dos arquivos copiados para o storage do aplicativo
     */
    /**
     * Método responsável por transferir os arquivos para o storage do aplicativo
     * uris URIs dos arquivos que devem ser copiados
     * Lista de URIs dos arquivos copiados para o storage do aplicativo
     * @param {...?} uris
     * @return {?}
     */
    FileUtilProvider.prototype.fileCacheToAppStorage = /**
     * Método responsável por transferir os arquivos para o storage do aplicativo
     * uris URIs dos arquivos que devem ser copiados
     * Lista de URIs dos arquivos copiados para o storage do aplicativo
     * @param {...?} uris
     * @return {?}
     */
    function () {
        var uris = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            uris[_i] = arguments[_i];
        }
        return __awaiter(this, void 0, void 0, function () {
            var promisesCopy, filesCopied, error_2, promisesRemove, filesUrl, error_3;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.isCordova === false) {
                            throw new CriticalError('Platform is not cordova');
                        }
                        promisesCopy = [];
                        uris.forEach((/**
                         * @param {?} uri
                         * @return {?}
                         */
                        function (uri) {
                            /** @type {?} */
                            var fileName = decodeURI(_this.getFilenameFromUri(uri));
                            /** @type {?} */
                            var path = _this.getPathFromUri(uri);
                            promisesCopy.push(_this.file.copyFile(path, fileName, _this.file.dataDirectory, fileName)
                                .then((/**
                             * @param {?} entry
                             * @return {?}
                             */
                            function (entry) {
                                return {
                                    sourcePath: path,
                                    sourceFileName: fileName,
                                    targetFile: entry
                                };
                            })));
                        }));
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, Promise.all(promisesCopy)];
                    case 2:
                        filesCopied = _a.sent();
                        return [3 /*break*/, 4];
                    case 3:
                        error_2 = _a.sent();
                        throw new CriticalError("Error in copy file", error_2);
                    case 4:
                        promisesRemove = [];
                        filesUrl = [];
                        filesCopied.forEach((/**
                         * @param {?} fileCopied
                         * @return {?}
                         */
                        function (fileCopied) {
                            filesUrl.push(fileCopied.targetFile.toURL());
                            promisesRemove.push(_this.file.removeFile(fileCopied.sourcePath, fileCopied.sourceFileName)
                                .then((/**
                             * @param {?} result
                             * @return {?}
                             */
                            function (result) {
                                return result;
                            })));
                        }));
                        _a.label = 5;
                    case 5:
                        _a.trys.push([5, 7, , 8]);
                        return [4 /*yield*/, Promise.all(promisesRemove)];
                    case 6:
                        _a.sent();
                        return [3 /*break*/, 8];
                    case 7:
                        error_3 = _a.sent();
                        throw new CriticalError("Error in remove file", error_3);
                    case 8: return [2 /*return*/, filesUrl];
                }
            });
        });
    };
    /**
     * @ignore
     * Método responsável por recuperar o nome do arquivo de um URI
     * uri URI do arquivo
     */
    /**
     * @ignore
     * Método responsável por recuperar o nome do arquivo de um URI
     * uri URI do arquivo
     * @private
     * @param {?} uri
     * @return {?}
     */
    FileUtilProvider.prototype.getFilenameFromUri = /**
     * @ignore
     * Método responsável por recuperar o nome do arquivo de um URI
     * uri URI do arquivo
     * @private
     * @param {?} uri
     * @return {?}
     */
    function (uri) {
        /** @type {?} */
        var fileName;
        try {
            /** @type {?} */
            var fileNameArray = uri.split('/');
            fileName = fileNameArray[fileNameArray.length - 1];
        }
        catch (error) {
            throw new CriticalError('Error (getFilenameFromUri)', error);
        }
        return fileName;
    };
    /**
     * @ignore
     * Método responsável por recuperar o path do arquivo de um URI
     * uri URI do arquivo
     */
    /**
     * @ignore
     * Método responsável por recuperar o path do arquivo de um URI
     * uri URI do arquivo
     * @private
     * @param {?} uri
     * @return {?}
     */
    FileUtilProvider.prototype.getPathFromUri = /**
     * @ignore
     * Método responsável por recuperar o path do arquivo de um URI
     * uri URI do arquivo
     * @private
     * @param {?} uri
     * @return {?}
     */
    function (uri) {
        /** @type {?} */
        var filePath;
        try {
            /** @type {?} */
            var fileNameArray = uri.split('/');
            fileNameArray.splice(fileNameArray.length - 1, 1);
            filePath = fileNameArray.join('/');
        }
        catch (error) {
            throw new CriticalError('Error (getPathFromUri)', error);
        }
        return filePath;
    };
    FileUtilProvider.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    FileUtilProvider.ctorParameters = function () { return [
        { type: Platform },
        { type: File }
    ]; };
    return FileUtilProvider;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Módulo que contém um provedor utilitário que auxilia na
 * utilização do plugin nativo de manipulação de arquivo
 */
var FileUtilModule = /** @class */ (function () {
    function FileUtilModule() {
    }
    FileUtilModule.decorators = [
        { type: NgModule, args: [{
                    providers: [
                        File,
                        FileUtilProvider
                    ]
                },] }
    ];
    return FileUtilModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { FileUtilModule, FileUtilProvider };

//# sourceMappingURL=totvs-log-mobile-foundation-src-file-util.js.map