import { __awaiter, __generator } from 'tslib';
import { GlobalService } from 'totvs-log-mobile-foundation';
import { BaseMessage, CriticalError } from 'totvs-log-base-foundation';
import { Platform } from '@ionic/angular';
import { Injectable, NgModule } from '@angular/core';
import { Geolocation } from '@ionic-native/geolocation/ngx';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// TODO - Verificar se é válido ter classes abstratas para os utilitários
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de geolocalização
 * com métodos que facilitam a solicitação de permissão e identificação de geolocalização
 */
var GeolocationUtilProvider = /** @class */ (function () {
    /**
     * @ignore
     * Método responsável por construir o serviço
     * platform Serviço relacionado a plataforma
     * geolocation Serviço do plugin nativo de geolocalização
     * global Serviço global da aplicação
     */
    function GeolocationUtilProvider(platform, geolocation, global) {
        var _this = this;
        this.geolocation = geolocation;
        this.global = global;
        /**
         * @ignore
         * Indica se a plataforma é cordova
         */
        this.isCordova = false;
        platform.ready().then((/**
         * @return {?}
         */
        function () { return _this.isCordova = platform.is('cordova'); }));
    }
    /**
     * Método responsável por obter a posição corrente do usuário
     * {GeolocationOptions} options Opções para recuperação da geolocalização
     * Posição
     */
    /**
     * Método responsável por obter a posição corrente do usuário
     * {GeolocationOptions} options Opções para recuperação da geolocalização
     * Posição
     * @param {?} options
     * @return {?}
     */
    GeolocationUtilProvider.prototype.getCurrentPosition = /**
     * Método responsável por obter a posição corrente do usuário
     * {GeolocationOptions} options Opções para recuperação da geolocalização
     * Posição
     * @param {?} options
     * @return {?}
     */
    function (options) {
        return __awaiter(this, void 0, void 0, function () {
            var msg;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.isCordova === false) {
                            msg = new BaseMessage({
                                literal: 'recursoNaoEstaDisponivelMsg'
                            });
                            this.global.msg.showMessage(msg);
                            throw new CriticalError('Platform is not cordova');
                        }
                        return [4 /*yield*/, this.geolocation.getCurrentPosition(options)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * Método responsável por calcular a distância entre dois pontos através de suas latitudes e longitudes
     * point1 Coordenadas do primeiro ponto
     * point2 Coordenadas do segundo ponto
     */
    /**
     * Método responsável por calcular a distância entre dois pontos através de suas latitudes e longitudes
     * point1 Coordenadas do primeiro ponto
     * point2 Coordenadas do segundo ponto
     * @param {?} point1
     * @param {?} point2
     * @return {?}
     */
    GeolocationUtilProvider.prototype.calculatePointsKMDistance = /**
     * Método responsável por calcular a distância entre dois pontos através de suas latitudes e longitudes
     * point1 Coordenadas do primeiro ponto
     * point2 Coordenadas do segundo ponto
     * @param {?} point1
     * @param {?} point2
     * @return {?}
     */
    function (point1, point2) {
        if (!this.validGeoPositionPoint(point1) || !this.validGeoPositionPoint(point2)) {
            throw new Error('ERROR - Invalid values');
        }
        if (point1.latitude === point2.latitude && point1.longitude === point2.longitude) {
            return 0;
        }
        /** @type {?} */
        var radlat1 = this.degreesToRad(point1.latitude);
        /** @type {?} */
        var radlat2 = this.degreesToRad(point2.latitude);
        /** @type {?} */
        var theta = point1.longitude - point2.longitude;
        /** @type {?} */
        var radtheta = this.degreesToRad(theta);
        /** @type {?} */
        var dist = Math.sin(radlat1) * Math.sin(radlat2) + Math.cos(radlat1) * Math.cos(radlat2) * Math.cos(radtheta);
        if (dist > 1) {
            dist = 1;
        }
        dist = Math.acos(dist);
        dist = dist * 180 / Math.PI;
        dist = dist * 60 * 1.1515;
        // KM
        return dist * 1.609344;
    };
    /**
     * Converte os graus decimais em radianos
     * degrees Graus decimais
     */
    /**
     * Converte os graus decimais em radianos
     * degrees Graus decimais
     * @param {?} degrees
     * @return {?}
     */
    GeolocationUtilProvider.prototype.degreesToRad = /**
     * Converte os graus decimais em radianos
     * degrees Graus decimais
     * @param {?} degrees
     * @return {?}
     */
    function (degrees) {
        return Math.PI * degrees / 180;
    };
    /**
     * Valida os valores do ponto recebido e retorna o resultado
     * point Ponto contendo latitude e longitude
     */
    /**
     * Valida os valores do ponto recebido e retorna o resultado
     * point Ponto contendo latitude e longitude
     * @param {?} point
     * @return {?}
     */
    GeolocationUtilProvider.prototype.validGeoPositionPoint = /**
     * Valida os valores do ponto recebido e retorna o resultado
     * point Ponto contendo latitude e longitude
     * @param {?} point
     * @return {?}
     */
    function (point) {
        if (!point) {
            return false;
        }
        if (isNaN(point.latitude) || isNaN(point.longitude)) {
            return false;
        }
        if (point.latitude === null || point.longitude === null) {
            return false;
        }
        if (point.latitude > 90 || point.latitude < -90) {
            return false;
        }
        if (point.longitude > 180 || point.longitude < -180) {
            return false;
        }
        return true;
    };
    GeolocationUtilProvider.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    GeolocationUtilProvider.ctorParameters = function () { return [
        { type: Platform },
        { type: Geolocation },
        { type: GlobalService }
    ]; };
    return GeolocationUtilProvider;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Módulo que contém um provedor utilitário que auxilia na
 * utilização do plugin nativo de geolocalização
 */
var GeolocationUtilModule = /** @class */ (function () {
    function GeolocationUtilModule() {
    }
    GeolocationUtilModule.decorators = [
        { type: NgModule, args: [{
                    providers: [
                        Geolocation,
                        GeolocationUtilProvider
                    ]
                },] }
    ];
    return GeolocationUtilModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { GeolocationUtilModule, GeolocationUtilProvider };

//# sourceMappingURL=totvs-log-mobile-foundation-src-geolocation-util.js.map