import { __awaiter, __generator } from 'tslib';
import { CriticalError, AuthBaseService } from 'totvs-log-base-foundation';
import { Platform } from '@ionic/angular';
import { Injectable, NgModule } from '@angular/core';
import { HTTP } from '@ionic-native/http/ngx';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// TODO - Verificar se é válido ter classes abstratas para os utilitários
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de requisição http
 * nativa do dispositivo com métodos que facilitam a utilização do plugin
 */
var HttpOauth2UtilProvider = /** @class */ (function () {
    /**
     * @ignore
     * Método construtor responsável por inicializar o serviço
     * platform Serviço relacionado a plataforma
     * file Serviço do plugin nativo de arquivo
     * events Serviço de eventos do Ionic
     */
    function HttpOauth2UtilProvider(platform, auth, http) {
        var _this = this;
        this.auth = auth;
        this.http = http;
        /**
         * @ignore
         * Indica se a plataforma é cordova
         */
        this.isCordova = false;
        platform.ready().then((/**
         * @return {?}
         */
        function () { return _this.isCordova = platform.is('cordova'); }));
    }
    /**
     * Método responsável por realizar o upload de um arquivo
     * @template T Objeto de resposta do servidor quando sucesso
     * url URL do Endpoint de Upload
     * filePath URI do arquivo no dispositivo
     * name Nome do arquivo
     * body Informações no corpo da mensagem
     * headers Informações de cabeçalho da mensagem
     * {IHttpUtilResult} Resultado da requisição
     */
    /**
     * Método responsável por realizar o upload de um arquivo
     * url URL do Endpoint de Upload
     * filePath URI do arquivo no dispositivo
     * name Nome do arquivo
     * body Informações no corpo da mensagem
     * headers Informações de cabeçalho da mensagem
     * {IHttpUtilResult} Resultado da requisição
     * @template T
     * @param {?} url
     * @param {?} filePath
     * @param {?=} name
     * @param {?=} body
     * @param {?=} headers
     * @return {?}
     */
    HttpOauth2UtilProvider.prototype.uploadFile = /**
     * Método responsável por realizar o upload de um arquivo
     * url URL do Endpoint de Upload
     * filePath URI do arquivo no dispositivo
     * name Nome do arquivo
     * body Informações no corpo da mensagem
     * headers Informações de cabeçalho da mensagem
     * {IHttpUtilResult} Resultado da requisição
     * @template T
     * @param {?} url
     * @param {?} filePath
     * @param {?=} name
     * @param {?=} body
     * @param {?=} headers
     * @return {?}
     */
    function (url, filePath, name, body, headers) {
        return __awaiter(this, void 0, void 0, function () {
            var user, postHeaders, postBody, result, response, error_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.isCordova === false) {
                            throw new CriticalError('Platform is not cordova (uploadFile)');
                        }
                        url = url.replace('{tenant}', this.auth.shortNameTenant || '');
                        // Injeção da autorização Oauth2
                        return [4 /*yield*/, this.auth.getUser()];
                    case 1:
                        user = _a.sent();
                        if (headers) {
                            postHeaders = Object.assign({}, headers, {
                                Authorization: user ? user.token_type + " " + user.access_token : ''
                            });
                        }
                        else {
                            postHeaders = {
                                Authorization: user ? user.token_type + " " + user.access_token : ''
                            };
                        }
                        if (body) {
                            postBody = body;
                        }
                        else {
                            postBody = {};
                        }
                        if (name === undefined || name === '') {
                            name = 'file';
                        }
                        result = {
                            status: 0,
                            data: undefined,
                            error: undefined,
                            sourceResponse: undefined
                        };
                        _a.label = 2;
                    case 2:
                        _a.trys.push([2, 4, , 5]);
                        return [4 /*yield*/, this.http.uploadFile(url, postBody, postHeaders, filePath, name)];
                    case 3:
                        response = _a.sent();
                        result.sourceResponse = response;
                        if (response) {
                            if (response.hasOwnProperty('status')) {
                                result.status = response.status;
                            }
                            if (response.hasOwnProperty('data')) {
                                try {
                                    result.data = (/** @type {?} */ (JSON.parse(response.data)));
                                }
                                catch (error) {
                                    result.error = error;
                                }
                            }
                        }
                        return [3 /*break*/, 5];
                    case 4:
                        error_1 = _a.sent();
                        result.error = error_1;
                        return [3 /*break*/, 5];
                    case 5: return [2 /*return*/, result];
                }
            });
        });
    };
    /**
     * Método que permite a alteração do modo de verificação do certificado SSL
     * mode Modo de verificação do certificado SSL
     */
    /**
     * Método que permite a alteração do modo de verificação do certificado SSL
     * mode Modo de verificação do certificado SSL
     * @param {?} mode
     * @return {?}
     */
    HttpOauth2UtilProvider.prototype.setSSLCertMode = /**
     * Método que permite a alteração do modo de verificação do certificado SSL
     * mode Modo de verificação do certificado SSL
     * @param {?} mode
     * @return {?}
     */
    function (mode) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.http.setSSLCertMode(mode)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    HttpOauth2UtilProvider.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    HttpOauth2UtilProvider.ctorParameters = function () { return [
        { type: Platform },
        { type: AuthBaseService },
        { type: HTTP }
    ]; };
    return HttpOauth2UtilProvider;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Módulo que contém um provedor utilitário que auxilia na
 * utilização do plugin nativo de requisição http
 */
var HttpOauth2UtilModule = /** @class */ (function () {
    function HttpOauth2UtilModule() {
    }
    HttpOauth2UtilModule.decorators = [
        { type: NgModule, args: [{
                    providers: [
                        HTTP,
                        HttpOauth2UtilProvider
                    ]
                },] }
    ];
    return HttpOauth2UtilModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { HttpOauth2UtilModule, HttpOauth2UtilProvider };

//# sourceMappingURL=totvs-log-mobile-foundation-src-http-util.js.map