import { __awaiter, __generator, __values, __spread } from 'tslib';
import { GlobalService } from 'totvs-log-mobile-foundation';
import { BaseMessage, CriticalError } from 'totvs-log-base-foundation';
import { Platform, Events } from '@ionic/angular';
import { ImagePicker, OutputType } from '@ionic-native/image-picker/ngx';
import { Injectable, NgModule } from '@angular/core';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe que contém os eventos que o utilitário assina e publica
 */
var ImagePickerUtilEvents = /** @class */ (function () {
    function ImagePickerUtilEvents() {
    }
    // FIXME - Nomes de eventos mais específicos, para evitar possíveis conflitos
    /**
     * Evento publicado pelo utilitário quando uma ou mais imagens são
     * selecionadas e geram uma cópia no cache do aplicativo
     */
    ImagePickerUtilEvents.CACHE_FILE_CREATED = 'CacheFile:Created';
    return ImagePickerUtilEvents;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// TODO - Verificar se é válido ter classes abstratas para os utilitários
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de seleção
 * de imagens do dispositivo com métodos que facilitam a solicitação de permissão,
 * apresentação e seleção de imagens
 */
var ImagePickerUtilProvider = /** @class */ (function () {
    /**
     * @ignore
     * Método responsável por construir o serviço
     * platform Serviço relacionado a plataforma
     * imagePicker Serviço do plugin nativo de galeria
     * global Serviço global da aplicação
     */
    function ImagePickerUtilProvider(platform, imagePicker, global, events) {
        var _this = this;
        this.imagePicker = imagePicker;
        this.global = global;
        this.events = events;
        /**
         * @ignore
         * Indica se a plataforma é cordova
         */
        this.isCordova = false;
        /**
         * @ignore
         * Opções padrões para a galeria
         */
        this.defaultOptions = {
            outputType: OutputType.FILE_URL,
            quality: 60
        };
        platform.ready().then((/**
         * @return {?}
         */
        function () { return _this.isCordova = platform.is('cordova'); }));
    }
    /**
     * Método responsável por abrir o seletor de imagens e retornar o caminho dos arquivos salvos no cache
     * Este método publica o evento ImagePickerUtilEvents.CACHE_FILE_CREATED quando um ou mais arquivos são selecionados
     * e criados em cache (Mais informações sobre eventos {@link https://ionicframework.com/docs/api/util/Events/|Ionic Events})
     * opt Customização das opções enviadas ao seletor de images
     * Lista de imagens escolhidas
     */
    /**
     * Método responsável por abrir o seletor de imagens e retornar o caminho dos arquivos salvos no cache
     * Este método publica o evento ImagePickerUtilEvents.CACHE_FILE_CREATED quando um ou mais arquivos são selecionados
     * e criados em cache (Mais informações sobre eventos {\@link https://ionicframework.com/docs/api/util/Events/|Ionic Events})
     * opt Customização das opções enviadas ao seletor de images
     * Lista de imagens escolhidas
     * @param {?=} opt
     * @return {?}
     */
    ImagePickerUtilProvider.prototype.chooseImage = /**
     * Método responsável por abrir o seletor de imagens e retornar o caminho dos arquivos salvos no cache
     * Este método publica o evento ImagePickerUtilEvents.CACHE_FILE_CREATED quando um ou mais arquivos são selecionados
     * e criados em cache (Mais informações sobre eventos {\@link https://ionicframework.com/docs/api/util/Events/|Ionic Events})
     * opt Customização das opções enviadas ao seletor de images
     * Lista de imagens escolhidas
     * @param {?=} opt
     * @return {?}
     */
    function (opt) {
        return __awaiter(this, void 0, void 0, function () {
            var e_1, _a, _b, msg, images, options, _c, _d, prop, hasPermission, msg, error_1;
            return __generator(this, function (_e) {
                switch (_e.label) {
                    case 0:
                        if (this.isCordova === false) {
                            msg = new BaseMessage({
                                literal: 'recursoNaoEstaDisponivelMsg'
                            });
                            this.global.msg.showMessage(msg);
                            throw new CriticalError('Platform is not cordova');
                        }
                        images = [];
                        _e.label = 1;
                    case 1:
                        _e.trys.push([1, 6, , 7]);
                        options = Object.assign({}, this.defaultOptions);
                        if (opt) {
                            try {
                                for (_c = __values(Object.keys(opt)), _d = _c.next(); !_d.done; _d = _c.next()) {
                                    prop = _d.value;
                                    options[prop] = opt[prop];
                                }
                            }
                            catch (e_1_1) { e_1 = { error: e_1_1 }; }
                            finally {
                                try {
                                    if (_d && !_d.done && (_a = _c.return)) _a.call(_c);
                                }
                                finally { if (e_1) throw e_1.error; }
                            }
                        }
                        return [4 /*yield*/, this.requestReadPermission()];
                    case 2:
                        hasPermission = _e.sent();
                        if (!hasPermission) return [3 /*break*/, 4];
                        return [4 /*yield*/, this.imagePicker.getPictures(options)];
                    case 3:
                        images = _e.sent();
                        (_b = this.events).publish.apply(_b, __spread([ImagePickerUtilEvents.CACHE_FILE_CREATED], images));
                        return [3 /*break*/, 5];
                    case 4:
                        msg = new BaseMessage({
                            literal: 'permissaoNaoConcedidaParaORecursoMsg'
                        });
                        this.global.msg.showMessage(msg);
                        throw new CriticalError('Permission denied');
                    case 5: return [3 /*break*/, 7];
                    case 6:
                        error_1 = _e.sent();
                        throw new CriticalError('Error in get pictures', error_1);
                    case 7: return [2 /*return*/, images];
                }
            });
        });
    };
    /**
     * Método responsável por requisitar permissão para acesso a galeria
     * Indicação se possui ou não permissão
     */
    /**
     * Método responsável por requisitar permissão para acesso a galeria
     * Indicação se possui ou não permissão
     * @return {?}
     */
    ImagePickerUtilProvider.prototype.requestReadPermission = /**
     * Método responsável por requisitar permissão para acesso a galeria
     * Indicação se possui ou não permissão
     * @return {?}
     */
    function () {
        return __awaiter(this, void 0, void 0, function () {
            var hasPermission, permission, permissionAfterRequest, error_2;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        hasPermission = false;
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 7, , 8]);
                        return [4 /*yield*/, this.imagePicker.hasReadPermission()];
                    case 2:
                        permission = _a.sent();
                        if (!!permission) return [3 /*break*/, 5];
                        return [4 /*yield*/, this.imagePicker.requestReadPermission()];
                    case 3:
                        _a.sent();
                        return [4 /*yield*/, this.imagePicker.hasReadPermission()];
                    case 4:
                        permissionAfterRequest = _a.sent();
                        hasPermission = permissionAfterRequest;
                        return [3 /*break*/, 6];
                    case 5:
                        hasPermission = true;
                        _a.label = 6;
                    case 6: return [3 /*break*/, 8];
                    case 7:
                        error_2 = _a.sent();
                        hasPermission = false;
                        return [3 /*break*/, 8];
                    case 8: return [2 /*return*/, hasPermission];
                }
            });
        });
    };
    ImagePickerUtilProvider.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    ImagePickerUtilProvider.ctorParameters = function () { return [
        { type: Platform },
        { type: ImagePicker },
        { type: GlobalService },
        { type: Events }
    ]; };
    return ImagePickerUtilProvider;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Módulo que contém um provedor utilitário que auxilia na
 * utilização do plugin nativo de galeria
 */
var ImagePickerUtilModule = /** @class */ (function () {
    function ImagePickerUtilModule() {
    }
    ImagePickerUtilModule.decorators = [
        { type: NgModule, args: [{
                    providers: [
                        ImagePicker,
                        ImagePickerUtilProvider
                    ]
                },] }
    ];
    return ImagePickerUtilModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { ImagePickerUtilEvents, ImagePickerUtilModule, ImagePickerUtilProvider };

//# sourceMappingURL=totvs-log-mobile-foundation-src-image-picker-util.js.map