import { GlobalService } from 'totvs-log-mobile-foundation';
import { QRScanner } from '@ionic-native/qr-scanner/ngx';
import { BaseMessage } from 'totvs-log-base-foundation';
import { Subject } from 'rxjs';
import { Injectable, NgModule } from '@angular/core';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// TODO - Verificar se é válido ter classes abstratas para os utilitários
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de leitura de QR Code
 * com métodos que facilitam a solicitação de permissão e leitura do código
 */
var QrScannerUtilProvider = /** @class */ (function () {
    /**
     * @ignore
     * Método responsável por construir o serviço
     * qrScanner Serviço do plugin nativo de leitura do QR-Code
     * global Serviço global da aplicação
     */
    function QrScannerUtilProvider(qrScanner, global) {
        this.qrScanner = qrScanner;
        this.global = global;
        /**
         * Propriedade que possui informações sobre o QR Code Scanner após ele estar
         * preparado, indicando assim se é possível ligar o flash ou trocar de camera,
         * por exemplo.
         */
        this.qrScannerLastStatus = new Subject();
    }
    /**
     * Método responsável por preparar o plugin para leitura do QR-Code
     * Indicação da preparação
     */
    /**
     * Método responsável por preparar o plugin para leitura do QR-Code
     * Indicação da preparação
     * @return {?}
     */
    QrScannerUtilProvider.prototype.prepare = /**
     * Método responsável por preparar o plugin para leitura do QR-Code
     * Indicação da preparação
     * @return {?}
     */
    function () {
        var _this = this;
        return new Promise((/**
         * @param {?} resolve
         * @return {?}
         */
        function (resolve) {
            _this.qrScanner.prepare()
                .then((/**
             * @param {?} status
             * @return {?}
             */
            function (status) {
                _this.qrScannerLastStatus.next(status);
                if (status.authorized) {
                    resolve(true);
                }
                else {
                    _this.global.msg.showMessage(new BaseMessage({
                        literal: 'permissaoNaoConcedidaParaORecursoMsg'
                    }));
                    resolve(false);
                }
            }))
                .catch((/**
             * @param {?} error
             * @return {?}
             */
            function (error) {
                if (error && error.hasOwnProperty('name') && error.name === 'CAMERA_ACCESS_DENIED') {
                    _this.global.msg.showMessage(new BaseMessage({
                        literal: 'permissaoNaoConcedidaParaORecursoMsg'
                    }));
                }
                else {
                    _this.global.msg.showMessage(new BaseMessage({
                        literal: 'recursoNaoEstaDisponivelMsg'
                    }));
                }
                resolve(false);
            }));
        }));
    };
    /**
     * Método responsável por apresentar o leitor do QR-Code
     * String com o conteúdo lido
     */
    /**
     * Método responsável por apresentar o leitor do QR-Code
     * String com o conteúdo lido
     * @return {?}
     */
    QrScannerUtilProvider.prototype.show = /**
     * Método responsável por apresentar o leitor do QR-Code
     * String com o conteúdo lido
     * @return {?}
     */
    function () {
        var _this = this;
        return new Promise((/**
         * @param {?} resolve
         * @param {?} reject
         * @return {?}
         */
        function (resolve, reject) {
            if (!_this.qrScannerLastStatus) {
                reject(undefined);
            }
            _this.scanSub = _this.qrScanner.scan().subscribe((/**
             * @param {?} result
             * @return {?}
             */
            function (result) {
                _this.qrScanner.hide();
                _this.scanSub.unsubscribe(); // stop scanning
                resolve(result);
            }), (/**
             * @param {?} error
             * @return {?}
             */
            function (error) {
                reject(error);
            }));
            _this.qrScanner.show()
                .then((/**
             * @return {?}
             */
            function () { }))
                .catch((/**
             * @param {?} error
             * @return {?}
             */
            function (error) { return reject(error); }));
        }));
    };
    /**
     * Método responsável por esconder a camera da leitura do QR-Code
     */
    /**
     * Método responsável por esconder a camera da leitura do QR-Code
     * @return {?}
     */
    QrScannerUtilProvider.prototype.hide = /**
     * Método responsável por esconder a camera da leitura do QR-Code
     * @return {?}
     */
    function () {
        var _this = this;
        this.qrScanner.getStatus().then((/**
         * @param {?} status
         * @return {?}
         */
        function (status) {
            if (status && status.showing) {
                _this.qrScanner.hide().then((/**
                 * @param {?} statusAfter
                 * @return {?}
                 */
                function (statusAfter) {
                    _this.qrScannerLastStatus.next(statusAfter);
                }));
            }
            if (status && status.scanning) {
                _this.scanSub.unsubscribe();
            }
        }));
    };
    /**
     * Método responsável por ligar ou desligar o flash
     */
    /**
     * Método responsável por ligar ou desligar o flash
     * @return {?}
     */
    QrScannerUtilProvider.prototype.flash = /**
     * Método responsável por ligar ou desligar o flash
     * @return {?}
     */
    function () {
        var _this = this;
        this.qrScanner.getStatus().then((/**
         * @param {?} status
         * @return {?}
         */
        function (status) {
            if (status && status.scanning && status.canEnableLight) {
                if (status.lightEnabled) {
                    _this.qrScanner.disableLight()
                        .then((/**
                     * @param {?} statusAfter
                     * @return {?}
                     */
                    function (statusAfter) {
                        _this.qrScannerLastStatus.next(statusAfter);
                    }))
                        .catch((/**
                     * @return {?}
                     */
                    function () { }));
                }
                else {
                    _this.qrScanner.enableLight()
                        .then((/**
                     * @param {?} statusAfter
                     * @return {?}
                     */
                    function (statusAfter) {
                        _this.qrScannerLastStatus.next(statusAfter);
                    }))
                        .catch((/**
                     * @return {?}
                     */
                    function () { }));
                }
            }
        })).catch((/**
         * @return {?}
         */
        function () { }));
    };
    /**
     * Método responsável por alternar entre a camera frontal e a camera traseira
     */
    /**
     * Método responsável por alternar entre a camera frontal e a camera traseira
     * @return {?}
     */
    QrScannerUtilProvider.prototype.changeCamera = /**
     * Método responsável por alternar entre a camera frontal e a camera traseira
     * @return {?}
     */
    function () {
        var _this = this;
        this.qrScanner.getStatus().then((/**
         * @param {?} status
         * @return {?}
         */
        function (status) {
            if (status && status.scanning && status.canChangeCamera) {
                if (status.currentCamera === 0) { // Back camera
                    _this.qrScanner.useFrontCamera()
                        .then((/**
                     * @param {?} statusAfter
                     * @return {?}
                     */
                    function (statusAfter) {
                        _this.qrScannerLastStatus.next(statusAfter);
                    }))
                        .catch((/**
                     * @return {?}
                     */
                    function () { }));
                }
                else {
                    _this.qrScanner.useBackCamera()
                        .then((/**
                     * @param {?} statusAfter
                     * @return {?}
                     */
                    function (statusAfter) {
                        _this.qrScannerLastStatus.next(statusAfter);
                    }))
                        .catch((/**
                     * @return {?}
                     */
                    function () { }));
                }
            }
        })).catch((/**
         * @return {?}
         */
        function () { }));
    };
    /**
     * Método responsável por destruir o plugin nativo desabilitando a leitura do QR-Code até
     * que o plugin seja novamente preparado
     */
    /**
     * Método responsável por destruir o plugin nativo desabilitando a leitura do QR-Code até
     * que o plugin seja novamente preparado
     * @return {?}
     */
    QrScannerUtilProvider.prototype.destroy = /**
     * Método responsável por destruir o plugin nativo desabilitando a leitura do QR-Code até
     * que o plugin seja novamente preparado
     * @return {?}
     */
    function () {
        var _this = this;
        this.qrScanner.getStatus().then((/**
         * @param {?} status
         * @return {?}
         */
        function (status) {
            if (status) {
                if (status.lightEnabled) {
                    _this.qrScanner.disableLight()
                        .then((/**
                     * @return {?}
                     */
                    function () { }))
                        .catch((/**
                     * @return {?}
                     */
                    function () { }));
                }
                _this.qrScanner.destroy()
                    .then((/**
                 * @return {?}
                 */
                function () { }))
                    .catch((/**
                 * @return {?}
                 */
                function () { }));
            }
        }));
    };
    QrScannerUtilProvider.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    QrScannerUtilProvider.ctorParameters = function () { return [
        { type: QRScanner },
        { type: GlobalService }
    ]; };
    return QrScannerUtilProvider;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Módulo que contém um provedor utilitário que auxilia na
 * utilização do plugin nativo de leitura de QR-Code
 */
var QRScannerUtilModule = /** @class */ (function () {
    function QRScannerUtilModule() {
    }
    QRScannerUtilModule.decorators = [
        { type: NgModule, args: [{
                    providers: [
                        QRScanner,
                        QrScannerUtilProvider
                    ]
                },] }
    ];
    return QRScannerUtilModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { QRScannerUtilModule, QrScannerUtilProvider };

//# sourceMappingURL=totvs-log-mobile-foundation-src-qr-scanner-util.js.map