import { GlobalService } from 'totvs-log-mobile-foundation';
import { BaseMessage } from 'totvs-log-base-foundation';
import { Platform } from '@ionic/angular';
import { Injectable, NgModule } from '@angular/core';
import { SpeechRecognition } from '@ionic-native/speech-recognition/ngx';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// TODO - Verificar se é válido ter classes abstratas para os utilitários
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de reconhecimento de voz
 * com métodos que facilitam a solicitação de permissão e reconhecimento
 */
var SpeechRecognitionUtilProvider = /** @class */ (function () {
    /**
     * @ignore
     * Método responsável por construir o serviço
     * platform Serviço relacionado a plataforma
     * speechRecognition Serviço do plugin nativo de reconhecimento de voz
     * global Serviço global da aplicação
     */
    function SpeechRecognitionUtilProvider(platform, speechRecognition, global) {
        var _this = this;
        this.speechRecognition = speechRecognition;
        this.global = global;
        /**
         * @ignore
         * Indica se a plataforma é cordova
         */
        this.isCordova = false;
        platform.ready().then((/**
         * @return {?}
         */
        function () { return _this.isCordova = platform.is('cordova'); }));
    }
    // FIXME - Em relação a implementação no IOS, provavelmente será necessário implementar o stopListening()
    /**
     * Método responsável por habilitar o reconheicimento de voz
     * Lista com os textos reconhecidos (geralmente o primeiro item é o mais provável)
     */
    // FIXME - Em relação a implementação no IOS, provavelmente será necessário implementar o stopListening()
    /**
     * Método responsável por habilitar o reconheicimento de voz
     * Lista com os textos reconhecidos (geralmente o primeiro item é o mais provável)
     * @return {?}
     */
    SpeechRecognitionUtilProvider.prototype.listening = 
    // FIXME - Em relação a implementação no IOS, provavelmente será necessário implementar o stopListening()
    /**
     * Método responsável por habilitar o reconheicimento de voz
     * Lista com os textos reconhecidos (geralmente o primeiro item é o mais provável)
     * @return {?}
     */
    function () {
        var _this = this;
        return new Promise((/**
         * @param {?} resolve
         * @param {?} reject
         * @return {?}
         */
        function (resolve, reject) {
            _this.verifyPermission().then((/**
             * @param {?} hasPermission
             * @return {?}
             */
            function (hasPermission) {
                if (hasPermission === false) {
                    _this.showToast('permissaoNaoConcedidaParaORecursoMsg');
                    resolve([]);
                }
                else {
                    _this.speechRecognition.isRecognitionAvailable()
                        .then((/**
                     * @param {?} available
                     * @return {?}
                     */
                    function (available) {
                        if (available === false) {
                            _this.showToast('recursoNaoEstaDisponivelMsg');
                            resolve([]);
                            return;
                        }
                        _this.speechRecognition.startListening(_this.options)
                            .subscribe((/**
                         * @param {?} matches
                         * @return {?}
                         */
                        function (matches) { return resolve(matches); }), (/**
                         * @param {?} onerror
                         * @return {?}
                         */
                        function (onerror) {
                            if (onerror && onerror instanceof Error && onerror.message !== '0') {
                                _this.showToast('erroAoUtilizarRecursoMsg');
                            }
                            reject(onerror);
                        }));
                    })).catch((/**
                     * @param {?} error
                     * @return {?}
                     */
                    function (error) {
                        _this.showToast('recursoNaoEstaDisponivelMsg');
                        reject(error);
                    }));
                }
            })).catch((/**
             * @param {?} error
             * @return {?}
             */
            function (error) {
                reject(error);
            }));
        }));
    };
    /**
     * @ignore
     * Método responsável por verificar se há permissão e se não houver, solicitar
     * Indicação se há ou não permissão
     */
    /**
     * @ignore
     * Método responsável por verificar se há permissão e se não houver, solicitar
     * Indicação se há ou não permissão
     * @private
     * @return {?}
     */
    SpeechRecognitionUtilProvider.prototype.verifyPermission = /**
     * @ignore
     * Método responsável por verificar se há permissão e se não houver, solicitar
     * Indicação se há ou não permissão
     * @private
     * @return {?}
     */
    function () {
        var _this = this;
        return new Promise((/**
         * @param {?} resolve
         * @param {?} reject
         * @return {?}
         */
        function (resolve, reject) {
            if (_this.isCordova === false) {
                _this.showToast('recursoNaoEstaDisponivelMsg');
                reject('Cordova is not available.');
                return;
            }
            _this.speechRecognition.hasPermission().then((/**
             * @param {?} hasPermission
             * @return {?}
             */
            function (hasPermission) {
                if (hasPermission) {
                    resolve(hasPermission);
                }
                else {
                    _this.requestPermission()
                        .then((/**
                     * @param {?} requestPermissionSuccessful
                     * @return {?}
                     */
                    function (requestPermissionSuccessful) { return resolve(requestPermissionSuccessful); }));
                }
            })).catch((/**
             * @return {?}
             */
            function () { return resolve(false); }));
        }));
    };
    /**
     * @ignore
     * Método responsável por solicitar permissão de voz
     * Indicação se a permissão foi ou não concedida
     */
    /**
     * @ignore
     * Método responsável por solicitar permissão de voz
     * Indicação se a permissão foi ou não concedida
     * @private
     * @return {?}
     */
    SpeechRecognitionUtilProvider.prototype.requestPermission = /**
     * @ignore
     * Método responsável por solicitar permissão de voz
     * Indicação se a permissão foi ou não concedida
     * @private
     * @return {?}
     */
    function () {
        var _this = this;
        return new Promise((/**
         * @param {?} resolve
         * @return {?}
         */
        function (resolve) {
            _this.speechRecognition.requestPermission()
                .then((/**
             * @return {?}
             */
            function () { return resolve(true); }), (/**
             * @return {?}
             */
            function () { return resolve(false); })).catch((/**
             * @return {?}
             */
            function () { return resolve(false); }));
        }));
    };
    /**
     * @ignore
     * Método para apresentar mensagens
     * literalMessage Literal da mensagem
     */
    /**
     * @ignore
     * Método para apresentar mensagens
     * literalMessage Literal da mensagem
     * @private
     * @param {?} literalMessage
     * @return {?}
     */
    SpeechRecognitionUtilProvider.prototype.showToast = /**
     * @ignore
     * Método para apresentar mensagens
     * literalMessage Literal da mensagem
     * @private
     * @param {?} literalMessage
     * @return {?}
     */
    function (literalMessage) {
        this.global.msg.showMessage(new BaseMessage({
            literal: literalMessage
        }));
    };
    SpeechRecognitionUtilProvider.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    SpeechRecognitionUtilProvider.ctorParameters = function () { return [
        { type: Platform },
        { type: SpeechRecognition },
        { type: GlobalService }
    ]; };
    return SpeechRecognitionUtilProvider;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Módulo que contém um provedor utilitário que auxilia na
 * utilização do plugin nativo de reconhecimento de voz
 */
var SpeechRecognitionUtilModule = /** @class */ (function () {
    function SpeechRecognitionUtilModule() {
    }
    SpeechRecognitionUtilModule.decorators = [
        { type: NgModule, args: [{
                    providers: [
                        SpeechRecognition,
                        SpeechRecognitionUtilProvider
                    ]
                },] }
    ];
    return SpeechRecognitionUtilModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { SpeechRecognitionUtilModule, SpeechRecognitionUtilProvider };

//# sourceMappingURL=totvs-log-mobile-foundation-src-speech-recognition-util.js.map