import { __awaiter, __generator } from 'tslib';
import { UtilsFn } from 'totvs-log-base-foundation';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe responsável pelo gerenciamento de variável em memória/storage do dispositivo
 * Ao utilizar os métodos da classe para atualizar o valor em memória, por padrão,
 * o valor é refletido no storage
 * @template T
 */
var StorageUtil = /** @class */ (function () {
    /**
     * Construtor da classe
     * storage Recebe o Storage via Angular Dependency Injection
     */
    function StorageUtil(storage) {
        this.storage = storage;
        /**
         * Campo id para facilitar buscas
         */
        this.fieldId = '';
        /**
         * Conteúdo a ser refletido
         */
        this.storageData = {};
        /**
         * Indica que a classe já foi inicializada corretamente
         */
        this.initialized = false;
        /**
         * Se uma operação que trava o registro está em andamento
         */
        this.lock = false;
    }
    Object.defineProperty(StorageUtil.prototype, "id", {
        get: /**
         * @return {?}
         */
        function () {
            return this.fieldId;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StorageUtil.prototype, "exclusive", {
        get: /**
         * @return {?}
         */
        function () {
            return this.exclusiveStorage;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StorageUtil.prototype, "suffixFn", {
        get: /**
         * @return {?}
         */
        function () {
            if (!this.suffixFunction) {
                return (/**
                 * @return {?}
                 */
                function () { return ''; });
            }
            else {
                return this.suffixFunction;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StorageUtil.prototype, "initFn", {
        get: /**
         * @return {?}
         */
        function () {
            return this.initFunc;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StorageUtil.prototype, "name", {
        get: /**
         * @return {?}
         */
        function () {
            return this.storageName;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StorageUtil.prototype, "data", {
        get: /**
         * @return {?}
         */
        function () {
            if (!this.isInit) {
                throw new Error("Storage: " + this.name + " est\u00E1 sendo lido sem ser inicializado.");
            }
            return this.getCurrentSuffixData();
        },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) {
            if (!this.isInit) {
                throw new Error("Storage: " + this.name + " est\u00E1 sendo definido sem ser inicializado.");
            }
            /** @type {?} */
            var suffix = this.currentSuffix;
            this.storageData[suffix] = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StorageUtil.prototype, "currentSuffix", {
        /**
         * Sufixo corrente quando storage exclusivo
         */
        get: /**
         * Sufixo corrente quando storage exclusivo
         * @return {?}
         */
        function () {
            /** @type {?} */
            var suffix = this.suffixFn();
            if (!suffix || suffix === '') {
                suffix = StorageUtil.SUFFIX_DEFAULT;
            }
            return suffix;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StorageUtil.prototype, "isInit", {
        get: /**
         * @return {?}
         */
        function () {
            return this.initialized;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StorageUtil.prototype, "isLock", {
        get: /**
         * @return {?}
         */
        function () {
            return this.lock;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Configura e realiza a leitura de dados do storage
     * { name:string, id? :string, exclusive?: boolean, suffixFn?: () => void, lazy?: boolean, initFn?: () => void}
     */
    /**
     * Configura e realiza a leitura de dados do storage
     * { name:string, id? :string, exclusive?: boolean, suffixFn?: () => void, lazy?: boolean, initFn?: () => void}
     * @param {?=} __0
     * @return {?}
     */
    StorageUtil.prototype.config = /**
     * Configura e realiza a leitura de dados do storage
     * { name:string, id? :string, exclusive?: boolean, suffixFn?: () => void, lazy?: boolean, initFn?: () => void}
     * @param {?=} __0
     * @return {?}
     */
    function (_a) {
        var _b = _a === void 0 ? {
            name: '',
            id: '',
            exclusive: false,
            suffixFn: (/**
             * @return {?}
             */
            function () { return ''; }),
            lazy: false,
            initFn: (/**
             * @return {?}
             */
            function () { return undefined; }),
        } : _a, name = _b.name, id = _b.id, exclusive = _b.exclusive, suffixFn = _b.suffixFn, lazy = _b.lazy, initFn = _b.initFn;
        if (!name || name === '') {
            throw new Error('Storage necessita de um nome (name)');
        }
        if (exclusive && !suffixFn) {
            throw new Error('Storage exclusivo necessita de uma função para retornar o sufixo (suffixFn)');
        }
        this.storageName = name;
        this.fieldId = id;
        this.exclusiveStorage = exclusive;
        this.suffixFunction = suffixFn;
        this.initFunc = initFn;
        if (lazy) {
            return Promise.resolve(undefined);
        }
        else {
            return this.read();
        }
    };
    /**
     * Atualiza o storage para refletir o valor contido em memória
     */
    /**
     * Atualiza o storage para refletir o valor contido em memória
     * @return {?}
     */
    StorageUtil.prototype.updateStorage = /**
     * Atualiza o storage para refletir o valor contido em memória
     * @return {?}
     */
    function () {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.isInit) {
                            throw new Error('Necessário chamar algum dos métodos de leitura (get,read)');
                        }
                        return [4 /*yield*/, UtilsFn.lock((/**
                             * @return {?}
                             */
                            function () { return _this.lock; }))];
                    case 1:
                        _a.sent();
                        this.lock = true;
                        return [4 /*yield*/, this.storage
                                .set(this.name, this.storageData)
                                .then((/**
                             * @return {?}
                             */
                            function () {
                                _this.lock = false;
                                return _this.data;
                            }))
                                .catch((/**
                             * @param {?} err
                             * @return {?}
                             */
                            function (err) {
                                _this.lock = false;
                                console.error(err);
                                return undefined;
                            }))];
                    case 2: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * Ferramenta utilizada para encontrar algum valor em um subnível
     * search
     * fields
     * @example
     * storage.data = [{ nome: 'joao', endereco: {rua: 'qualquer', numero: 123}}]
     * storage.find(123,'endereco','numero').then(endereco =>{
     * endereco.numero = 234
     * storage.updateStorage()})
     */
    /**
     * Ferramenta utilizada para encontrar algum valor em um subnível
     * search
     * fields
     * \@example
     * storage.data = [{ nome: 'joao', endereco: {rua: 'qualquer', numero: 123}}]
     * storage.find(123,'endereco','numero').then(endereco =>{
     * endereco.numero = 234
     * storage.updateStorage()})
     * @param {?} search
     * @param {...?} fields
     * @return {?}
     */
    StorageUtil.prototype.find = /**
     * Ferramenta utilizada para encontrar algum valor em um subnível
     * search
     * fields
     * \@example
     * storage.data = [{ nome: 'joao', endereco: {rua: 'qualquer', numero: 123}}]
     * storage.find(123,'endereco','numero').then(endereco =>{
     * endereco.numero = 234
     * storage.updateStorage()})
     * @param {?} search
     * @param {...?} fields
     * @return {?}
     */
    function (search) {
        var fields = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            fields[_i - 1] = arguments[_i];
        }
        return __awaiter(this, void 0, void 0, function () {
            var array;
            var _this = this;
            return __generator(this, function (_a) {
                array = this.data;
                array.find((/**
                 * @param {?} element
                 * @return {?}
                 */
                function (element) {
                    /** @type {?} */
                    var ret = _this.recursiveField(fields, element) === search;
                    if (ret) {
                        fields.pop();
                        if (fields.length > 0) {
                            return _this.recursiveField(fields, element);
                        }
                        else {
                            return element;
                        }
                    }
                    return ret;
                }));
                throw new Error('Item não encontrado no storage: ' + search);
            });
        });
    };
    /**
     * Atualiza a variavel em memória e reflete no storage.
     * Pode-se acessar um subnível passando no fields o nível que deseja acessar,
     * @example
     * // exemplo 1:
     * storage.data == {campoa:'valor', campob:{ campox: 'valor2'}}
     * storage.set('novovalor',true,'campob','campox')
     * // Resultado objeto: {campoa:'valor', campob:{ campox: 'novovalor'}}
     * // exemplo 2:
     * storage.data == [ {campoa:'valor', campob:{ campox: 'valor2'}}, {campoa:'valor', campob:{ campox: 'valork'}} ]
     * storage.set('novovalorx',true,1,'campob','campox')
     * // Caso atualizar o storage.data diretamente, chamar updateStorage para refletir os dados no storage
     * newData
     * save
     * fields
     */
    /**
     * Atualiza a variavel em memória e reflete no storage.
     * Pode-se acessar um subnível passando no fields o nível que deseja acessar,
     * \@example
     * // exemplo 1:
     * storage.data == {campoa:'valor', campob:{ campox: 'valor2'}}
     * storage.set('novovalor',true,'campob','campox')
     * // Resultado objeto: {campoa:'valor', campob:{ campox: 'novovalor'}}
     * // exemplo 2:
     * storage.data == [ {campoa:'valor', campob:{ campox: 'valor2'}}, {campoa:'valor', campob:{ campox: 'valork'}} ]
     * storage.set('novovalorx',true,1,'campob','campox')
     * // Caso atualizar o storage.data diretamente, chamar updateStorage para refletir os dados no storage
     * newData
     * save
     * fields
     * @param {?} newData
     * @param {?=} save
     * @param {...?} fields
     * @return {?}
     */
    StorageUtil.prototype.set = /**
     * Atualiza a variavel em memória e reflete no storage.
     * Pode-se acessar um subnível passando no fields o nível que deseja acessar,
     * \@example
     * // exemplo 1:
     * storage.data == {campoa:'valor', campob:{ campox: 'valor2'}}
     * storage.set('novovalor',true,'campob','campox')
     * // Resultado objeto: {campoa:'valor', campob:{ campox: 'novovalor'}}
     * // exemplo 2:
     * storage.data == [ {campoa:'valor', campob:{ campox: 'valor2'}}, {campoa:'valor', campob:{ campox: 'valork'}} ]
     * storage.set('novovalorx',true,1,'campob','campox')
     * // Caso atualizar o storage.data diretamente, chamar updateStorage para refletir os dados no storage
     * newData
     * save
     * fields
     * @param {?} newData
     * @param {?=} save
     * @param {...?} fields
     * @return {?}
     */
    function (newData, save) {
        if (save === void 0) { save = true; }
        var fields = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            fields[_i - 2] = arguments[_i];
        }
        return __awaiter(this, void 0, void 0, function () {
            var field, data;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.isInit) {
                            throw new Error('Necessário chamar algum dos métodos de leitura (get,read)');
                        }
                        if (fields.length === 0) {
                            this.data = newData;
                        }
                        else {
                            field = fields.pop();
                            data = this.recursiveField(fields, this.data);
                            data[field] = newData;
                        }
                        if (!save) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.updateStorage()];
                    case 1: return [2 /*return*/, _a.sent()];
                    case 2: return [2 /*return*/, this.data];
                }
            });
        });
    };
    /**
     * Faz a leitura do storage se ainda não foi feita ou da variável de memória
     * search Busca em campo que representa uma chave primária do objeto
     */
    /**
     * Faz a leitura do storage se ainda não foi feita ou da variável de memória
     * search Busca em campo que representa uma chave primária do objeto
     * @param {?} search
     * @return {?}
     */
    StorageUtil.prototype.get = /**
     * Faz a leitura do storage se ainda não foi feita ou da variável de memória
     * search Busca em campo que representa uma chave primária do objeto
     * @param {?} search
     * @return {?}
     */
    function (search) {
        return __awaiter(this, void 0, void 0, function () {
            var array;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.read()];
                    case 1:
                        _a.sent();
                        if (this.id && search) {
                            try {
                                array = this.data;
                                return [2 /*return*/, array.find((/**
                                     * @param {?} element
                                     * @return {?}
                                     */
                                    function (element) {
                                        return element[_this.id] === search;
                                    }))];
                            }
                            catch (error) {
                                throw error;
                            }
                        }
                        else {
                            return [2 /*return*/, this.data];
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Faz a leitura dos dados que estão no storage
     */
    /**
     * Faz a leitura dos dados que estão no storage
     * @return {?}
     */
    StorageUtil.prototype.read = /**
     * Faz a leitura dos dados que estão no storage
     * @return {?}
     */
    function () {
        return __awaiter(this, void 0, void 0, function () {
            var suffix;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.name) {
                            throw new Error('Necessário chamar o método config');
                        }
                        return [4 /*yield*/, UtilsFn.lock((/**
                             * @return {?}
                             */
                            function () { return _this.lock; }))];
                    case 1:
                        _a.sent();
                        suffix = this.currentSuffix;
                        if (!this.storageData.hasOwnProperty(suffix)) return [3 /*break*/, 2];
                        return [2 /*return*/, this.data];
                    case 2:
                        if (!this.isInit) return [3 /*break*/, 3];
                        this.storageData[suffix] = this.init(null);
                        return [3 /*break*/, 5];
                    case 3: return [4 /*yield*/, this.storage
                            .get(this.name)
                            .then((/**
                         * @param {?} value
                         * @return {?}
                         */
                        function (value) {
                            return _this.updateData(value);
                        }))
                            .catch((/**
                         * @param {?} err
                         * @return {?}
                         */
                        function (err) {
                            console.error(err);
                            return undefined;
                        }))];
                    case 4: return [2 /*return*/, _a.sent()];
                    case 5: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Remove um item de um array, atualiza no storage e retorna uma promessa de item atualizado.
     * search
     */
    /**
     * Remove um item de um array, atualiza no storage e retorna uma promessa de item atualizado.
     * search
     * @param {?} search
     * @return {?}
     */
    StorageUtil.prototype.delete = /**
     * Remove um item de um array, atualiza no storage e retorna uma promessa de item atualizado.
     * search
     * @param {?} search
     * @return {?}
     */
    function (search) {
        var _this = this;
        if (!this.isInit) {
            throw new Error('Necessário chamar algum dos métodos de leitura (get,read)');
        }
        if (this.id && search) {
            return new Promise((/**
             * @param {?} resolve
             * @param {?} reject
             * @return {?}
             */
            function (resolve, reject) {
                /** @type {?} */
                var array = _this.data;
                /** @type {?} */
                var index = array.findIndex((/**
                 * @param {?} element
                 * @return {?}
                 */
                function (element) {
                    return element[_this.id] === search;
                }));
                if (index > -1) {
                    array.splice(index, 1);
                    _this.set(array).then((/**
                     * @param {?} v
                     * @return {?}
                     */
                    function (v) {
                        resolve(_this.data);
                    }));
                }
                else {
                    reject("Registro " + search + " n\u00E3o encontrado.");
                }
            }));
        }
    };
    /**
     * Atualiza o valor em memória lida do storage
     * value
     */
    /**
     * Atualiza o valor em memória lida do storage
     * value
     * @private
     * @param {?} value
     * @return {?}
     */
    StorageUtil.prototype.updateData = /**
     * Atualiza o valor em memória lida do storage
     * value
     * @private
     * @param {?} value
     * @return {?}
     */
    function (value) {
        if (value === undefined || value === null || typeof value !== 'object') {
            this.storageData = {};
        }
        else {
            this.storageData = value;
        }
        this.initialized = true;
        return this.data;
    };
    /**
     * Inicializa o armazenamento com base na função que o usuário informou
     */
    /**
     * Inicializa o armazenamento com base na função que o usuário informou
     * @private
     * @param {?} data
     * @return {?}
     */
    StorageUtil.prototype.init = /**
     * Inicializa o armazenamento com base na função que o usuário informou
     * @private
     * @param {?} data
     * @return {?}
     */
    function (data) {
        if ((data === undefined || data === null) && this.initFn) {
            data = this.initFn();
        }
        return data;
    };
    /**
     * Função recursíva para encontrar subníveis automáticamente
     * x array de campos recursivos
     * data a informação
     * level nível de iteração
     */
    /**
     * Função recursíva para encontrar subníveis automáticamente
     * x array de campos recursivos
     * data a informação
     * level nível de iteração
     * @private
     * @param {?} fields
     * @param {?} data
     * @param {?=} level
     * @return {?}
     */
    StorageUtil.prototype.recursiveField = /**
     * Função recursíva para encontrar subníveis automáticamente
     * x array de campos recursivos
     * data a informação
     * level nível de iteração
     * @private
     * @param {?} fields
     * @param {?} data
     * @param {?=} level
     * @return {?}
     */
    function (fields, data, level) {
        if (level === void 0) { level = 0; }
        if (level < fields.length - 1) {
            return this.recursiveField(fields, data[fields[level]], ++level);
        }
        return data[fields[level]];
    };
    /**
     * Recupera os dados do sufíxo atual
     */
    /**
     * Recupera os dados do sufíxo atual
     * @private
     * @return {?}
     */
    StorageUtil.prototype.getCurrentSuffixData = /**
     * Recupera os dados do sufíxo atual
     * @private
     * @return {?}
     */
    function () {
        /** @type {?} */
        var suffix = this.currentSuffix;
        if (this.storageData.hasOwnProperty(suffix) === false) {
            this.storageData[suffix] = this.init(null);
        }
        return this.storageData[suffix];
    };
    /**
     * Sufixo padrão
     */
    StorageUtil.SUFFIX_DEFAULT = '__stDefault0__';
    return StorageUtil;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { StorageUtil };

//# sourceMappingURL=totvs-log-mobile-foundation-src-storage-util.js.map