import { __awaiter, __generator } from 'tslib';
import { forkJoin, of } from 'rxjs';
import { Storage } from '@ionic/storage';
import { Platform, Events } from '@ionic/angular';
import { StorageUtil } from 'totvs-log-mobile-foundation/src/storage-util';
import { delay } from 'rxjs/operators';
import { Network } from '@ionic-native/network/ngx';
import { Injectable, InjectionToken, Inject, Injector, NgModule } from '@angular/core';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe abstrata que todo recurso de sincronização deve implementar,
 * independente do sentido
 * Para registrar o serviço em que a sincronização automática reconheça:
 * export const entidadeToken = new InjectionToken<entidadeProvider>('');
 * export class entidadeProvider extends SyncHandler{}
 * {
 * provide: entidadeToken,
 * useClass: entidadeProvider
 * }
 * @abstract
 */
var  /**
 * Classe abstrata que todo recurso de sincronização deve implementar,
 * independente do sentido
 * Para registrar o serviço em que a sincronização automática reconheça:
 * export const entidadeToken = new InjectionToken<entidadeProvider>('');
 * export class entidadeProvider extends SyncHandler{}
 * {
 * provide: entidadeToken,
 * useClass: entidadeProvider
 * }
 * @abstract
 */
SyncHandler = /** @class */ (function () {
    function SyncHandler() {
    }
    return SyncHandler;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe que contém todos os eventos disponíbilizados pelo serviço gerenciador de
 * sincronismo
 */
var SyncEvents = /** @class */ (function () {
    function SyncEvents() {
    }
    /**
     * Evento que o sincronizador dispara ao terminar um sincronismo.
     * Deve-se filtrar pelo nome da entidade
     */
    SyncEvents.GOT_ENTITY_SYNC_DATA = 'Sync:GotEntitySyncData';
    return SyncEvents;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var SYNC_CONFIG = new InjectionToken('SYNC_CONFIG: ISyncConfig token de configuração da sincronização');
/**
 * Serviço provedor de sincronismo utilizando serviços externos
 */
var SyncProvider = /** @class */ (function () {
    /**
     * Construtor da classe
     * platform Platform do ionic
     * authService Serviço de autenticação
     * storage Serviço de armazenamento do ionic
     * config Token de configuração do serviço
     * network Serviço de rede do ionic
     * events Serviço de eventos do ionic
     * injector Injetor de dependências do angular
     */
    function SyncProvider(platform, storage, config, network, events, injector) {
        var _this = this;
        this.storage = storage;
        this.config = config;
        this.network = network;
        this.events = events;
        this.injector = injector;
        /**
         * @ignore
         */
        this.syncing = false;
        platform.ready().then((/**
         * @return {?}
         */
        function () { return __awaiter(_this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!platform.is('cordova')) return [3 /*break*/, 2];
                        this.syncStorage = new StorageUtil(this.storage);
                        return [4 /*yield*/, this.syncStorage.config({
                                name: 'SyncEntitiesState',
                                exclusive: true,
                                lazy: true,
                                suffixFn: (/**
                                 * @return {?}
                                 */
                                function () { return _this.storageName; }),
                                initFn: (/**
                                 * @return {?}
                                 */
                                function () {
                                    /** @type {?} */
                                    var init = [];
                                    config.entities.forEach((/**
                                     * @param {?} e
                                     * @return {?}
                                     */
                                    function (e) { return init.push(_this.newState(e)); }));
                                    return init;
                                })
                            })];
                    case 1:
                        _a.sent();
                        this.setDefaults();
                        this.subscribeEvents();
                        _a.label = 2;
                    case 2: return [2 /*return*/];
                }
            });
        }); }));
    }
    Object.defineProperty(SyncProvider.prototype, "isSyncing", {
        /**
         * Se a sincronização está acontecendo
         */
        get: /**
         * Se a sincronização está acontecendo
         * @return {?}
         */
        function () {
            return this.syncing;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SyncProvider.prototype, "storageNameFunction", {
        set: /**
         * @param {?} storageNameFunction
         * @return {?}
         */
        function (storageNameFunction) {
            this.storageNameFn = storageNameFunction;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SyncProvider.prototype, "storageName", {
        /**
         * Atributo que retorna o nome do storage
         */
        get: /**
         * Atributo que retorna o nome do storage
         * @return {?}
         */
        function () {
            if (this.storageNameFn) {
                return this.storageNameFn();
            }
            else {
                return 'sync';
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SyncProvider.prototype, "autoWatchEnabled", {
        /**
         * Retorna se o sincronismo está ativo
         */
        get: /**
         * Retorna se o sincronismo está ativo
         * @return {?}
         */
        function () {
            var _this = this;
            if (this.currentNetworkType === 'none') {
                return false;
            }
            if (!this.config.autoWatch) {
                return false;
            }
            if (this.config.autoWatch.enabledOnNetwork === 'any' || this.config.autoWatch.enabledOnNetwork === this.currentNetworkType) {
                return true;
            }
            else {
                /** @type {?} */
                var array = (/** @type {?} */ (this.config.autoWatch.enabledOnNetwork));
                if (array.findIndex((/**
                 * @param {?} x
                 * @return {?}
                 */
                function (x) { return x === 'any' || x === _this.currentNetworkType; })) !== -1) {
                    return true;
                }
            }
            return false;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Dispara o sincronismo de todas as entidades registradas.
     * Consulte isSyncing para saber se a sincronização está em andamento.
     */
    /**
     * Dispara o sincronismo de todas as entidades registradas.
     * Consulte isSyncing para saber se a sincronização está em andamento.
     * @return {?}
     */
    SyncProvider.prototype.syncAll = /**
     * Dispara o sincronismo de todas as entidades registradas.
     * Consulte isSyncing para saber se a sincronização está em andamento.
     * @return {?}
     */
    function () {
        return __awaiter(this, void 0, void 0, function () {
            var error_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(this.syncing === false)) return [3 /*break*/, 4];
                        this.syncing = true;
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, this.syncStorage.read()];
                    case 2:
                        _a.sent();
                        this.syncAllByOrder(this.config.entities);
                        return [3 /*break*/, 4];
                    case 3:
                        error_1 = _a.sent();
                        console.error('Sync autowatch stopped. Reason', error_1);
                        this.syncing = false;
                        this.stopAutoWatch();
                        return [3 /*break*/, 4];
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Função recursiva que dispara a sincronização de entidades conforme ordem recebida
     * entities Lista com todas as entidades
     * order Ordem da execução
     */
    /**
     * Função recursiva que dispara a sincronização de entidades conforme ordem recebida
     * entities Lista com todas as entidades
     * order Ordem da execução
     * @param {?} entities
     * @param {?=} order
     * @return {?}
     */
    SyncProvider.prototype.syncAllByOrder = /**
     * Função recursiva que dispara a sincronização de entidades conforme ordem recebida
     * entities Lista com todas as entidades
     * order Ordem da execução
     * @param {?} entities
     * @param {?=} order
     * @return {?}
     */
    function (entities, order) {
        if (order === void 0) { order = 0; }
        return __awaiter(this, void 0, void 0, function () {
            var entitiesByOrder, result;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        entitiesByOrder = [];
                        entities.forEach((/**
                         * @param {?} e
                         * @return {?}
                         */
                        function (e) {
                            if ((order === 0 && (e.order === undefined || e.order === 0)) ||
                                (order !== 0 && (e.order === order))) {
                                entitiesByOrder.push(e);
                            }
                        }));
                        if (!(entitiesByOrder.length > 0)) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.syncAllEntities(entitiesByOrder)];
                    case 1:
                        result = _a.sent();
                        // Aborta demais sincronizações
                        if (!result) {
                            return [2 /*return*/];
                        }
                        _a.label = 2;
                    case 2:
                        if (entities.some((/**
                         * @param {?} e
                         * @return {?}
                         */
                        function (e) { return e.order > order; }))) {
                            this.syncAllByOrder(entities, order + 1);
                        }
                        else {
                            this.syncing = false;
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Dispara o sincronismo de todas as entidades registradas conforme ordem
     * Publica o evento SyncEvents.GOT_ENTITY_SYNC_DATA da finalização da sincronização da entidade.
     * entities Lista com as entidades que deverão ser sincronizadas
     * Indicação se ocorreu com sucesso
     */
    /**
     * Dispara o sincronismo de todas as entidades registradas conforme ordem
     * Publica o evento SyncEvents.GOT_ENTITY_SYNC_DATA da finalização da sincronização da entidade.
     * entities Lista com as entidades que deverão ser sincronizadas
     * Indicação se ocorreu com sucesso
     * @param {?} entities
     * @return {?}
     */
    SyncProvider.prototype.syncAllEntities = /**
     * Dispara o sincronismo de todas as entidades registradas conforme ordem
     * Publica o evento SyncEvents.GOT_ENTITY_SYNC_DATA da finalização da sincronização da entidade.
     * entities Lista com as entidades que deverão ser sincronizadas
     * Indicação se ocorreu com sucesso
     * @param {?} entities
     * @return {?}
     */
    function (entities) {
        var _this = this;
        return new Promise((/**
         * @param {?} resolve
         * @return {?}
         */
        function (resolve) {
            /** @type {?} */
            var requests = [];
            entities.forEach((/**
             * @param {?} e
             * @return {?}
             */
            function (e) {
                /** @type {?} */
                var provider = (/** @type {?} */ (_this.injector.get(e.providerToken)));
                /** @type {?} */
                var state = _this.getState(e);
                state.isSyncing = true;
                requests.push(provider.getSyncData(state));
            }));
            forkJoin(requests).subscribe((/**
             * @param {?} x
             * @return {?}
             */
            function (x) {
                if (x) {
                    x.forEach((/**
                     * @param {?} e
                     * @return {?}
                     */
                    function (e) {
                        if (e) {
                            _this.syncStorage.data[_this.getStateIdx(e.state.entity)] = e.state;
                        }
                    }));
                    _this.syncStorage.updateStorage()
                        .then((/**
                     * @return {?}
                     */
                    function () {
                        x.forEach((/**
                         * @param {?} e
                         * @return {?}
                         */
                        function (e) {
                            _this.events.publish(SyncEvents.GOT_ENTITY_SYNC_DATA, e);
                        }));
                        resolve(true);
                    }));
                }
                else {
                    _this.syncing = false;
                    resolve(false);
                }
            }), (/**
             * @return {?}
             */
            function () {
                _this.syncing = false;
                resolve(false);
            }));
        }));
    };
    /**
     * Inicia o serviço que executa temporalmente a sincronização de todas as entidades,
     * caso o dispositivo esteja na rede escolhida
     * now Se o serviço deve iniciar imediatamente ou deve aguardar o próximo cíclo
     */
    /**
     * Inicia o serviço que executa temporalmente a sincronização de todas as entidades,
     * caso o dispositivo esteja na rede escolhida
     * now Se o serviço deve iniciar imediatamente ou deve aguardar o próximo cíclo
     * @param {?} now
     * @return {?}
     */
    SyncProvider.prototype.startAutoWatch = /**
     * Inicia o serviço que executa temporalmente a sincronização de todas as entidades,
     * caso o dispositivo esteja na rede escolhida
     * now Se o serviço deve iniciar imediatamente ou deve aguardar o próximo cíclo
     * @param {?} now
     * @return {?}
     */
    function (now) {
        var _this = this;
        this.watchNetwork();
        this.stopAutoWatch();
        if (now) {
            this.nowToken = setTimeout((/**
             * @return {?}
             */
            function () {
                if (_this.autoWatchEnabled) {
                    _this.syncAll();
                }
            }));
            this.intervalToken = setInterval((/**
             * @return {?}
             */
            function () {
                if (_this.autoWatchEnabled) {
                    _this.syncAll();
                }
            }), this.getNextCall());
        }
        else {
            this.scheduleToken = of({}).pipe(delay(this.getNextCall())).subscribe((/**
             * @return {?}
             */
            function () {
                _this.intervalToken = setInterval((/**
                 * @return {?}
                 */
                function () {
                    if (_this.autoWatchEnabled) {
                        _this.syncAll();
                    }
                }), _this.getNextCall());
            }));
        }
    };
    /**
     * Cancela a sincronização automática.
     */
    /**
     * Cancela a sincronização automática.
     * @return {?}
     */
    SyncProvider.prototype.stopAutoWatch = /**
     * Cancela a sincronização automática.
     * @return {?}
     */
    function () {
        // TODO: Implementar uma espera da sincronização em andamento terminar antes de encerrar os timers ativos.
        if (this.nowToken) {
            clearInterval(this.nowToken);
            this.nowToken = undefined;
        }
        if (this.scheduleToken) {
            this.scheduleToken.unsubscribe();
            this.scheduleToken = undefined;
        }
        if (this.intervalToken) {
            clearInterval(this.intervalToken);
            this.intervalToken = undefined;
        }
    };
    /**
     * Permite que seja trocado quais os tipos de rede
     * que a sincronização automática está ativa
     * types
     */
    /**
     * Permite que seja trocado quais os tipos de rede
     * que a sincronização automática está ativa
     * types
     * @param {...?} types
     * @return {?}
     */
    SyncProvider.prototype.changeEnabledAutoWatchNetWorks = /**
     * Permite que seja trocado quais os tipos de rede
     * que a sincronização automática está ativa
     * types
     * @param {...?} types
     * @return {?}
     */
    function () {
        var types = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            types[_i] = arguments[_i];
        }
        this.config.autoWatch.enabledOnNetwork = types;
        this.setDefaults();
    };
    /**
     * Permite que sejam trocadas as configurações de sincronização automática
     * newConfiguration Novas configurações
     */
    /**
     * Permite que sejam trocadas as configurações de sincronização automática
     * newConfiguration Novas configurações
     * @param {?=} newConfiguration
     * @return {?}
     */
    SyncProvider.prototype.changeSyncWatch = /**
     * Permite que sejam trocadas as configurações de sincronização automática
     * newConfiguration Novas configurações
     * @param {?=} newConfiguration
     * @return {?}
     */
    function (newConfiguration) {
        this.config.autoWatch = newConfiguration;
        if (this.config.autoWatch) {
            this.startAutoWatch(true);
        }
    };
    /**
     * Recupera o estado de sincronização da entidade
     * entity a entidade de sincronização
     */
    /**
     * Recupera o estado de sincronização da entidade
     * entity a entidade de sincronização
     * @private
     * @param {?} entity
     * @return {?}
     */
    SyncProvider.prototype.getState = /**
     * Recupera o estado de sincronização da entidade
     * entity a entidade de sincronização
     * @private
     * @param {?} entity
     * @return {?}
     */
    function (entity) {
        /** @type {?} */
        var e = this.syncStorage.data.find((/**
         * @param {?} x
         * @return {?}
         */
        function (x) { return x.entity.name === entity.name; }));
        if (e === undefined) {
            /** @type {?} */
            var newEntity = this.newState(entity);
            this.syncStorage.data.push(newEntity);
            e = newEntity;
        }
        return e;
    };
    /**
     * Recupera o índice do estado de sincronização da entidade no storage
     * entity a entidade de sincronização
     */
    /**
     * Recupera o índice do estado de sincronização da entidade no storage
     * entity a entidade de sincronização
     * @private
     * @param {?} entity
     * @return {?}
     */
    SyncProvider.prototype.getStateIdx = /**
     * Recupera o índice do estado de sincronização da entidade no storage
     * entity a entidade de sincronização
     * @private
     * @param {?} entity
     * @return {?}
     */
    function (entity) {
        /** @type {?} */
        var idx = this.syncStorage.data.findIndex((/**
         * @param {?} x
         * @return {?}
         */
        function (x) { return x.entity.name === entity.name; }));
        if (idx === -1) {
            /** @type {?} */
            var newEntity = this.newState(entity);
            idx = this.syncStorage.data.push(newEntity) - 1;
        }
        return idx;
    };
    /**
     * Cria um novo estado de sincronização para a entidade
     * entity a entidade de sincronização
     */
    /**
     * Cria um novo estado de sincronização para a entidade
     * entity a entidade de sincronização
     * @private
     * @param {?} entity
     * @return {?}
     */
    SyncProvider.prototype.newState = /**
     * Cria um novo estado de sincronização para a entidade
     * entity a entidade de sincronização
     * @private
     * @param {?} entity
     * @return {?}
     */
    function (entity) {
        return { entity: entity, entityVersion: undefined, isSyncing: false };
    };
    /**
     * Calcula o tempo em ms para a próxima chamada da sincronização
     * Quando não configurado, o tempo será um minuto
     */
    /**
     * Calcula o tempo em ms para a próxima chamada da sincronização
     * Quando não configurado, o tempo será um minuto
     * @private
     * @return {?}
     */
    SyncProvider.prototype.getNextCall = /**
     * Calcula o tempo em ms para a próxima chamada da sincronização
     * Quando não configurado, o tempo será um minuto
     * @private
     * @return {?}
     */
    function () {
        if (this.config.autoWatch) {
            /** @type {?} */
            var times = this.config.autoWatch.checkQuantity;
            if (times === undefined || times <= 0 || isNaN(times)) {
                times = 1;
            }
            /** @type {?} */
            var type = this.config.autoWatch.checkType;
            if (type === 'days') {
                return SyncProvider.daysMs * times;
            }
            else if (type === 'hours') {
                return SyncProvider.hoursMs * times;
            }
            else if (type === 'minutes') {
                return SyncProvider.minutesMs * times;
            }
            else if (type === 'seconds') {
                return SyncProvider.secondsMs * times;
            }
            else { // Defaults for new types
                return SyncProvider.minutesMs * times;
            }
        }
        else {
            return SyncProvider.minutesMs;
        }
    };
    /**
     * @ignore
     * Configura os valores padrão, quando não informados
     */
    /**
     * @ignore
     * Configura os valores padrão, quando não informados
     * @private
     * @return {?}
     */
    SyncProvider.prototype.setDefaults = /**
     * @ignore
     * Configura os valores padrão, quando não informados
     * @private
     * @return {?}
     */
    function () {
        if (this.config.autoWatch) {
            if (this.config.autoWatch.enabledOnNetwork instanceof Array) {
                /** @type {?} */
                var array = (/** @type {?} */ (this.config.autoWatch.enabledOnNetwork));
                if (array.length === 0) {
                    this.config.autoWatch.enabledOnNetwork = 'any';
                }
            }
        }
    };
    /**
     * @ignore
     * Assina a todos os eventos do interesse da classe
     */
    /**
     * @ignore
     * Assina a todos os eventos do interesse da classe
     * @private
     * @return {?}
     */
    SyncProvider.prototype.subscribeEvents = /**
     * @ignore
     * Assina a todos os eventos do interesse da classe
     * @private
     * @return {?}
     */
    function () {
        if (this.config.autoWatch) {
            this.currentNetworkType = this.network.type;
            if (this.config.autoWatch.auto) {
                this.startAutoWatch(false);
            }
            this.watchNetwork();
        }
    };
    /**
     * Assina o evento de trocas de rede
     */
    /**
     * Assina o evento de trocas de rede
     * @private
     * @return {?}
     */
    SyncProvider.prototype.watchNetwork = /**
     * Assina o evento de trocas de rede
     * @private
     * @return {?}
     */
    function () {
        var _this = this;
        if (!this.onNetworkChange) {
            this.onNetworkChange = this.network.onchange().subscribe((/**
             * @return {?}
             */
            function () {
                setTimeout((/**
                 * @return {?}
                 */
                function () {
                    _this.currentNetworkType = _this.network.type;
                }), 3000);
            }));
        }
    };
    /**
     * Constante convertendo 1 dia em milisegundos
     */
    SyncProvider.daysMs = 86400000;
    /**
     * Constante convertendo 1 hora em milisegundos
     */
    SyncProvider.hoursMs = 3600000;
    /**
     * Constante convertendo 1 minuto em milisegundos
     */
    SyncProvider.minutesMs = 60000;
    /**
     * Constante convertendo 1 segundo em milisegundos
     */
    SyncProvider.secondsMs = 1000;
    SyncProvider.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    SyncProvider.ctorParameters = function () { return [
        { type: Platform },
        { type: Storage },
        { type: undefined, decorators: [{ type: Inject, args: [SYNC_CONFIG,] }] },
        { type: Network },
        { type: Events },
        { type: Injector }
    ]; };
    return SyncProvider;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Módulo com as dependências para sincronização mobile
 */
var SyncModule = /** @class */ (function () {
    function SyncModule() {
    }
    /**
     * @return {?}
     */
    SyncModule.forRoot = /**
     * @return {?}
     */
    function () {
        return {
            ngModule: SyncModule,
            providers: [
                SyncProvider,
                Network
            ]
        };
    };
    SyncModule.decorators = [
        { type: NgModule, args: [{},] }
    ];
    return SyncModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { SyncHandler, SyncEvents, SyncProvider, SYNC_CONFIG, SyncModule };

//# sourceMappingURL=totvs-log-mobile-foundation-src-sync.js.map