import { TranslateService, TranslateModule, TranslateLoader } from '@ngx-translate/core';
import { TranslateHttpLoader } from '@ngx-translate/http-loader';
import { ReplaySubject } from 'rxjs';
import { map, take } from 'rxjs/operators';
import { CommonModule, DatePipe } from '@angular/common';
import { WebStorageStateStore } from 'oidc-client';
import { InAppBrowser } from '@ionic-native/in-app-browser/ngx';
import { CriticalError, I18nBaseService, AbstractShowMessage, ShowMessageType, ShowMessageLinkActionType, MessageDisplayerService, LoadingDefault, ApiDateTimeService, HttpInterceptorBaseService, MessageUtils, DefaultDateViewConverterService, DateViewConverter, DateTimeModule, TimeViewConverter, DefaultTimeViewConverterService, AuthBaseService, AUTH_SETTINGS, AUTH_TYPE, AuthHandlerType, UtilsFn, AuthInterceptorHandlerService, UserPermissionService } from 'totvs-log-base-foundation';
import { OidcCordovaIabEvents, OidcCordovaIFrameNavigator, OidcCordovaPopupNavigator } from 'totvs-log-mobile-foundation/src/oidc-cordova';
import { HttpClient, HTTP_INTERCEPTORS } from '@angular/common/http';
import { __extends, __awaiter, __generator, __values } from 'tslib';
import { StorageUtil } from 'totvs-log-mobile-foundation/src/storage-util';
import { Storage } from '@ionic/storage';
import { ToastController, AlertController, LoadingController, Events, Platform } from '@ionic/angular';
import { Injectable, Inject, InjectionToken, NgModule, Pipe, APP_INITIALIZER, Injector } from '@angular/core';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Serviço utilizado para carregar as literais
 */
var IntlLiteralsLoaderService = /** @class */ (function () {
    /**
     * Método construtor da classe
     * translate
     */
    function IntlLiteralsLoaderService(translate) {
        this.translate = translate;
        /**
         * Responsável por receber as configurações do serviço de tradução para carregar as literais
         */
        this.mobileOptions = { language: 'pt-br', url: '' };
    }
    /**
     * Método responsável por setar as opções e retornar a literal atualizada
     * options
     */
    /**
     * Método responsável por setar as opções e retornar a literal atualizada
     * options
     * @param {?} options
     * @return {?}
     */
    IntlLiteralsLoaderService.prototype.updateOptions = /**
     * Método responsável por setar as opções e retornar a literal atualizada
     * options
     * @param {?} options
     * @return {?}
     */
    function (options) {
        try {
            Object.assign(this.mobileOptions, options);
            return this.load();
        }
        catch (error) {
            throw new CriticalError(error);
        }
    };
    /**
     * Método responsável por setar o carregamento e retornar a literal atualizada
     */
    /**
     * Método responsável por setar o carregamento e retornar a literal atualizada
     * @return {?}
     */
    IntlLiteralsLoaderService.prototype.load = /**
     * Método responsável por setar o carregamento e retornar a literal atualizada
     * @return {?}
     */
    function () {
        return this.translate.getTranslation(this.mobileOptions.language)
            .pipe(map((/**
         * @param {?} res
         * @return {?}
         */
        function (res) { return res; })));
    };
    IntlLiteralsLoaderService.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    IntlLiteralsLoaderService.ctorParameters = function () { return [
        { type: TranslateService }
    ]; };
    return IntlLiteralsLoaderService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Token de configuração da url de internacionalização
 * @type {?}
 */
var INTL_CONFIG = new InjectionToken('Configurações da internacionalização');
/**
 * Serviço de internacionalização da aplicação
 */
var IntlService = /** @class */ (function (_super) {
    __extends(IntlService, _super);
    /**
     * Método construtor que inicializa a classe
     * translate Serviço de internacionalização
     */
    function IntlService(config, intlLiteralsLoader) {
        var _this = _super.call(this, intlLiteralsLoader) || this;
        /**
         * Atributo de opções de internacionalização
         */
        _this.options = { language: 'pt-br', url: '' };
        /** @type {?} */
        var matchedNavigatorAppLanguage;
        if (config.supportedLanguages) {
            matchedNavigatorAppLanguage = config.supportedLanguages.find((/**
             * @param {?} appSupportedLanguage
             * @return {?}
             */
            function (appSupportedLanguage) {
                return appSupportedLanguage.language === navigator.language;
            }));
        }
        if (!matchedNavigatorAppLanguage) {
            matchedNavigatorAppLanguage = { language: 'pt-br' };
        }
        _this.options.language = matchedNavigatorAppLanguage.language.toLowerCase();
        return _this;
    }
    IntlService.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    IntlService.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: Inject, args: [INTL_CONFIG,] }] },
        { type: IntlLiteralsLoaderService }
    ]; };
    return IntlService;
}(I18nBaseService));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Serviço responsável pela apresentação de mensagens
 */
var ShowMessageService = /** @class */ (function (_super) {
    __extends(ShowMessageService, _super);
    /**
     * Método construtor que inicializa a classe
     * toastCtrl Serviço de TOAST
     * alertCtrl Serviço de Alerta
     * i18n Serviço de internacionalização
     */
    function ShowMessageService(toastCtrl, alertCtrl) {
        var _this = _super.call(this) || this;
        _this.toastCtrl = toastCtrl;
        _this.alertCtrl = alertCtrl;
        return _this;
    }
    /**
     * Método responsável por apresentar uma mensagem
     */
    /**
     * Método responsável por apresentar uma mensagem
     * @param {?} showMessageOptions
     * @return {?}
     */
    ShowMessageService.prototype.displayMessage = /**
     * Método responsável por apresentar uma mensagem
     * @param {?} showMessageOptions
     * @return {?}
     */
    function (showMessageOptions) {
        return __awaiter(this, void 0, void 0, function () {
            var _a;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = showMessageOptions.messageType;
                        switch (_a) {
                            case ShowMessageType.NotificationSuccess: return [3 /*break*/, 1];
                            case ShowMessageType.DialogAlert: return [3 /*break*/, 3];
                            case ShowMessageType.DialogConfirmation: return [3 /*break*/, 3];
                        }
                        return [3 /*break*/, 5];
                    case 1: return [4 /*yield*/, this.showNotification(showMessageOptions)];
                    case 2:
                        _b.sent();
                        return [3 /*break*/, 6];
                    case 3: return [4 /*yield*/, this.showDialog(showMessageOptions)];
                    case 4:
                        _b.sent();
                        return [3 /*break*/, 6];
                    case 5: throw new Error('Tipo de mensagem inválido');
                    case 6: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Método responsável por apresentar uma mensagem de notificação
     */
    /**
     * Método responsável por apresentar uma mensagem de notificação
     * @private
     * @param {?} showMessageOptions
     * @return {?}
     */
    ShowMessageService.prototype.showNotification = /**
     * Método responsável por apresentar uma mensagem de notificação
     * @private
     * @param {?} showMessageOptions
     * @return {?}
     */
    function (showMessageOptions) {
        return __awaiter(this, void 0, void 0, function () {
            var toast;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.toastCtrl.create({
                            message: showMessageOptions.messageText,
                            duration: showMessageOptions.messageLinkActionType ===
                                ShowMessageLinkActionType.No
                                ? 3000
                                : 10000,
                            position: 'middle',
                            showCloseButton: showMessageOptions.messageLinkActionType !==
                                ShowMessageLinkActionType.No,
                            closeButtonText: showMessageOptions.messageLinkLabel
                        })];
                    case 1:
                        toast = _a.sent();
                        toast.onDidDismiss().then((/**
                         * @return {?}
                         */
                        function () {
                            if (showMessageOptions.messageLinkActionType ===
                                ShowMessageLinkActionType.LinkNewTab) {
                                _this.openLink(showMessageOptions.messageLinkAction);
                            }
                            else if (showMessageOptions.messageLinkActionType ===
                                ShowMessageLinkActionType.Method &&
                                showMessageOptions.messageLinkAction) {
                                showMessageOptions.messageLinkAction();
                            }
                        }));
                        return [4 /*yield*/, toast.present()];
                    case 2:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Método responsável por apresentar uma mensagem de diálogo
     */
    /**
     * Método responsável por apresentar uma mensagem de diálogo
     * @private
     * @param {?} showMessageOptions
     * @return {?}
     */
    ShowMessageService.prototype.showDialog = /**
     * Método responsável por apresentar uma mensagem de diálogo
     * @private
     * @param {?} showMessageOptions
     * @return {?}
     */
    function (showMessageOptions) {
        return __awaiter(this, void 0, void 0, function () {
            var buttons, alert;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        buttons = [];
                        if (showMessageOptions.messageCancelLabel &&
                            showMessageOptions.messageCancelLabel.trim() !== '' &&
                            showMessageOptions.messageType === ShowMessageType.DialogConfirmation) {
                            buttons.push({
                                text: showMessageOptions.messageCancelLabel,
                                handler: (/**
                                 * @return {?}
                                 */
                                function () {
                                    alert.dismiss(false);
                                    return false;
                                })
                            });
                        }
                        if (showMessageOptions.messageConfirmLabel &&
                            showMessageOptions.messageConfirmLabel.trim() !== '') {
                            buttons.push({
                                text: showMessageOptions.messageConfirmLabel,
                                handler: (/**
                                 * @return {?}
                                 */
                                function () {
                                    alert.dismiss(true);
                                    return false;
                                })
                            });
                        }
                        return [4 /*yield*/, this.alertCtrl.create({
                                message: showMessageOptions.messageDetails,
                                header: showMessageOptions.messageTitle,
                                subHeader: showMessageOptions.messageText,
                                buttons: buttons
                            })];
                    case 1:
                        alert = _a.sent();
                        alert.onDidDismiss().then((/**
                         * @param {?} data
                         * @return {?}
                         */
                        function (data) {
                            switch (data) {
                                case true:
                                    if (showMessageOptions.messageConfirmAction) {
                                        showMessageOptions.messageConfirmAction();
                                    }
                                    break;
                                default:
                                    if (showMessageOptions.messageCancelAction) {
                                        showMessageOptions.messageCancelAction();
                                    }
                            }
                        }));
                        return [4 /*yield*/, alert.present()];
                    case 2:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Método responsável pela abertura de um link
     */
    /**
     * Método responsável pela abertura de um link
     * @private
     * @param {?} link
     * @return {?}
     */
    ShowMessageService.prototype.openLink = /**
     * Método responsável pela abertura de um link
     * @private
     * @param {?} link
     * @return {?}
     */
    function (link) {
        link = link.indexOf('http://') === -1 ? "http://" + link : link;
        window.open(link, '_blank');
    };
    ShowMessageService.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    ShowMessageService.ctorParameters = function () { return [
        { type: ToastController },
        { type: AlertController }
    ]; };
    return ShowMessageService;
}(AbstractShowMessage));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ShowMessageModule = /** @class */ (function () {
    function ShowMessageModule() {
    }
    ShowMessageModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [],
                    imports: [],
                    exports: [],
                    providers: [
                        ShowMessageService,
                        MessageDisplayerService,
                        {
                            provide: AbstractShowMessage,
                            useClass: ShowMessageService
                        }
                    ]
                },] }
    ];
    return ShowMessageModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Pipe de internacionalização da aplicação
 */
var I18NPipe = /** @class */ (function () {
    /**
     * Método construtor que inicializa a classe
     * i18nService Serviço de internacionalização
     */
    function I18NPipe(i18nService) {
        this.i18nService = i18nService;
    }
    /**
     * Método responsável pela transformação da string
     * query
     * data
     */
    /**
     * Método responsável pela transformação da string
     * query
     * data
     * @param {?} query
     * @param {?} data
     * @return {?}
     */
    I18NPipe.prototype.transform = /**
     * Método responsável pela transformação da string
     * query
     * data
     * @param {?} query
     * @param {?} data
     * @return {?}
     */
    function (query, data) {
        if (!query || query.length === 0) {
            return '';
        }
        /** @type {?} */
        var ret = this.cached(query, data);
        if (ret) {
            return ret;
        }
        if (!data) {
            data = [''];
        }
        else if (!Array.isArray(data)) {
            data = [data];
        }
        return this.i18nService.format(data, query);
    };
    /**
     * Método responsável por retornar o que está em cache
     * query
     * args
     */
    /**
     * Método responsável por retornar o que está em cache
     * query
     * args
     * @private
     * @param {?} query
     * @param {...?} args
     * @return {?}
     */
    I18NPipe.prototype.cached = /**
     * Método responsável por retornar o que está em cache
     * query
     * args
     * @private
     * @param {?} query
     * @param {...?} args
     * @return {?}
     */
    function (query) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        return '';
    };
    I18NPipe.decorators = [
        { type: Pipe, args: [{ name: 'format' },] }
    ];
    /** @nocollapse */
    I18NPipe.ctorParameters = function () { return [
        { type: I18nBaseService }
    ]; };
    return I18NPipe;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var i18nFactory = (/**
 * @param {?} i18n
 * @param {?} config
 * @return {?}
 */
function (i18n, config) {
    return (/**
     * @return {?}
     */
    function () { return new Promise((/**
     * @param {?} resolve
     * @return {?}
     */
    function (resolve) {
        if (config.lazy) {
            resolve(true);
        }
        else {
            i18n.updateOptions({}).subscribe((/**
             * @return {?}
             */
            function () {
                resolve(true);
            }), (/**
             * @param {?} error
             * @return {?}
             */
            function (error) {
                console.error(error);
                resolve(false);
            }));
        }
    })); });
});

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Função responsável por indicar o diretório que contém os arquivos de tradução
 * http
 * @param {?} http
 * @return {?}
 */
function createTranslateLoader(http) {
    return new TranslateHttpLoader(http, './assets/i18n/', '.json');
}
var ɵ0 = i18nFactory;
/**
 * Módulo responsável pela configuração da internacionalização da aplicação
 */
var IntlModule = /** @class */ (function () {
    function IntlModule() {
    }
    IntlModule.decorators = [
        { type: NgModule, args: [{
                    imports: [
                        TranslateModule.forRoot({
                            loader: {
                                provide: TranslateLoader,
                                useFactory: createTranslateLoader,
                                deps: [HttpClient],
                            },
                        }),
                    ],
                    declarations: [I18NPipe],
                    exports: [TranslateModule, I18NPipe],
                    providers: [
                        IntlLiteralsLoaderService,
                        {
                            provide: I18nBaseService,
                            useClass: IntlService,
                        },
                        {
                            provide: APP_INITIALIZER,
                            useFactory: ɵ0,
                            deps: [I18nBaseService, INTL_CONFIG],
                            multi: true,
                        },
                    ],
                },] }
    ];
    return IntlModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Serviço responsável pela apresentação de loading
 */
var ShowLoadingService = /** @class */ (function (_super) {
    __extends(ShowLoadingService, _super);
    /**
     * Método construtor que inicializa a classe
     * i18n Serviço de internacionalização
     * loadingController Serviço de loading
     */
    function ShowLoadingService(i18n, loadingController) {
        var _this = _super.call(this) || this;
        _this.i18n = i18n;
        _this.loadingController = loadingController;
        _this.loadingActived = new ReplaySubject(1);
        _this.thfContentTemplate = '<span class="thf-loading-icon"></span>{content}';
        return _this;
    }
    /**
     * @return {?}
     */
    ShowLoadingService.prototype.showLoading = /**
     * @return {?}
     */
    function () {
        var _this = this;
        if (!this.active) {
            this.active = true;
            /** @type {?} */
            var content = '';
            if (this.loadingText) {
                content = this.loadingText.text;
            }
            this.loadingController
                .create({
                message: this.thfContentTemplate.replace('{content}', content),
                spinner: null,
            })
                .then((/**
             * @param {?} loading
             * @return {?}
             */
            function (loading) {
                _this.loading = loading;
                _this.loading.present().then((/**
                 * @return {?}
                 */
                function () {
                    _this.loadingActived.next();
                }));
            }));
        }
        return this.loadingActived.pipe(take(1));
    };
    /**
     * @return {?}
     */
    ShowLoadingService.prototype.hideLoading = /**
     * @return {?}
     */
    function () {
        var _this = this;
        this.loadingActived.pipe(take(1)).subscribe((/**
         * @return {?}
         */
        function () {
            _this.active = false;
            _this.loadingText = undefined;
            _this.loading.dismiss();
        }));
    };
    /**
     * @param {?} loadingText
     * @return {?}
     */
    ShowLoadingService.prototype.setLoadingText = /**
     * @param {?} loadingText
     * @return {?}
     */
    function (loadingText) {
        var _this = this;
        this.loadingActived.pipe(take(1)).subscribe((/**
         * @return {?}
         */
        function () {
            setTimeout((/**
             * @return {?}
             */
            function () {
                /** @type {?} */
                var baseMessage = _this.i18n.text(loadingText);
                if (_this.loading) {
                    _this.loading.message = _this.thfContentTemplate.replace('{content}', baseMessage.text);
                    // this.loading.forceUpdate();
                }
                _this.loadingText = baseMessage;
            }), 33);
        }));
    };
    ShowLoadingService.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    ShowLoadingService.ctorParameters = function () { return [
        { type: I18nBaseService },
        { type: LoadingController }
    ]; };
    return ShowLoadingService;
}(LoadingDefault));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Serviço responsável por fornecer atributos e métodos que
 * poderão ser utilizados em toda a aplicação
 */
var GlobalService = /** @class */ (function () {
    /**
     * Método construtor que inicializa o serviço
     * i18n Serviço de internacionalização
     * msg Serviço de mensageria para usuário
     * date Serviço de formatação de data
     */
    function GlobalService(i18n, msg, loading, date) {
        this.i18n = i18n;
        this.msg = msg;
        this.loading = loading;
        this.date = date;
    }
    GlobalService.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    GlobalService.ctorParameters = function () { return [
        { type: I18nBaseService },
        { type: MessageDisplayerService },
        { type: ShowLoadingService },
        { type: ApiDateTimeService }
    ]; };
    return GlobalService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Serviço utilizado para realizar o tratamento das respostas
 */
var DefaultHttpInterceptorService = /** @class */ (function (_super) {
    __extends(DefaultHttpInterceptorService, _super);
    /**
     * Método construtor da classe
     * inj
     */
    function DefaultHttpInterceptorService(inj) {
        var _this = _super.call(this) || this;
        _this.inj = inj;
        return _this;
    }
    /**
     * Método responsável por realizar o tratamento de respostas de sucesso
     * response
     */
    /**
     * Método responsável por realizar o tratamento de respostas de sucesso
     * response
     * @param {?} response
     * @return {?}
     */
    DefaultHttpInterceptorService.prototype.interceptResponse = /**
     * Método responsável por realizar o tratamento de respostas de sucesso
     * response
     * @param {?} response
     * @return {?}
     */
    function (response) {
        var e_1, _a;
        this.msg = this.inj.get(MessageDisplayerService);
        if (response.body && response.body.totvs_messages) {
            if (response.body.totvs_messages instanceof Array) {
                /** @type {?} */
                var messages = response.body.totvs_messages;
                try {
                    for (var messages_1 = __values(messages), messages_1_1 = messages_1.next(); !messages_1_1.done; messages_1_1 = messages_1.next()) {
                        var message = messages_1_1.value;
                        this.showTotvsMessage(message);
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (messages_1_1 && !messages_1_1.done && (_a = messages_1.return)) _a.call(messages_1);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
            }
            else {
                this.showTotvsMessage((/** @type {?} */ (response.body.totvs_messages)));
            }
        }
    };
    /**
     * Método responsável por realizar o tratamento de respostas de erro
     * response
     * request
     */
    /**
     * Método responsável por realizar o tratamento de respostas de erro
     * response
     * request
     * @param {?} response
     * @param {?} request
     * @return {?}
     */
    DefaultHttpInterceptorService.prototype.interceptError = /**
     * Método responsável por realizar o tratamento de respostas de erro
     * response
     * request
     * @param {?} response
     * @param {?} request
     * @return {?}
     */
    function (response, request) {
        /** @type {?} */
        var error = (/** @type {?} */ (response.error));
        if (error && error.message && !HttpInterceptorBaseService.isErrorNotificationDisabled(request)) {
            /** @type {?} */
            var title = HttpInterceptorBaseService.titleFromTotvsError(error);
            /** @type {?} */
            var detail = HttpInterceptorBaseService.detailFromTotvsError(error);
            this.showTotvsMessage({
                type: 'error',
                code: title,
                detail: detail
            });
        }
    };
    /**
     * Método responsável por mostrar a mensagem
     * message
     */
    /**
     * Método responsável por mostrar a mensagem
     * message
     * @param {?} message
     * @return {?}
     */
    DefaultHttpInterceptorService.prototype.showTotvsMessage = /**
     * Método responsável por mostrar a mensagem
     * message
     * @param {?} message
     * @return {?}
     */
    function (message) {
        /** @type {?} */
        var messageType = MessageUtils.showMessageTypeFromTotvsMessage(message);
        this.msg.displayMessage({
            messageType: messageType,
            messageText: message.code,
            messageDetails: message.detail
        });
    };
    /**
     * Método responsável por realizar o tratamento da requisição/retorno
     * req
     * next
     */
    /**
     * Método responsável por realizar o tratamento da requisição/retorno
     * req
     * next
     * @param {?} req
     * @param {?} next
     * @return {?}
     */
    DefaultHttpInterceptorService.prototype.intercept = /**
     * Método responsável por realizar o tratamento da requisição/retorno
     * req
     * next
     * @param {?} req
     * @param {?} next
     * @return {?}
     */
    function (req, next) {
        return _super.prototype.intercept.call(this, req, next);
    };
    DefaultHttpInterceptorService.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    DefaultHttpInterceptorService.ctorParameters = function () { return [
        { type: Injector }
    ]; };
    return DefaultHttpInterceptorService;
}(HttpInterceptorBaseService));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var InterceptorModule = /** @class */ (function () {
    function InterceptorModule() {
    }
    InterceptorModule.decorators = [
        { type: NgModule, args: [{
                    imports: [],
                    exports: [],
                    providers: [
                        DefaultHttpInterceptorService,
                        {
                            provide: HTTP_INTERCEPTORS,
                            useClass: DefaultHttpInterceptorService,
                            multi: true
                        }
                    ]
                },] }
    ];
    return InterceptorModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var DateTimeModule$1 = /** @class */ (function () {
    function DateTimeModule$$1() {
    }
    DateTimeModule$$1.decorators = [
        { type: NgModule, args: [{
                    declarations: [],
                    imports: [
                        CommonModule,
                        DateTimeModule
                    ],
                    exports: [],
                    providers: [
                        {
                            provide: DateViewConverter,
                            useClass: DefaultDateViewConverterService
                        },
                        {
                            provide: TimeViewConverter,
                            useClass: DefaultTimeViewConverterService
                        },
                        DatePipe,
                        DefaultDateViewConverterService,
                        DefaultTimeViewConverterService,
                        ApiDateTimeService
                    ]
                },] }
    ];
    return DateTimeModule$$1;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ShowLoadingModule = /** @class */ (function () {
    function ShowLoadingModule() {
    }
    ShowLoadingModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [],
                    imports: [],
                    exports: [],
                    providers: [
                        LoadingController,
                        ShowLoadingService
                    ]
                },] }
    ];
    return ShowLoadingModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FndModule = /** @class */ (function () {
    function FndModule() {
    }
    /**
     * @param {?} config
     * @return {?}
     */
    FndModule.forRoot = /**
     * @param {?} config
     * @return {?}
     */
    function (config) {
        return {
            ngModule: FndModule,
            providers: [GlobalService, { provide: INTL_CONFIG, useValue: config }],
        };
    };
    FndModule.decorators = [
        { type: NgModule, args: [{
                    imports: [
                        IntlModule,
                        ShowMessageModule,
                        InterceptorModule,
                        DateTimeModule$1,
                        ShowLoadingModule,
                    ],
                    declarations: [],
                    exports: [
                        IntlModule,
                        ShowMessageModule,
                        InterceptorModule,
                        DateTimeModule$1,
                        ShowLoadingModule,
                    ],
                },] }
    ];
    return FndModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var AuthService = /** @class */ (function (_super) {
    __extends(AuthService, _super);
    /**
     * Método construtor da classe
     * authSettings Token de injeção com as configurações de autenticação
     * authType Token de injeção com o tipo de autenticação
     */
    function AuthService(authSettings, authType, events) {
        var _this = _super.call(this, authSettings) || this;
        _this.events = events;
        _this.authType = authType;
        return _this;
    }
    /**
     * Método responsável executar o processo de login/autenticação
     */
    /**
     * Método responsável executar o processo de login/autenticação
     * @return {?}
     */
    AuthService.prototype.loginFlow = /**
     * Método responsável executar o processo de login/autenticação
     * @return {?}
     */
    function () {
        return __awaiter(this, void 0, void 0, function () {
            var loaded_1, doneFn_1, loadedFn_1, failedFn_1, doneProm_1, error_1;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(this.authType.type === AuthHandlerType.oauth2)) return [3 /*break*/, 4];
                        doneProm_1 = new Promise((/**
                         * @param {?} resolve
                         * @param {?} reject
                         * @return {?}
                         */
                        function (resolve, reject) {
                            doneFn_1 = resolve;
                        }));
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        loadedFn_1 = (/**
                         * @return {?}
                         */
                        function () {
                            console.log('loadedFn');
                            loaded_1 = true;
                            UtilsFn.timeout(doneProm_1, 1000);
                            _this.events.unsubscribe(OidcCordovaIabEvents.LoadSuccessful, loadedFn_1);
                        });
                        this.events.subscribe(OidcCordovaIabEvents.LoadSuccessful, loadedFn_1);
                        failedFn_1 = (/**
                         * @return {?}
                         */
                        function () {
                            console.log('failedFn');
                            loaded_1 = false;
                            UtilsFn.timeout(doneProm_1, 1000);
                            _this.events.unsubscribe(OidcCordovaIabEvents.LoadFailed, failedFn_1);
                        });
                        this.events.subscribe(OidcCordovaIabEvents.LoadFailed, failedFn_1);
                        return [4 /*yield*/, this.manager.signinPopup()];
                    case 2:
                        _a.sent();
                        doneFn_1();
                        return [3 /*break*/, 4];
                    case 3:
                        error_1 = _a.sent();
                        console.log('catch');
                        doneFn_1();
                        this.events.unsubscribe(OidcCordovaIabEvents.LoadFailed, failedFn_1);
                        this.events.unsubscribe(OidcCordovaIabEvents.LoadSuccessful, loadedFn_1);
                        console.log('error', error_1);
                        if (!loaded_1 || error_1) {
                            throw error_1;
                        }
                        return [3 /*break*/, 4];
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Método responsável executar o processo de logout
     */
    /**
     * Método responsável executar o processo de logout
     * @return {?}
     */
    AuthService.prototype.logoutFlow = /**
     * Método responsável executar o processo de logout
     * @return {?}
     */
    function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(this.authType.type === AuthHandlerType.oauth2)) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.manager.signoutPopup()];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2: return [2 /*return*/];
                }
            });
        });
    };
    AuthService.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    AuthService.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: Inject, args: [AUTH_SETTINGS,] }] },
        { type: undefined, decorators: [{ type: Inject, args: [AUTH_TYPE,] }] },
        { type: Events }
    ]; };
    return AuthService;
}(AuthBaseService));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?} iab
 * @param {?} events
 * @param {?} authSettings
 * @return {?}
 */
function authFactory(iab, events, authSettings) {
    return Object.assign(authSettings, {
        popupNavigator: new OidcCordovaPopupNavigator(iab, events),
        iframeNavigator: new OidcCordovaIFrameNavigator(iab),
        checkSessionInterval: authSettings.checkSessionInterval || 30000,
        userStore: new WebStorageStateStore({ store: window.localStorage }),
    });
}
/** @type {?} */
var AUTH_FACTORY = new InjectionToken('');
/**
 * Módulo que exporta e ativa as dependências para utilizar autenticação com fluxo oauth2
 * Para garantir que os serviços tenham uma única instância, usar AuthModule.forRoot() ao importar
 * @export
 * FndModule
 */
var AuthModule = /** @class */ (function () {
    function AuthModule() {
    }
    /**
     * @dynamic
     */
    /**
     * \@dynamic
     * @param {?} authSettings
     * @param {?} handlerTypeSettings
     * @return {?}
     */
    AuthModule.forRoot = /**
     * \@dynamic
     * @param {?} authSettings
     * @param {?} handlerTypeSettings
     * @return {?}
     */
    function (authSettings, handlerTypeSettings) {
        return {
            ngModule: AuthModule,
            providers: [
                InAppBrowser,
                UserPermissionService,
                {
                    provide: AuthBaseService,
                    useClass: AuthService,
                },
                {
                    provide: AUTH_FACTORY,
                    useValue: authSettings
                },
                {
                    provide: AUTH_SETTINGS,
                    useFactory: authFactory,
                    deps: [InAppBrowser, Events, AUTH_FACTORY],
                },
                { provide: AUTH_TYPE, useValue: handlerTypeSettings },
            ],
        };
    };
    AuthModule.decorators = [
        { type: NgModule, args: [{
                    providers: [
                        {
                            provide: HTTP_INTERCEPTORS,
                            useClass: AuthInterceptorHandlerService,
                            multi: true,
                        },
                    ],
                },] }
    ];
    return AuthModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe que contém todos os eventos relacionados ao Provider de Tenant
 */
var MultiTenantEvents = /** @class */ (function () {
    function MultiTenantEvents() {
    }
    /**
     * Evento disparado quando é realizado uma mudança do tenant corrente e este é diferente do anterior
     */
    MultiTenantEvents.CURRENT_TENANT_CHANGED = 'CurrentTenant:Changed';
    return MultiTenantEvents;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var MultiTenantService = /** @class */ (function () {
    /**
     * Método construtor responsável por inicializar o serviço
     * @param platform Serviço relacionado a plataforma
     * @param storage Serviço relacionado ao storage
     */
    function MultiTenantService(platform, storage, events) {
        var _this = this;
        this.storage = storage;
        this.events = events;
        /**
         * @ignore
         * Indica se a plataforma é cordova
         */
        this.isCordova = false;
        this.ready = new Promise((/**
         * @param {?} resolve
         * @return {?}
         */
        function (resolve) {
            _this.readyFn = resolve;
        }));
        platform.ready().then((/**
         * @return {?}
         */
        function () {
            _this.isCordova = platform.is('cordova');
            if (_this.isCordova) {
                _this.tenantsStorage = new StorageUtil(_this.storage);
                _this.tenantsStorage
                    .config({
                    name: 'Tenants',
                    id: 'shortNameTenant',
                    exclusive: false,
                    lazy: false,
                    initFn: (/**
                     * @return {?}
                     */
                    function () { return []; }),
                })
                    .then((/**
                 * @return {?}
                 */
                function () { }));
                _this.currentTenantStorage = new StorageUtil(_this.storage);
                _this.currentTenantStorage
                    .config({
                    name: 'CurrentTenant',
                    id: 'shortNameTenant',
                    exclusive: false,
                    lazy: true,
                    initFn: (/**
                     * @return {?}
                     */
                    function () { return undefined; }),
                })
                    .then((/**
                 * @return {?}
                 */
                function () { }));
                _this.readyFn();
            }
        }));
    }
    Object.defineProperty(MultiTenantService.prototype, "tenants", {
        get: /**
         * @return {?}
         */
        function () {
            return this.tenantsStorage ? this.tenantsStorage.data : [];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MultiTenantService.prototype, "isReady", {
        get: /**
         * @return {?}
         */
        function () {
            return this.ready;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    MultiTenantService.prototype.getCurrentTenant = /**
     * @return {?}
     */
    function () {
        return __awaiter(this, void 0, void 0, function () {
            var currentTenant, error_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.ready];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, this.currentTenantStorage.read()];
                    case 2:
                        currentTenant = (/** @type {?} */ ((_a.sent())));
                        _a.label = 3;
                    case 3:
                        _a.trys.push([3, 5, , 7]);
                        return [4 /*yield*/, this.tenantsStorage.get(currentTenant.shortNameTenant)];
                    case 4:
                        if (_a.sent()) {
                            return [2 /*return*/, currentTenant];
                        }
                        return [3 /*break*/, 7];
                    case 5:
                        error_1 = _a.sent();
                        // console.error(error);
                        this.currentTenantStorage.data = undefined;
                        return [4 /*yield*/, this.currentTenantStorage.updateStorage()];
                    case 6:
                        _a.sent();
                        return [2 /*return*/, undefined];
                    case 7: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * @param {?} currentTenant
     * @return {?}
     */
    MultiTenantService.prototype.setCurrentTenant = /**
     * @param {?} currentTenant
     * @return {?}
     */
    function (currentTenant) {
        return __awaiter(this, void 0, void 0, function () {
            var tenantChanged;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.ready];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, this.currentTenantStorage.read()];
                    case 2:
                        _a.sent();
                        if (!this.currentTenantStorage.data) {
                            this.currentTenantStorage.data = currentTenant;
                        }
                        tenantChanged = currentTenant.shortNameTenant !==
                            this.currentTenantStorage.data.shortNameTenant;
                        this.currentTenantStorage.data = currentTenant;
                        return [4 /*yield*/, this.currentTenantStorage.updateStorage()];
                    case 3:
                        _a.sent();
                        if (tenantChanged) {
                            this.events.publish(MultiTenantEvents.CURRENT_TENANT_CHANGED, currentTenant);
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * @param {?} tenant
     * @return {?}
     */
    MultiTenantService.prototype.addTenant = /**
     * @param {?} tenant
     * @return {?}
     */
    function (tenant) {
        return __awaiter(this, void 0, void 0, function () {
            var tenants;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.ready];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, this.tenantsStorage.read()];
                    case 2:
                        tenants = (/** @type {?} */ ((_a.sent())));
                        // Verificação de duplicidade de tenant
                        if (tenants && tenants.length > 0) {
                            tenants.forEach((/**
                             * @param {?} tenantFromStorage
                             * @param {?} index
                             * @return {?}
                             */
                            function (tenantFromStorage, index) {
                                if (tenantFromStorage.shortNameTenant === tenant.shortNameTenant &&
                                    tenantFromStorage.isDemo === tenant.isDemo) {
                                    tenants.splice(index, 1);
                                }
                            }));
                        }
                        this.tenantsStorage.data = tenants;
                        this.tenantsStorage.data.push(tenant);
                        this.sortTenantsByShortName();
                        return [4 /*yield*/, this.tenantsStorage.updateStorage()];
                    case 3:
                        _a.sent();
                        return [2 /*return*/, tenant];
                }
            });
        });
    };
    /**
     * @param {?} tenant
     * @return {?}
     */
    MultiTenantService.prototype.deleteTenant = /**
     * @param {?} tenant
     * @return {?}
     */
    function (tenant) {
        return __awaiter(this, void 0, void 0, function () {
            var error_2;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.ready];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2:
                        _a.trys.push([2, 5, , 6]);
                        return [4 /*yield*/, this.tenantsStorage.delete(tenant.shortNameTenant)];
                    case 3:
                        _a.sent();
                        return [4 /*yield*/, this.tenantsStorage.read()];
                    case 4:
                        _a.sent();
                        return [3 /*break*/, 6];
                    case 5:
                        error_2 = _a.sent();
                        console.error(error_2);
                        return [3 /*break*/, 6];
                    case 6: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * @private
     * @return {?}
     */
    MultiTenantService.prototype.sortTenantsByShortName = /**
     * @private
     * @return {?}
     */
    function () {
        /** @type {?} */
        var sortBy = (/**
         * @param {?} fn
         * @return {?}
         */
        function (fn) { return (/**
         * @param {?} a
         * @param {?} b
         * @return {?}
         */
        function (a, b) { return -(fn(a) < fn(b)) || +(fn(a) > fn(b)); }); });
        /** @type {?} */
        var getShortName = (/**
         * @param {?} o
         * @return {?}
         */
        function (o) { return o.shortNameTenant; });
        /** @type {?} */
        var sortByShortName = sortBy(getShortName);
        this.tenantsStorage.data.sort(sortByShortName);
    };
    MultiTenantService.SHORT_NAME_TENANT_FIELD = 'tenantId';
    MultiTenantService.IS_DEMONSTRATION = 'isDemo';
    MultiTenantService.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    MultiTenantService.ctorParameters = function () { return [
        { type: Platform },
        { type: Storage },
        { type: Events }
    ]; };
    return MultiTenantService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var MultiTenantModule = /** @class */ (function () {
    function MultiTenantModule() {
    }
    /**
     * @return {?}
     */
    MultiTenantModule.forRoot = /**
     * @return {?}
     */
    function () {
        return {
            ngModule: MultiTenantModule,
            providers: [
                MultiTenantService
            ],
        };
    };
    MultiTenantModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [],
                    imports: [],
                },] }
    ];
    return MultiTenantModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { FndModule, authFactory, AuthModule, AuthService, DateTimeModule$1 as DateTimeModule, GlobalService, InterceptorModule, DefaultHttpInterceptorService, i18nFactory, IntlLiteralsLoaderService, IntlModule, I18NPipe, IntlService, INTL_CONFIG, createTranslateLoader, ShowMessageModule, ShowMessageService, ShowLoadingService, ShowLoadingModule, MultiTenantEvents, MultiTenantService, MultiTenantModule };

//# sourceMappingURL=totvs-log-mobile-foundation.js.map