import { GlobalService } from 'totvs-log-mobile-foundation';
import { ICameraUtil } from './camera-util.interface';
import { CameraOptions, Camera } from '@ionic-native/camera/ngx';
import { Platform, Events } from '@ionic/angular';
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de camera
 * com métodos que facilitam a solicitação de permissão e apresentação da
 * camera
 */
export declare class CameraUtilProvider implements ICameraUtil {
    private camera;
    private global;
    private events;
    /**
     * @ignore
     * Indica se a plataforma é cordova
     */
    private isCordova;
    /**
     * @ignore
     * Opções padrões para a camera
     */
    private defaultOptions;
    /**
     * @ignore
     * Método responsável por construir o serviço
     * platform Serviço relacionado a plataforma
     * camera Serviço do plugin nativo de camera
     * global Serviço global da aplicação
     */
    constructor(platform: Platform, camera: Camera, global: GlobalService, events: Events);
    /**
     * Método responsável por apresentar a camera para que o usuário
     * possa tirar uma foto
     * Este método publica o evento CameraUtilEvents.CACHE_FILE_CREATED quando um arquivo é criado em
     * cache (Mais informações sobre eventos {@link https://ionicframework.com/docs/api/util/Events/|Ionic Events})
     * {CameraOptions} options Opções da camera
     * @returns Promessa com o FILE_URI (opção padrão) ou o que for definido na opção 'destinationType' que pode ser passada como parâmetro
     */
    takePicture(options?: CameraOptions): Promise<string>;
    /**
     * Método responsável por limpar as imagens em cache geradas durante
     * a utilização do plugin enquanto estiver ativo
     */
    cleanup(): void;
}
