import { File, RemoveResult } from '@ionic-native/file/ngx';
import { Platform } from '@ionic/angular';
import { IFileUtil } from './file-util.interface';
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de arquivo
 * com métodos que facilitam a manipulação de arquivos no aplicativo
 */
export declare class FileUtilProvider implements IFileUtil {
    private file;
    /**
     * @ignore
     * Indica se a plataforma é cordova
     */
    private isCordova;
    /**
     * @ignore
     * Método construtor responsável por inicializar o serviço
     * platform Serviço relacionado a plataforma
     * file Serviço do plugin nativo de arquivo
     * events Serviço de eventos do Ionic
     */
    constructor(platform: Platform, file: File);
    /**
     * Método responsável por remover um arquivo
     * uri URI do arquivo que deve ser removido
     * Objeto que representa o resultado da remoção do arquivo
     */
    removeFile(uri: string): Promise<RemoveResult>;
    /**
     * Método responsável por transferir os arquivos para o storage do aplicativo
     * uris URIs dos arquivos que devem ser copiados
     * Lista de URIs dos arquivos copiados para o storage do aplicativo
     */
    fileCacheToAppStorage(...uris: string[]): Promise<string[]>;
    /**
     * @ignore
     * Método responsável por recuperar o nome do arquivo de um URI
     * uri URI do arquivo
     */
    private getFilenameFromUri;
    /**
     * @ignore
     * Método responsável por recuperar o path do arquivo de um URI
     * uri URI do arquivo
     */
    private getPathFromUri;
}
