import { QRScannerStatus } from '@ionic-native/qr-scanner/ngx';
import { Subject } from 'rxjs';
/**
 * Interface que define as propriedades e comportamentos do
 * utilitário de leitura de QR-Code
 */
export interface IQrScannerUtil {
    /**
     * Propriedade que possui informações sobre o QR Code Scanner após ele estar
     * preparado, indicando assim se é possível ligar o flash ou trocar de camera,
     * por exemplo.
     */
    qrScannerLastStatus: Subject<QRScannerStatus>;
    /**
     * Método responsável por preparar o plugin para leitura do QR-Code
     * Indicação da preparação
     */
    prepare(): Promise<boolean>;
    /**
     * Método responsável por apresentar o leitor do QR-Code
     * String com o conteúdo lido
     */
    show(): Promise<string>;
    /**
     * Método responsável por esconder a camera da leitura do QR-Code
     */
    hide(): void;
    /**
     * Método responsável por ligar ou desligar o flash
     */
    flash(): void;
    /**
     * Método responsável por alternar entre a camera frontal e a camera traseira
     */
    changeCamera(): void;
    /**
     * Método responsável por destruir o plugin nativo desabilitando a leitura do QR-Code até
     * que o plugin seja novamente preparado
     */
    destroy(): void;
}
