import { SQLiteObject, DbTransaction } from '@ionic-native/sqlite/ngx';
import { ISyncItemDto } from 'totvs-log-base-foundation';
import { IChildren } from './children.interface';
/**
 * Utilitário com funções para sincronização de dados em banco sqlite
 */
export declare class SyncSqLiteUtil {
    private cn;
    /**
     * Creates an instance of SyncDbUtils.
     * cn a conexão que será utilizada para salvar os dados
     */
    constructor(cn: SQLiteObject);
    /**
     * Cria um relacionamento 1-n, inserindo na entidade filha o id do pai
     *
     * @template T
     * items os registros pai
     * fatherTable o nome da tabela pi
     * @returns Os registros relacionados
     */
    updateChildrenRelation<T extends ISyncItemDto>(items: T[], fatherTable: string): Array<IChildren<T>>;
    /**
     * Armazena os dados enviados, substituindo o registro anterior e criando tabelas para
     * entidades filhas(arrays)
     * data A requisição de sync com os dados
     * mainTable O nome da tabela principal
     * deleteBefore se deve sempre deletar antes de inserir o registro
     * currentTx a transação corrente, não deve ser passado sem entender o mecanismo interno
     */
    storeData<T extends ISyncItemDto>(data: {
        items: T[];
    }, mainTable: string, deleteBefore?: boolean, currentTx?: DbTransaction): Promise<void>;
    /**
     * Retorna se a tabela existe
     * table o nome da tabela que se deseja verificar se existe
     */
    tableExists(table: string): Promise<boolean>;
    /**
     * Cria a tabela desejada, se ela não existir
     * table o nome da tabela
     * fields os campos pertencentes a tabela
     */
    createTable(table: string, ...fields: string[]): Promise<void>;
    /**
     * Verifica se os campos já existem na tabela e os adiciona
     * table O nome da tabela
     * fields os campos que serão adicionados, se ainda não existirem na tabela
     */
    checkAndAddFields(table: string, ...fields: string[]): Promise<void>;
    /**
     * Retorna um array com os nomes dos campos da entidade, removendo os que são do tipo array
     * obj o objeto que se deseja avaliar
     */
    private modelWithoutArrays;
    /**
     * Executa o sql necessário para realizar o upsert de um registro, dentro de uma transação
     * tx a transação corrente
     * table o nome da tabela
     * model os campos envolvidos
     * item o item que se deseja inserir
     * deleteBefore se deve sempre excluir o registro antes de inserir
     */
    private upsert;
    /**
     * Na consulta de campos, verifica se o campo está presente na tabela
     * rs o recordset com os registros
     * field o nome do campo desejado
     */
    private checkField;
    /**
     * Cria ou atualização a estrutura da tabela dentro de uma transação
     * tx a transação corrente
     * mainTable o nome da tabela
     * model os campos da tabela
     * afterTx o que fazer após a transação ocorrer com sucesso
     */
    private checkTableStateTx;
}
