import { Observable } from 'rxjs';
import { ISyncEntityState } from './sync-entity-state.interface';
import { ISyncResultBase } from './sync-result.interface';
/**
 * Classe abstrata que todo recurso de sincronização deve implementar,
 * independente do sentido
 * Para registrar o serviço em que a sincronização automática reconheça:
 * export const entidadeToken = new InjectionToken<entidadeProvider>('');
 * export class entidadeProvider extends SyncHandler{}
 * {
 * provide: entidadeToken,
 * useClass: entidadeProvider
 * }
 */
export declare abstract class SyncHandler {
    /**
     * Deve ser implementado para que o gerenciador de sincronização realize a chamada
     * do método. Quando for sincronizar
     * currentVersion
     * @returns Deve retornar o resultado da sincronização
     */
    abstract getSyncData(currentState: ISyncEntityState): Observable<ISyncResultBase>;
}
